/**
 * Cookie Away - Background Service Worker
 * 
 * This script manages the core functionality of the extension:
 * - Initializes default preferences if not set
 * - Manages communication between components
 * - Handles events like installation and updates
 * - Tracks successes and failures for user feedback
 * - Collects data about unhandled cookie banners
 * - Checks for extension updates
 * 
 * Version: 3.1.0
 */

// Default cookie preferences when the extension is first installed
const DEFAULT_PREFERENCES = {
  necessary: true,  // Always enabled
  analytics: false,
  marketing: false,
  personalization: false,
  sendGPC: true     // Send Global Privacy Control signal
};

// Initialize site exceptions list
const DEFAULT_SITE_EXCEPTIONS = {};

// Initialize settings
const DEFAULT_SETTINGS = {
  autoApplyConsent: true,
  notifyOnAction: true,
  collectStats: true,         // Track statistics for user feedback
  reportUnhandledBanners: true // Send data about unhandled banners
};

// Activity stats structure
const DEFAULT_STATS = {
  totalAttempts: 0,
  successCount: 0,
  failureCount: 0,
  byPlatform: {},          // Success/failure counts by platform
  recentActivity: [],      // Recent activity log
  unhandledBanners: []     // Info about unhandled cookie banners
};

// Maximum number of entries to keep
const MAX_RECENT_ACTIVITY = 20; // Limit site activity to 20 sites
const MAX_UNHANDLED_BANNERS = 50;

// Update server configuration
// For production use:
const UPDATE_SERVER = 'https://buy.cookieaway.com';
// For development testing:
const DEV_SERVER = 'http://localhost:5000'; // Development server for testing
const CHECK_UPDATE_INTERVAL = 24 * 60 * 60 * 1000; // Check once a day

// Performance statistics configuration
const STATS_SUBMIT_INTERVAL = 7 * 24 * 60 * 60 * 1000; // Submit stats once a week
const STATS_SERVER = UPDATE_SERVER; // Use production server for statistics
const MAX_RETRY_ATTEMPTS = 5; // Maximum number of retry attempts for stats submission
const RETRY_DELAY_BASE = 60 * 1000; // Base delay between retries (1 minute)
const MAX_BATCH_SIZE = 50; // Maximum number of pending stats entries to send in one batch

// Handle first installation or update
chrome.runtime.onInstalled.addListener((details) => {
  if (details.reason === 'install') {
    console.log('Cookie Response Manager installed');
    
    // Initialize default preferences
    chrome.storage.local.set({
      preferences: DEFAULT_PREFERENCES,
      siteExceptions: DEFAULT_SITE_EXCEPTIONS,
      settings: DEFAULT_SETTINGS,
      stats: DEFAULT_STATS
    });
    
    // Open the options page for first-time setup
    chrome.runtime.openOptionsPage();
  } else if (details.reason === 'update') {
    console.log('Cookie Response Manager updated to version ' + chrome.runtime.getManifest().version);
    
    // Check for stats existence and initialize if missing
    chrome.storage.local.get('stats', (data) => {
      if (!data.stats) {
        chrome.storage.local.set({ stats: DEFAULT_STATS });
      }
    });
    
    // Check for updated settings and add new ones if missing
    chrome.storage.local.get('settings', (data) => {
      if (data.settings) {
        const updatedSettings = { ...DEFAULT_SETTINGS, ...data.settings };
        chrome.storage.local.set({ settings: updatedSettings });
      }
    });
  }
  // Schedule version check
  scheduleVersionCheck();
});

// Function to check for updates
async function checkForUpdates() {
  try {
    const currentVersion = chrome.runtime.getManifest().version;
    // Use production server in the real extension
    const response = await fetch(`${UPDATE_SERVER}/api/version?version=${currentVersion}`);
    
    if (response.ok) {
      const data = await response.json();
      
      // If an update is available, store the update information
      if (data.updateAvailable) {
        chrome.storage.local.set({ 
          updateInfo: {
            available: true,
            currentVersion: currentVersion,
            newVersion: data.currentVersion,
            updateUrl: data.updateUrl,
            releaseNotes: data.releaseNotes,
            lastChecked: new Date().toISOString()
          }
        });
        
        // Create a notification about the update
        chrome.action.setBadgeText({ text: 'NEW' });
        chrome.action.setBadgeBackgroundColor({ color: '#FF0000' });
      } else {
        // No update available, clear any previous update info
        chrome.storage.local.set({ 
          updateInfo: {
            available: false,
            lastChecked: new Date().toISOString()
          }
        });
        // Clear badge if no update is available
        chrome.action.setBadgeText({ text: '' });
      }
    }
  } catch (error) {
    console.error('Error checking for updates:', error);
  }
}

// Schedule regular update checks
function scheduleVersionCheck() {
  // Check for updates immediately
  checkForUpdates();
  
  // Then check periodically
  setInterval(checkForUpdates, CHECK_UPDATE_INTERVAL);
  
  // Also schedule stats submission if enabled in settings
  scheduleStatsSubmission();
}

// Generate a unique anonymous client ID for stats collection
async function getOrCreateClientId() {
  try {
    const data = await chrome.storage.local.get('clientId');
    if (data.clientId) {
      return data.clientId;
    }
    
    // Create a new client ID if none exists
    const newClientId = crypto.randomUUID ? crypto.randomUUID() : 
      Date.now().toString(36) + Math.random().toString(36).substring(2);
    
    await chrome.storage.local.set({ clientId: newClientId });
    return newClientId;
  } catch (error) {
    console.error('Error generating client ID:', error);
    return null;
  }
}

// Add a new stats entry to the pending queue for batch processing
async function queueStatsForSubmission() {
  try {
    const data = await chrome.storage.local.get(['settings', 'stats', 'clientId', 'pendingStats']);
    const settings = data.settings || DEFAULT_SETTINGS;
    
    // Only submit if stats collection is enabled in settings
    if (!settings.collectStats) {
      return;
    }
    
    // Get the client ID (or create one if it doesn't exist)
    const clientId = data.clientId || await getOrCreateClientId();
    if (!clientId) {
      return; // Skip submission if we can't get a client ID
    }
    
    const stats = data.stats || DEFAULT_STATS;
    
    // Prepare the stats payload - only include aggregated data, no URLs
    const payload = {
      clientId: clientId,
      totalAttempts: stats.totalAttempts,
      successCount: stats.successCount,
      failureCount: stats.failureCount,
      byPlatform: stats.byPlatform,
      timestamp: new Date().toISOString()
    };
    
    // Add to pending stats queue for batch processing
    const pendingStats = data.pendingStats || [];
    pendingStats.push(payload);
    
    // Store pending stats
    await chrome.storage.local.set({ 
      pendingStats,
      lastStatsQueued: new Date().toISOString() 
    });
    
    // Trigger the batch submission process
    processPendingStats();
    
    console.log('Performance stats queued for submission');
  } catch (error) {
    console.error('Error queueing performance stats:', error);
  }
}

// Process a batch of pending statistics and submit them to the server
async function processPendingStats(retryCount = 0) {
  try {
    // Get pending stats from storage
    const data = await chrome.storage.local.get(['pendingStats', 'settings', 'processingStats']);
    const pendingStats = data.pendingStats || [];
    const settings = data.settings || DEFAULT_SETTINGS;
    const isProcessing = data.processingStats === true;
    
    // Prevent multiple simultaneous processing
    if (isProcessing && retryCount === 0) {
      console.log('Stats processing already in progress, skipping');
      return;
    }
    
    // Mark as processing
    if (retryCount === 0) {
      await chrome.storage.local.set({ processingStats: true });
    }
    
    // Check if there are any pending stats to submit
    if (pendingStats.length === 0) {
      console.log('No pending stats to submit');
      await chrome.storage.local.set({ processingStats: false });
      return;
    }
    
    // Check if stats collection is still enabled
    if (!settings.collectStats) {
      console.log('Stats collection disabled, clearing pending stats');
      await chrome.storage.local.set({ 
        pendingStats: [],
        processingStats: false 
      });
      return;
    }
    
    // Take a batch of pending stats (limited by MAX_BATCH_SIZE)
    const MAX_BATCH_SIZE = 50; // Maximum entries per batch
    const batchSize = Math.min(pendingStats.length, MAX_BATCH_SIZE);
    const batch = pendingStats.slice(0, batchSize);
    
    console.log(`Submitting batch of ${batch.length} stats entries (${pendingStats.length} total pending)`);
    
    // Prepare the batch payload
    const batchPayload = {
      batched: true,
      entries: batch
    };
    
    // Add timeout to fetch request
    const controller = new AbortController();
    const timeoutId = setTimeout(() => controller.abort(), 30000); // 30 second timeout
    
    try {
      // Submit to the server
      const response = await fetch(`${STATS_SERVER || 'https://buy.cookieaway.com'}/api/stats/batch`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: JSON.stringify(batch), // Send array directly, not wrapped in an object
        signal: controller.signal
      });
      
      // Clear the timeout
      clearTimeout(timeoutId);
      
      if (response.ok) {
        const result = await response.json();
        console.log('Batch stats submission result:', result);
        
        let processedCount = batchSize; // Default to all processed
        
        // If the server provides detailed results, use the actual processed count
        if (result && result.stats && typeof result.stats.processed === 'number') {
          processedCount = result.stats.processed;
        }
        
        // Remove successfully submitted stats from pending queue
        const remainingStats = pendingStats.slice(processedCount);
        await chrome.storage.local.set({ 
          pendingStats: remainingStats,
          lastStatsSubmit: new Date().toISOString(),
          processingStats: remainingStats.length > 0 // Only keep processing if more remain
        });
        
        // If there are more pending stats, process the next batch
        if (remainingStats.length > 0) {
          // Reset retry count since last batch was successful
          setTimeout(() => processPendingStats(0), 1000); // Small delay between batches
        }
      } else {
        // Handle submission error with retry logic
        const errorText = await response.text();
        console.error('Error submitting stats batch:', errorText);
        
        await handleBatchSubmissionError(pendingStats, batch, retryCount, 
          `Server error: ${response.status} ${response.statusText}`);
      }
    } catch (fetchError) {
      clearTimeout(timeoutId);
      console.error('Fetch error during stats submission:', fetchError);
      
      // Check if it was a timeout
      const errorMessage = fetchError.name === 'AbortError' 
        ? 'Request timed out' 
        : fetchError.message;
        
      await handleBatchSubmissionError(pendingStats, batch, retryCount, errorMessage);
    }
  } catch (error) {
    console.error('Unexpected error during stats processing:', error);
    
    // Mark as not processing in case of critical error
    await chrome.storage.local.set({ processingStats: false });
    
    // Retry on unexpected errors
    const MAX_RETRY_ATTEMPTS = 5;
    if (retryCount < MAX_RETRY_ATTEMPTS) {
      const RETRY_DELAY_BASE = 30000; // 30 seconds
      const delay = RETRY_DELAY_BASE * Math.pow(2, retryCount);
      console.log(`Retrying batch submission in ${delay/1000} seconds due to unexpected error (attempt ${retryCount + 1}/${MAX_RETRY_ATTEMPTS})`);
      
      setTimeout(() => {
        processPendingStats(retryCount + 1);
      }, delay);
    }
  }
}

// Helper function to handle errors during batch submission
async function handleBatchSubmissionError(pendingStats, batch, retryCount, errorMessage) {
  const MAX_RETRY_ATTEMPTS = 5;
  const RETRY_DELAY_BASE = 30000; // 30 seconds
  
  if (retryCount < MAX_RETRY_ATTEMPTS) {
    // Exponential backoff for retries
    const delay = RETRY_DELAY_BASE * Math.pow(2, retryCount);
    console.log(`Retrying batch submission in ${delay/1000} seconds (attempt ${retryCount + 1}/${MAX_RETRY_ATTEMPTS})`);
    
    // Keep marked as processing during retry
    setTimeout(() => {
      processPendingStats(retryCount + 1);
    }, delay);
  } else {
    console.error(`Giving up on batch submission after ${MAX_RETRY_ATTEMPTS} attempts`);
    
    // Try fallback to single entry submission
    await tryFallbackToSingleSubmission(pendingStats[0]);
    
    // Remove the first entry that we attempted to submit individually
    const remainingStats = pendingStats.slice(1);
    
    // Store the failed batch info for diagnostics
    const data = await chrome.storage.local.get(['failedStatsBatches']);
    const failedBatches = data.failedStatsBatches || [];
    
    failedBatches.push({
      timestamp: new Date().toISOString(),
      batchSize: batch.length,
      error: errorMessage || 'Max retry attempts exceeded'
    });
    
    // Limit the number of failed batches we store
    const maxFailedBatches = 10;
    if (failedBatches.length > maxFailedBatches) {
      failedBatches.splice(0, failedBatches.length - maxFailedBatches);
    }
    
    // Update storage and mark as not processing
    await chrome.storage.local.set({ 
      failedStatsBatches: failedBatches,
      pendingStats: remainingStats,
      processingStats: false
    });
    
    // If we still have more entries, restart processing after a delay
    if (remainingStats.length > 0) {
      setTimeout(() => processPendingStats(0), 60000); // 1 minute delay before retrying
    }
  }
}

// Fallback to submitting a single stats entry when batch fails
async function tryFallbackToSingleSubmission(statsEntry) {
  if (!statsEntry) return false;
  
  try {
    console.log('Attempting fallback to single entry submission');
    
    const controller = new AbortController();
    const timeoutId = setTimeout(() => controller.abort(), 30000); // 30 second timeout
    
    const response = await fetch(`${STATS_SERVER || 'https://buy.cookieaway.com'}/api/stats/submit`, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json'
      },
      body: JSON.stringify(statsEntry),
      signal: controller.signal
    });
    
    clearTimeout(timeoutId);
    
    if (response.ok) {
      console.log('Single entry fallback submission successful');
      return true;
    } else {
      console.error('Single entry fallback submission failed:', response.status, response.statusText);
      return false;
    }
  } catch (error) {
    console.error('Error in fallback submission:', error);
    return false;
  }
}

// Submit anonymous performance statistics to the server (legacy function for backward compatibility)
async function submitPerformanceStats() {
  try {
    // Queue stats for batch submission instead of immediate submission
    await queueStatsForSubmission();
  } catch (error) {
    console.error('Error in legacy submitPerformanceStats:', error);
  }
}

// Schedule regular submission of performance statistics
function scheduleStatsSubmission() {
  // Check when stats were last submitted
  chrome.storage.local.get(['settings', 'lastStatsSubmit'], (data) => {
    const settings = data.settings || DEFAULT_SETTINGS;
    
    // Only proceed if stats collection is enabled
    if (!settings.collectStats) {
      return;
    }
    
    const lastSubmit = data.lastStatsSubmit ? new Date(data.lastStatsSubmit) : null;
    const now = new Date();
    
    // Submit immediately if never submitted or if it's been more than a week
    if (!lastSubmit || (now - lastSubmit) > STATS_SUBMIT_INTERVAL) {
      submitPerformanceStats();
    }
    
    // Schedule periodic submission
    setInterval(submitPerformanceStats, STATS_SUBMIT_INTERVAL);
  });
}

// Listen for messages from content scripts or popup
chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
  if (message.action === 'getPreferences') {
    // Return current preferences
    chrome.storage.local.get(['preferences', 'siteExceptions', 'settings'], (data) => {
      sendResponse({
        preferences: data.preferences || DEFAULT_PREFERENCES,
        siteExceptions: data.siteExceptions || DEFAULT_SITE_EXCEPTIONS,
        settings: data.settings || DEFAULT_SETTINGS
      });
    });
    return true; // Required for async response
  }
  
  else if (message.action === 'setPreferences') {
    // Update preferences
    chrome.storage.local.set({ preferences: message.preferences }, () => {
      sendResponse({ success: true });
    });
    return true;
  }
  
  else if (message.action === 'addSiteException') {
    // Add site to exceptions list
    chrome.storage.local.get('siteExceptions', (data) => {
      const siteExceptions = data.siteExceptions || {};
      siteExceptions[message.domain] = message.preferences;
      
      chrome.storage.local.set({ siteExceptions }, () => {
        sendResponse({ success: true });
      });
    });
    return true;
  }
  
  else if (message.action === 'removeSiteException') {
    // Remove site from exceptions list
    chrome.storage.local.get('siteExceptions', (data) => {
      const siteExceptions = data.siteExceptions || {};
      delete siteExceptions[message.domain];
      
      chrome.storage.local.set({ siteExceptions }, () => {
        sendResponse({ success: true });
      });
    });
    return true;
  }
  
  else if (message.action === 'updateSettings') {
    // Update general settings
    chrome.storage.local.set({ settings: message.settings }, () => {
      sendResponse({ success: true });
    });
    return true;
  }

  else if (message.action === 'consentHandled') {
    // Notification that a consent banner was handled (success or failure)
    chrome.storage.local.get(['settings', 'stats', 'statsSubmissionCounter'], (data) => {
      const settings = data.settings || DEFAULT_SETTINGS;
      const stats = data.stats || DEFAULT_STATS;
      const statsSubmissionCounter = data.statsSubmissionCounter || 0;
      
      // Update stats if enabled
      if (settings.collectStats) {
        // Increment total attempts
        stats.totalAttempts++;
        
        // Track success or failure
        if (message.success) {
          stats.successCount++;
        } else {
          stats.failureCount++;
        }
        
        // Track by platform
        if (!stats.byPlatform[message.cmpType]) {
          stats.byPlatform[message.cmpType] = { success: 0, failure: 0 };
        }
        
        if (message.success) {
          stats.byPlatform[message.cmpType].success++;
        } else {
          stats.byPlatform[message.cmpType].failure++;
        }
        
        // Add to recent activity
        const activity = {
          timestamp: new Date().toISOString(),
          url: new URL(message.url).hostname,
          cmpType: message.cmpType,
          success: message.success
        };
        
        stats.recentActivity.unshift(activity);
        
        // Limit size of recent activity
        if (stats.recentActivity.length > MAX_RECENT_ACTIVITY) {
          stats.recentActivity = stats.recentActivity.slice(0, MAX_RECENT_ACTIVITY);
        }
        
        // Increment submission counter
        const newCounter = statsSubmissionCounter + 1;
        
        // Save updated stats
        chrome.storage.local.set({ 
          stats,
          statsSubmissionCounter: newCounter 
        });
        
        // Queue stats for submission after every 5 consent handlings or a specified threshold
        const SUBMISSION_THRESHOLD = 5;
        if (newCounter >= SUBMISSION_THRESHOLD) {
          console.log(`Reached submission threshold (${newCounter}), queueing stats for submission`);
          queueStatsForSubmission();
          // Reset counter after queueing
          chrome.storage.local.set({ statsSubmissionCounter: 0 });
        }
      }
      
      // Show notification if enabled
      if (settings.notifyOnAction) {
        // Create a notification (future improvement could use chrome.notifications API)
        console.log(`${message.success ? 'Successfully handled' : 'Failed to handle'} ${message.cmpType} banner on ${message.url}`);
      }
    });
    return false;
  }
  
  else if (message.action === 'reportUnhandledBanner') {
    // Report an undetected cookie banner
    chrome.storage.local.get(['settings', 'stats'], (data) => {
      const settings = data.settings || DEFAULT_SETTINGS;
      const stats = data.stats || DEFAULT_STATS;
      
      if (settings.reportUnhandledBanners) {
        // Add to unhandled banners list
        const bannerInfo = {
          timestamp: new Date().toISOString(),
          url: message.url,
          htmlSnapshot: message.htmlSnapshot,
          userSubmitted: true
        };
        
        stats.unhandledBanners.unshift(bannerInfo);
        
        // Limit size of unhandled banners list
        if (stats.unhandledBanners.length > MAX_UNHANDLED_BANNERS) {
          stats.unhandledBanners = stats.unhandledBanners.slice(0, MAX_UNHANDLED_BANNERS);
        }
        
        // Save updated stats
        chrome.storage.local.set({ stats }, () => {
          sendResponse({ success: true });
        });
      } else {
        sendResponse({ success: false, reason: 'Reporting is disabled' });
      }
    });
    return true;
  }
  
  else if (message.action === 'getStats') {
    // Return current stats
    chrome.storage.local.get('stats', (data) => {
      sendResponse({
        stats: data.stats || DEFAULT_STATS
      });
    });
    return true;
  }
  
  else if (message.action === 'clearStats') {
    // Reset stats
    chrome.storage.local.set({ stats: DEFAULT_STATS }, () => {
      sendResponse({ success: true });
    });
    return true;
  }
  
  else if (message.action === 'clearSiteActivity') {
    // Clear only site activity while keeping overall statistics
    chrome.storage.local.get('stats', (data) => {
      const stats = data.stats || DEFAULT_STATS;
      
      // Keep the totals but clear the activity log
      stats.recentActivity = [];
      
      chrome.storage.local.set({ stats }, () => {
        sendResponse({ success: true });
      });
    });
    return true;
  }
  
  else if (message.action === 'checkUpdates') {
    // Manually check for updates
    checkForUpdates().then(() => {
      // Get the update info to return to caller
      chrome.storage.local.get('updateInfo', (data) => {
        sendResponse({ success: true, updateInfo: data.updateInfo || {} });
      });
    }).catch(error => {
      console.error('Error checking for updates:', error);
      sendResponse({ success: false, error: error.message });
    });
    return true;
  }
  
  else if (message.action === 'performUpdate') {
    // Handle direct update request
    handleUpdateRequest(message.version, sendResponse);
    return true;
  }
  
  else if (message.action === 'enableGPC') {
    // Enable Global Privacy Control for this session
    console.log('Cookie Away: Enabling Global Privacy Control (GPC) header');
    
    // In Manifest V3, we can't use webRequestBlocking or modify headers directly
    // Instead, we'll use a declarativeNetRequest rule or implement GPC in content scripts
    
    // For Manifest V3, we'll set a flag in storage that content scripts can check
    chrome.storage.local.set({ gpcEnabled: true }, function() {
      console.log('Cookie Away: GPC enabled flag set in storage');
    });
    
    // Note: The actual GPC implementation for Manifest V3 will be handled in content scripts
    // by injecting the appropriate meta tag and navigator property
    
    sendResponse({ success: true });
    return true;
  }
});

// Handle the extension update request
function handleUpdateRequest(version, sendResponse) {
  try {
    console.log('Handling update request to version:', version);
    
    // Clear any update badge notification
    chrome.action.setBadgeText({ text: '' });
    
    // Update the update info to reflect that the user is updating
    chrome.storage.local.get('updateInfo', (data) => {
      const updateInfo = data.updateInfo || {};
      
      updateInfo.updateInProgress = true;
      updateInfo.targetVersion = version;
      
      chrome.storage.local.set({ updateInfo }, () => {
        sendResponse({ 
          success: true, 
          message: 'Update in progress',
          version: version 
        });
      });
    });
  } catch (error) {
    console.error('Error handling update request:', error);
    sendResponse({ 
      success: false, 
      error: error.message 
    });
  }
}
