/**
 * Cookie Away - Consent Manager Platform Detector
 * 
 * This script:
 * 1. Detects the presence of a cookie consent banner
 * 2. Identifies which CMP is being used
 * 3. Calls the appropriate handler to apply the user's preferences
 * 
 * Version: 3.1.0
 * - Now supports 25+ cookie consent platforms
 * - Implements safety measures to prevent unwanted tab openings
 * - Limits detection attempts to 2 per page to prevent infinite loops
 * - Added generic handler for unrecognized banners
 * - Collects data on unhandled banners for improvement
 * - Added specific handler for European Union websites
 * - Enhanced detection for simple cookie banners with comprehensive reject button patterns
 * - Improved protection against redirection loops
 */

// Global flag to track if a CMP has been detected
window.cookieResponseManagerDetected = false;

// Enable more detailed debugging
const DEBUG_MODE = true;

function debugLog(...args) {
  if (DEBUG_MODE) {
    console.log('Cookie Away [DEBUG]:', ...args);
  }
}

// Main detection and handling function
async function detectAndHandleCookieBanners() {
  // Add initial debug message
  debugLog('Detector started on URL:', window.location.href);
  
  // Wait for the DOM to be fully loaded
  if (document.readyState !== 'complete') {
    debugLog('DOM not ready, waiting for load event');
    window.addEventListener('load', detectAndHandleCookieBanners);
    return;
  }
  
  // Check if we've already attempted detection twice
  if (!canAttemptDetection()) {
    console.log(`Cookie Away: Skipping detection, attempt ${window.cookieResponseManagerAttempts} of max 2`);
    return;
  }
  
  // Check if we're in an iframe or not the main window
  if (window !== window.top) {
    console.log('Cookie Away: Running in iframe, skipping detection for safety');
    return;
  }
  
  // Get current hostname to check if it's a problematic domain
  const hostname = window.location.hostname;
  
  // Skip detection for problematic domains known to cause issues
  const problematicDomains = ['dbschenker.com', 'dbschenker.'];
  if (problematicDomains.some(domain => hostname.includes(domain))) {
    console.log(`Cookie Away: Domain ${hostname} is in blocklist, skipping detection`);
    return;
  }
  
  // Get user preferences before proceeding
  const data = await getUserPreferences(hostname);
  
  // If auto-apply is disabled, don't proceed
  if (!data.settings.autoApplyConsent) {
    console.log('Cookie Away: Auto-apply is disabled');
    return;
  }
  
  // If this domain is in the exceptions list and set to "no action", don't proceed
  if (data.isException && data.preferences.noAction) {
    console.log('Cookie Away: Domain is in exceptions list with "no action"');
    return;
  }
  
  // Flag to avoid multiple concurrent detections
  if (window.cookieResponseManagerInProgress) {
    console.log('Cookie Away: Detection already in progress');
    return;
  }
  
  window.cookieResponseManagerInProgress = true;
  
  // ------------ DETECTION STRATEGIES -------------
  // Try all specific handlers first

  let detected = false;
  
  // Google (high priority handler for Google properties)
  if (!detected && typeof detectGoogleCookieBanner === 'function' && detectGoogleCookieBanner()) {
    console.log('Cookie Away: Google cookie banner detected');
    window.cookieResponseManagerDetected = true;
    handleGoogleCookieBanner(data.preferences);
    detected = true;
  }
  
  // Facebook/Meta (high priority handler for Meta properties)
  if (!detected && typeof detectFacebookCookieBanner === 'function' && detectFacebookCookieBanner()) {
    console.log('Cookie Away: Facebook/Meta cookie banner detected');
    window.cookieResponseManagerDetected = true;
    handleFacebookCookieBanner(data.preferences);
    detected = true;
  }
  
  // OneTrust
  if (!detected && typeof detectOneTrust === 'function' && detectOneTrust()) {
    console.log('Cookie Away: OneTrust detected');
    window.cookieResponseManagerDetected = true;
    handleOneTrust(data.preferences);
    detected = true;
  }
  
  // Cookiebot
  if (!detected && typeof detectCookiebot === 'function' && detectCookiebot()) {
    console.log('Cookie Away: Cookiebot detected');
    window.cookieResponseManagerDetected = true;
    handleCookiebot(data.preferences);
    detected = true;
  }
  
  // Iubenda
  if (!detected && typeof detectIubenda === 'function' && detectIubenda()) {
    console.log('Cookie Away: Iubenda detected');
    window.cookieResponseManagerDetected = true;
    handleIubenda(data.preferences);
    detected = true;
  }
  
  // Usercentrics
  if (!detected && typeof detectUsercentrics === 'function' && detectUsercentrics()) {
    console.log('Cookie Away: Usercentrics detected');
    window.cookieResponseManagerDetected = true;
    handleUsercentrics(data.preferences);
    detected = true;
  }
  
  // TrustArc
  if (!detected && typeof detectTrustArc === 'function' && detectTrustArc()) {
    console.log('Cookie Away: TrustArc detected');
    window.cookieResponseManagerDetected = true;
    handleTrustArc(data.preferences);
    detected = true;
  }
  
  // Tealium
  if (!detected && typeof detectTealium === 'function' && detectTealium()) {
    console.log('Cookie Away: Tealium detected');
    window.cookieResponseManagerDetected = true;
    handleTealium(data.preferences);
    detected = true;
  }
  
  // Cookie Information
  if (!detected && typeof detectCookieInformation === 'function' && detectCookieInformation()) {
    console.log('Cookie Away: Cookie Information detected');
    window.cookieResponseManagerDetected = true;
    handleCookieInformation(data.preferences);
    detected = true;
  }
  
  // Crownpeak (Evidon)
  if (!detected && typeof detectCrownpeak === 'function' && detectCrownpeak()) {
    console.log('Cookie Away: Crownpeak/Evidon detected');
    window.cookieResponseManagerDetected = true;
    handleCrownpeak(data.preferences);
    detected = true;
  }
  
  // Osano
  if (!detected && typeof detectOsano === 'function' && detectOsano()) {
    console.log('Cookie Away: Osano detected');
    window.cookieResponseManagerDetected = true;
    handleOsano(data.preferences);
    detected = true;
  }
  
  // Termly
  if (!detected && typeof detectTermly === 'function' && detectTermly()) {
    console.log('Cookie Away: Termly detected');
    window.cookieResponseManagerDetected = true;
    handleTermly(data.preferences);
    detected = true;
  }
  
  // Quantcast Choice
  if (!detected && typeof detectQuantcast === 'function' && detectQuantcast()) {
    console.log('Cookie Away: Quantcast detected');
    window.cookieResponseManagerDetected = true;
    handleQuantcast(data.preferences);
    detected = true;
  }
  
  // Secure Privacy
  if (!detected && typeof detectSecurePrivacy === 'function' && detectSecurePrivacy()) {
    console.log('Cookie Away: Secure Privacy detected');
    window.cookieResponseManagerDetected = true;
    handleSecurePrivacy(data.preferences);
    detected = true;
  }
  
  // Didomi
  if (!detected && typeof detectDidomi === 'function' && detectDidomi()) {
    console.log('Cookie Away: Didomi detected');
    window.cookieResponseManagerDetected = true;
    handleDidomi(data.preferences);
    detected = true;
  }
  
  // Ketch
  if (!detected && typeof detectKetch === 'function' && detectKetch()) {
    console.log('Cookie Away: Ketch detected');
    window.cookieResponseManagerDetected = true;
    handleKetch(data.preferences);
    detected = true;
  }
  
  // Piwik PRO CMP
  if (!detected && typeof detectPiwikPro === 'function' && detectPiwikPro()) {
    console.log('Cookie Away: Piwik PRO detected');
    window.cookieResponseManagerDetected = true;
    handlePiwikPro(data.preferences);
    detected = true;
  }
  
  // Sourcepoint
  if (!detected && typeof detectSourcepoint === 'function' && detectSourcepoint()) {
    console.log('Cookie Away: Sourcepoint detected');
    window.cookieResponseManagerDetected = true;
    handleSourcepoint(data.preferences);
    detected = true;
  }
  
  // Finsweet Cookie Consent
  if (!detected && typeof detectFinsweet === 'function' && detectFinsweet()) {
    console.log('Cookie Away: Finsweet detected');
    window.cookieResponseManagerDetected = true;
    handleFinsweet(data.preferences);
    detected = true;
  }
  
  // Klaro!
  if (!detected && typeof detectKlaro === 'function' && detectKlaro()) {
    console.log('Cookie Away: Klaro detected');
    window.cookieResponseManagerDetected = true;
    handleKlaro(data.preferences);
    detected = true;
  }
  
  // Tarteaucitron.js
  if (!detected && typeof detectTarteaucitron === 'function' && detectTarteaucitron()) {
    console.log('Cookie Away: Tarteaucitron detected');
    window.cookieResponseManagerDetected = true;
    handleTarteaucitron(data.preferences);
    detected = true;
  }
  
  // CookieYes
  if (!detected && typeof detectCookieYes === 'function' && detectCookieYes()) {
    console.log('Cookie Away: CookieYes detected');
    window.cookieResponseManagerDetected = true;
    handleCookieYes(data.preferences);
    detected = true;
  }
  
  // Europa (EU websites)
  if (!detected && typeof detectEuropa === 'function' && detectEuropa()) {
    console.log('Cookie Away: Europa EU cookie banner detected');
    window.cookieResponseManagerDetected = true;
    handleEuropa(data.preferences);
    detected = true;
  }
  
  // Facebook/Meta handler (handles cookie policy redirection issues)
  if (!detected && typeof detectFacebookCookieBanner === 'function' && detectFacebookCookieBanner()) {
    console.log('Cookie Away: Facebook cookie banner or policy page detected');
    window.cookieResponseManagerDetected = true;
    handleFacebookCookieBanner(data.preferences);
    detected = true;
  }

  // DJI handler for their specific cookie banner format
  if (!detected && typeof detectDJICookieBanner === 'function') {
    debugLog('Checking for DJI cookie banner');
    const bannerElement = detectDJICookieBanner();
    if (bannerElement) {
      debugLog('DJI cookie banner detected, applying handler');
      console.log('Cookie Away: DJI cookie banner detected');
      window.cookieResponseManagerDetected = true;
      const success = await handleDJICookieBanner();
      notifyBackgroundScript('DJI', success);
      detected = true;
    }
  }
  
  // Amazon handler for their specific cookie banner format
  if (!detected && typeof detectAmazonCookieBanner === 'function') {
    debugLog('Checking for Amazon cookie banner');
    const bannerElement = detectAmazonCookieBanner();
    if (bannerElement) {
      debugLog('Amazon cookie banner detected, applying handler');
      console.log('Cookie Away: Amazon cookie banner detected');
      window.cookieResponseManagerDetected = true;
      const success = await handleAmazonCookieBanner();
      notifyBackgroundScript('Amazon', success);
      detected = true;
    }
  }
  
  // Try simple banner handler before the generic handler
  if (!detected && typeof detectSimpleCookieBanner === 'function') {
    debugLog('Trying simple banner detection');
    if (detectSimpleCookieBanner()) {
      debugLog('Simple cookie banner detected, applying handler');
      console.log('Cookie Away: Simple cookie banner detected, using simple banner handler');
      window.cookieResponseManagerDetected = true;
      handleSimpleCookieBanner(data.preferences);
      detected = true;
    } else {
      debugLog('No simple cookie banner detected');
    }
  }
  
  // If no specific handler found, try the generic handler as a last resort
  if (!detected && typeof detectGenericCookieBanner === 'function') {
    debugLog('Trying generic cookie banner detection');
    if (detectGenericCookieBanner()) {
      debugLog('Generic cookie banner detected, applying handler');
      console.log('Cookie Away: Unknown cookie banner detected, using generic handler');
      window.cookieResponseManagerDetected = true;
      handleGenericCookieBanner(data.preferences);
      detected = true;
    } else {
      debugLog('No generic cookie banner detected');
    }
  }
  
  if (!detected) {
    debugLog('No cookie banner detected on this page');
    console.log('Cookie Away: No cookie banner detected');
  }
  
  // Clear the in-progress flag
  debugLog('Detection process complete');
  window.cookieResponseManagerInProgress = false;
}

// Notify the background script that a consent banner was handled
function notifyBackgroundScript(cmpType, success) {
  try {
    debugLog(`Notifying background script: CMP type = ${cmpType}, success = ${success}`);
    
    chrome.runtime.sendMessage({
      action: 'consentHandled',
      cmpType: cmpType,
      success: success,
      url: window.location.href
    }, response => {
      if (chrome.runtime.lastError) {
        debugLog('Error sending message:', chrome.runtime.lastError);
      } else {
        debugLog('Background notification sent successfully');
      }
    });
  } catch (error) {
    debugLog(`Failed to notify background script: ${error.message}`);
    console.log(`Cookie Away: Failed to notify background script - ${error.message}`);
  }
}

// Function to report an unhandled banner (can be called from popup)
function reportUnhandledBanner() {
  if (window.cookieResponseManagerCaptured) {
    console.log('Cookie Away: Banner already captured');
    return;
  }
  
  try {
    // Look for cookie-related elements
    const cookieElements = document.querySelectorAll(
      '[class*="cookie"], [id*="cookie"], [class*="consent"], [id*="consent"], ' +
      '[class*="privacy"], [id*="privacy"], [class*="gdpr"], [id*="gdpr"]'
    );
    
    let bestCandidate = null;
    
    // Find the most promising element
    for (const element of cookieElements) {
      if (isElementVisible(element)) {
        if (!bestCandidate || element.textContent.length > bestCandidate.textContent.length) {
          bestCandidate = element;
        }
      }
    }
    
    if (bestCandidate) {
      // Create a snapshot of the element
      const snapshot = createElementSnapshot(bestCandidate, 0);
      
      // Send to background script
      chrome.runtime.sendMessage({
        action: 'reportUnhandledBanner',
        url: window.location.href,
        htmlSnapshot: snapshot,
        userSubmitted: true
      });
      
      window.cookieResponseManagerCaptured = true;
      return true;
    }
    
    return false;
  } catch (error) {
    console.log(`Cookie Away: Error reporting banner - ${error.message}`);
    return false;
  }
}

// Helper function to create a minimal snapshot of an element's structure
function createElementSnapshot(element, depth) {
  // Prevent too deep recursion
  if (!element || depth > 3) {
    return null;
  }
  
  const snapshot = {
    tagName: element.tagName,
    id: element.id || null,
    className: element.className || null,
    textContent: element.textContent ? element.textContent.substring(0, 200) : null
  };
  
  // Only include a limited set of children
  if (element.children.length > 0 && depth < 2) {
    snapshot.children = [];
    
    // Limit number of children to save space
    const maxChildren = 5;
    const childrenArray = Array.from(element.children);
    
    for (let i = 0; i < Math.min(childrenArray.length, maxChildren); i++) {
      const childSnapshot = createElementSnapshot(childrenArray[i], depth + 1);
      if (childSnapshot) {
        snapshot.children.push(childSnapshot);
      }
    }
  }
  
  return snapshot;
}

// Only run once at the start
detectAndHandleCookieBanners();

// Check again after a short delay to handle banners that load dynamically
// This is the last attempt - limited to a maximum of 2 detection attempts
setTimeout(detectAndHandleCookieBanners, 1500);
