/**
 * Cookie Away - Amazon Handler
 * 
 * This handler is specifically designed to handle cookie banners on Amazon.com,
 * Amazon.de, and other Amazon regional sites which have a unique cookie consent flow.
 */

// Detect Amazon cookie banners
function detectAmazonCookieBanner() {
  try {
    // Check if we're on an Amazon domain
    if (!window.location.hostname.includes('amazon')) {
      return false;
    }
    
    console.log('Cookie Away: Checking for Amazon cookie banner');
    
    // Amazon uses different IDs and classes across regions. We'll check all known patterns.
    // Comprehensive list of Amazon banner selectors
    const bannerSelectors = [
      // Standard Amazon banner elements
      '#sp-cc', 
      '#cookie-consent-banner', 
      '.consent-banner', 
      '#consent-page',
      '#a-popover', // Amazon popover component
      '#consent', // Some regions use this
      '#cookieBanner',
      '.gdpr-consent-banner',
      
      // Region-specific selectors
      '#customer-preferences', // Often used on Amazon.de
      '#cookie-preferences',
      '.cookie-policy-banner',
      
      // Attribute-based selectors
      '[aria-label*="cookie"]',
      '[aria-labelledby*="consent"]',
      '[data-feature-name="euCookiePreference"]'
    ];
    
    // Try all known Amazon selectors
    for (const selector of bannerSelectors) {
      const element = document.querySelector(selector);
      if (element && isElementVisible(element)) {
        console.log(`Cookie Away: Amazon cookie banner detected with selector: ${selector}`);
        return element;
      }
    }
    
    // Check for dialogs with an iframe containing cookie settings
    // (Amazon sometimes embeds cookie settings in iframes)
    const iframeContainers = Array.from(document.querySelectorAll('div[role="dialog"], div[class*="modal"]'))
      .filter(el => {
        const hasIframe = el.querySelector('iframe');
        return isElementVisible(el) && hasIframe;
      });
    
    if (iframeContainers.length > 0) {
      console.log('Cookie Away: Amazon cookie banner detected with iframe');
      return iframeContainers[0];
    }
    
    // Fallback check for more generic cookie banners
    const cookieBanner = document.querySelector('[class*="cookie-banner"], [class*="cookie-consent"], [id*="cookie-banner"], [class*="consent"], [id*="consent"]');
    
    if (cookieBanner && isElementVisible(cookieBanner)) {
      console.log('Cookie Away: Amazon secondary cookie banner detected');
      return cookieBanner;
    }
    
    // Last resort: look for dialog-like elements that mention cookies
    const potentialBanners = Array.from(document.querySelectorAll('div[role="dialog"], div[class*="modal"], div[class*="popup"], div[class*="a-layer"]'))
      .filter(el => {
        if (!isElementVisible(el)) return false;
        
        // Check both the element text and its aria-label
        const text = el.textContent.toLowerCase();
        const ariaLabel = (el.getAttribute('aria-label') || '').toLowerCase();
        
        return text.includes('cookie') || 
               text.includes('consent') || 
               text.includes('privacy') ||
               ariaLabel.includes('cookie') || 
               ariaLabel.includes('consent');
      });
    
    if (potentialBanners.length > 0) {
      console.log('Cookie Away: Amazon alternate cookie banner detected');
      return potentialBanners[0];
    }
    
    // Check if we're on a cookie preferences/settings page (after redirect)
    if (window.location.pathname.includes('cookie') || 
        window.location.pathname.includes('preferences') || 
        window.location.pathname.includes('consent')) {
      
      // On the cookie preferences page, look for form elements
      const prefForm = document.querySelector('form, div[role="form"], div[class*="preference"], div[class*="cookie-setting"]');
      
      if (prefForm && prefForm.querySelectorAll('input[type="checkbox"], button').length > 0) {
        console.log('Cookie Away: Amazon cookie preferences page detected');
        return prefForm;
      }
    }
    
    return false;
  } catch (error) {
    console.log(`Cookie Away: Error detecting Amazon banner - ${error.message}`);
    return false;
  }
}

// Handle Amazon cookie banner
async function handleAmazonCookieBanner() {
  try {
    const bannerElement = detectAmazonCookieBanner();
    if (!bannerElement) return false;
    
    console.log('Cookie Away: Handling Amazon cookie banner');
    
    // Get user preferences
    const hostname = window.location.hostname;
    const { preferences } = await getUserPreferences(hostname);
    
    // Detect which language/region we're on
    const isGermanSite = window.location.hostname.includes('.de');
    const isFrenchSite = window.location.hostname.includes('.fr');
    const isItalianSite = window.location.hostname.includes('.it');
    const isSpanishSite = window.location.hostname.includes('.es');
    
    // Check if we're on a preferences page
    const isPreferencesPage = window.location.pathname.includes('cookie') || 
                              window.location.pathname.includes('preferences') || 
                              window.location.pathname.includes('consent') ||
                              window.location.pathname.includes('customer-preferences');
    
    // Log all potential buttons for debugging
    const allButtons = bannerElement.querySelectorAll('button, a, [role="button"], input[type="submit"]');
    console.log(`Cookie Away: Found ${allButtons.length} buttons in Amazon banner`);
    
    for (let i = 0; i < allButtons.length; i++) {
      const btn = allButtons[i];
      console.log(`Cookie Away: Amazon Button ${i}: "${btn.textContent.trim()}" (${btn.tagName})`);
    }
    
    let buttonToClick = null;
    
    // Handle preferences page differently
    if (isPreferencesPage) {
      console.log('Cookie Away: Detected we are on an Amazon preferences page');
      
      // Check for checkboxes - this is common on preferences pages
      const checkboxes = document.querySelectorAll('input[type="checkbox"]');
      let checkboxToggled = false;
      
      if (checkboxes.length > 0) {
        console.log(`Cookie Away: Found ${checkboxes.length} checkboxes on Amazon preferences page`);
        
        // Process checkboxes only if user wants to reject all
        if (preferences.rejectAll) {
          // Log all checkbox labels
          for (const checkbox of checkboxes) {
            // Skip disabled checkboxes (usually necessary ones)
            if (checkbox.disabled || checkbox.readOnly) continue;
            
            // Find the associated label
            const label = checkbox.closest('label') || 
                          document.querySelector(`label[for="${checkbox.id}"]`) ||
                          checkbox.parentElement;
                          
            const labelText = label ? label.textContent.trim() : 'No label';
            console.log(`Cookie Away: Checkbox - "${labelText}" - Checked: ${checkbox.checked}`);
            
            // Determine if this is a necessary/required cookie checkbox
            const isNecessary = label && (
              label.textContent.toLowerCase().includes('necessary') ||
              label.textContent.toLowerCase().includes('essential') ||
              label.textContent.toLowerCase().includes('required') ||
              label.textContent.toLowerCase().includes('functional') ||
              label.textContent.toLowerCase().includes('notwendig') ||
              label.textContent.toLowerCase().includes('erforderlich') ||
              label.textContent.toLowerCase().includes('nécessaire') ||
              label.textContent.toLowerCase().includes('essenziale')
            );
            
            // Uncheck non-necessary checkboxes if user wants to reject all
            if (!isNecessary && checkbox.checked) {
              console.log(`Cookie Away: Unchecking non-necessary Amazon checkbox - "${labelText}"`);
              checkbox.checked = false;
              
              // Trigger change event for dynamic forms
              const event = new Event('change', { bubbles: true });
              checkbox.dispatchEvent(event);
              checkboxToggled = true;
            }
          }
        }
      }
      
      // Look for save/submit button
      const saveButtonPatterns = [
        'save', 'submit', 'confirm', 'apply', 
        'save settings', 'save preferences', 'continue',
        'save changes', 'fertig', 'speichern', 'bestätigen',
        'enregistrer', 'sauvegarder', 'confirmer',
        'salvare', 'confermare',
        'guardar', 'confirmar'
      ];
      
      const saveButton = findButtonByText(document, saveButtonPatterns);
      
      if (saveButton) {
        console.log('Cookie Away: Found save button on Amazon preferences page');
        buttonToClick = saveButton;
      } 
      // If no save button but we changed checkboxes, look for any button that's not cancel/close
      else if (checkboxToggled) {
        const buttons = document.querySelectorAll('button, input[type="button"], input[type="submit"]');
        
        for (const button of buttons) {
          const buttonText = button.textContent.toLowerCase().trim();
          
          // Skip cancel/close buttons
          if (!buttonText.includes('cancel') && !buttonText.includes('close') && 
              !buttonText.includes('abort') && !buttonText.includes('back') &&
              !buttonText.includes('annuler') && !buttonText.includes('fermer') &&
              !buttonText.includes('abbrechen') && !buttonText.includes('zurück')) {
            
            console.log(`Cookie Away: Using non-cancel button as save - "${buttonText}"`);
            buttonToClick = button;
            break;
          }
        }
      }
    } 
    // Standard cookie banner handling
    else {
      // First look for buttons with specific IDs that Amazon uses
      if (preferences.rejectAll) {
        // Look for the standard reject button IDs first
        buttonToClick = document.querySelector('#sp-cc-rejectall, #sp-cc-customize, .reject-consent, [name="reject"], [id*="reject"]');
        
        if (buttonToClick) {
          console.log('Cookie Away: Found Amazon reject button by ID');
        } else {
          // Build language-specific text patterns for rejecting cookies
          const germanRejectText = ['ablehnen', 'nur notwendige', 'erforderlich', 'nicht akzeptieren'];
          const englishRejectText = ['reject', 'customize', 'necessary cookies only', 'only necessary', 'decline'];
          const frenchRejectText = ['refuser', 'personnaliser', 'uniquement nécessaires', 'refuser tous'];
          const italianRejectText = ['rifiuta', 'personalizza', 'solo necessari', 'rifiuta tutti'];
          const spanishRejectText = ['rechazar', 'personalizar', 'solo necesarias', 'rechazar todas'];
          
          // Combine text patterns based on site language
          let rejectText = englishRejectText;
          if (isGermanSite) rejectText = rejectText.concat(germanRejectText);
          if (isFrenchSite) rejectText = rejectText.concat(frenchRejectText);
          if (isItalianSite) rejectText = rejectText.concat(italianRejectText);
          if (isSpanishSite) rejectText = rejectText.concat(spanishRejectText);
          
          // Enhanced policy link detection patterns
          const policyLinkPatterns = [
            'policy', 'learn more', 'read more', 'more information',
            'datenschutz', 'weitere infos', 'cookie-richtlinie', 'mehr erfahren',
            'politique', 'en savoir plus', 'plus d\'informations',
            'politica', 'saperne di più', 'maggiori informazioni',
            'política', 'más información', 'saber más'
          ];
          
          // Look for buttons with reject text
          for (const button of allButtons) {
            const buttonText = button.textContent.toLowerCase().trim();
            
            // Check if button text contains any of the reject patterns
            if (rejectText.some(text => buttonText.includes(text))) {
              // Skip if it looks like a privacy policy link
              const isPolicyLink = policyLinkPatterns.some(pattern => buttonText.includes(pattern)) ||
                                   button.getAttribute('href')?.includes('policy') ||
                                   button.getAttribute('href')?.includes('privacy');
                                 
              if (!isPolicyLink) {
                buttonToClick = button;
                console.log(`Cookie Away: Found Amazon reject button by text "${buttonText}"`);
                break;
              } else {
                console.log(`Cookie Away: Skipping policy link "${buttonText}"`);
              }
            }
          }
          
          // Still nothing? Look for customize/settings buttons
          if (!buttonToClick) {
            // Multi-language settings button patterns
            const settingsPatterns = [
              'customize', 'settings', 'preferences', 'cookie settings',
              'manage cookies', 'manage preferences', 'einstellungen',
              'configure', 'paramètres', 'ajustes', 'personalizar',
              'preferenze', 'impostazioni'
            ];
            
            const settingsButton = findButtonByText(bannerElement, settingsPatterns);
            
            if (settingsButton) {
              console.log('Cookie Away: Attempting to click settings button to access Amazon preferences');
              const settingsClicked = safeClick(settingsButton);
              
              if (settingsClicked) {
                console.log('Cookie Away: Clicked settings button, waiting for panel to appear');
                return true; // Return true as we've started a process
              }
            }
          }
          
          // Final attempt - Amazon often has "accept" as first button and "reject/customize" as second
          if (!buttonToClick && allButtons.length >= 2) {
            // Skip the first button (usually accept) and use the second button
            const secondButton = allButtons[1];
            const buttonText = secondButton.textContent.toLowerCase().trim();
            
            // Verify it's not a policy link
            const isPolicyLink = policyLinkPatterns.some(pattern => buttonText.includes(pattern)) ||
                                 secondButton.getAttribute('href')?.includes('policy') ||
                                 secondButton.getAttribute('href')?.includes('privacy');
            
            if (!isPolicyLink) {
              buttonToClick = secondButton;
              console.log(`Cookie Away: Using second button "${buttonText}" as fallback for Amazon banner`);
            }
          }
        }
      } else {
        // If user wants to accept cookies
        buttonToClick = document.querySelector('#sp-cc-accept, .accept-consent, [name="accept"], [id*="accept"]');
        
        if (!buttonToClick) {
          // Multi-language accept patterns
          const acceptPatterns = {
            default: ['accept', 'accept all', 'allow all', 'agree'],
            german: ['akzeptieren', 'zustimmen', 'alle akzeptieren'],
            french: ['accepter', 'accepter tout', 'tout accepter'],
            italian: ['accetta', 'accetta tutti', 'consenti tutti'],
            spanish: ['aceptar', 'aceptar todo', 'permitir todo']
          };
          
          // Combine appropriate language patterns
          let acceptText = acceptPatterns.default;
          if (isGermanSite) acceptText = acceptText.concat(acceptPatterns.german);
          if (isFrenchSite) acceptText = acceptText.concat(acceptPatterns.french);
          if (isItalianSite) acceptText = acceptText.concat(acceptPatterns.italian);
          if (isSpanishSite) acceptText = acceptText.concat(acceptPatterns.spanish);
          
          for (const button of allButtons) {
            const buttonText = button.textContent.toLowerCase().trim();
            if (acceptText.some(text => buttonText.includes(text))) {
              buttonToClick = button;
              console.log(`Cookie Away: Found Amazon accept button by text "${buttonText}"`);
              break;
            }
          }
          
          // If still nothing found, use the first button as a fallback (usually accept)
          if (!buttonToClick && allButtons.length > 0) {
            buttonToClick = allButtons[0];
            console.log(`Cookie Away: Using first button as fallback accept - "${buttonToClick.textContent.trim()}"`);
          }
        }
      }
    }
    
    // Click the button if found
    if (buttonToClick) {
      console.log(`Cookie Away: Attempting to click Amazon ${isPreferencesPage ? 'preferences' : 'banner'} button: "${buttonToClick.textContent.trim()}"`);
      const clickResult = safeClick(buttonToClick);
      
      if (clickResult) {
        console.log('Cookie Away: Successfully clicked Amazon button');
        return true;
      } else {
        console.log('Cookie Away: Failed to click Amazon button');
        
        // One last attempt - if button has an onclick handler, try to call it directly
        if (typeof buttonToClick.onclick === 'function') {
          try {
            console.log('Cookie Away: Trying to call onclick handler directly');
            buttonToClick.onclick();
            console.log('Cookie Away: Successfully called onclick handler');
            return true;
          } catch (e) {
            console.log(`Cookie Away: Failed to call onclick handler - ${e.message}`);
          }
        }
        
        // For Amazon iframes, try to find and click inside the iframe
        const iframes = document.querySelectorAll('iframe');
        if (iframes.length > 0) {
          console.log(`Cookie Away: Checking ${iframes.length} iframes for cookie controls`);
          
          // We can't directly access iframe content due to same-origin policy
          // but we can try to find and click elements that might be in the iframe
          for (const iframe of iframes) {
            try {
              // Try to access iframe content (will fail if cross-origin)
              const iframeDoc = iframe.contentDocument || iframe.contentWindow?.document;
              
              if (iframeDoc) {
                console.log('Cookie Away: Successfully accessed iframe document');
                
                // Look for reject/accept buttons in the iframe
                const iframeButtons = iframeDoc.querySelectorAll('button, a, [role="button"]');
                console.log(`Cookie Away: Found ${iframeButtons.length} buttons in iframe`);
                
                for (const button of iframeButtons) {
                  const buttonText = button.textContent.toLowerCase().trim();
                  console.log(`Cookie Away: Iframe button: "${buttonText}"`);
                  
                  if ((preferences.rejectAll && 
                       (buttonText.includes('reject') || buttonText.includes('only necessary'))) ||
                      (!preferences.rejectAll && buttonText.includes('accept'))) {
                    
                    const iframeClickResult = safeClick(button);
                    if (iframeClickResult) {
                      console.log(`Cookie Away: Successfully clicked button in iframe: "${buttonText}"`);
                      return true;
                    }
                  }
                }
              }
            } catch (err) {
              // Expected error for cross-origin iframes
              console.log('Cookie Away: Could not access iframe content (likely cross-origin)');
            }
          }
        }
        
        return false;
      }
    } else {
      console.log('Cookie Away: No suitable button found in Amazon banner');
      return false;
    }
  } catch (error) {
    console.log(`Cookie Away: Error in Amazon handler - ${error.message}`);
    return false;
  }
}