/**
 * Cookie Away - Cookiebot Handler
 * 
 * This script provides detection and interaction functions for Cookiebot consent dialogs
 */

// Detect if Cookiebot is present on the page
function detectCookiebot() {
  // Check for Cookiebot script
  if (
    document.querySelector('script[src*="cookiebot"]') ||
    document.querySelector('script[src*="consent.cookiebot.com"]') ||
    window.Cookiebot ||
    document.getElementById('CookiebotWidget') ||
    document.getElementById('CybotCookiebotDialog')
  ) {
    return true;
  }
  
  // Look for Cookiebot DOM elements
  const cookiebotElements = [
    '#CybotCookiebotDialog',
    '#CookiebotWidget',
    '.CybotCookiebotDialogBodyButton',
    '[id^="Cookiebot"]',
    '[class^="CybotCookiebot"]'
  ];
  
  for (const selector of cookiebotElements) {
    if (document.querySelector(selector)) {
      return true;
    }
  }
  
  return false;
}

// Handle Cookiebot consent dialog based on user preferences
function handleCookiebot(preferences) {
  // Wait for the Cookiebot dialog to be fully loaded
  const waitForDialog = () => {
    // Find dialog element
    const dialog = document.getElementById('CybotCookiebotDialog');
    
    // If dialog is not found or not visible, retry after a short delay
    if (!dialog || !isElementVisible(dialog)) {
      setTimeout(waitForDialog, 300);
      return;
    }
    
    // Try to find Cookiebot buttons by their IDs first
    const detailsButton = document.getElementById('CybotCookiebotDialogDetailBodyContentCookieContainerTypes');
    const acceptAllButton = document.getElementById('CybotCookiebotDialogBodyLevelButtonLevelOptinAllowAll');
    const acceptNecessaryButton = document.getElementById('CybotCookiebotDialogBodyLevelButtonLevelOptinDeclineAll') || 
                                  document.getElementById('CybotCookiebotDialogBodyButtonDecline');
    const saveButton = document.getElementById('CybotCookiebotDialogBodyLevelButtonLevelOptinAllowallSelection') || 
                        document.getElementById('CybotCookiebotDialogBodyButtonAccept');
    
    // If we couldn't find by ID, look for elements with specific texts
    if (!detailsButton) {
      const possibleDetailsButton = findButtonByText(dialog, ['details', 'customize', 'preferences']);
      if (possibleDetailsButton) detailsButton = possibleDetailsButton;
    }
    
    // Decision logic based on user preferences
    if (preferences.necessary && !preferences.analytics && !preferences.marketing && !preferences.personalization) {
      // User wants only necessary cookies
      if (acceptNecessaryButton) {
        safeClick(acceptNecessaryButton);
        notifyBackgroundScript('Cookiebot', true);
        return;
      }
      
      // If no direct "necessary only" button, try to go to details and uncheck categories
      if (detailsButton) {
        safeClick(detailsButton);
        
        // Wait for details panel to load
        setTimeout(() => {
          // Uncheck all non-necessary checkboxes
          const checkboxes = document.querySelectorAll('#CybotCookiebotDialogDetailBodyContentCookieContainerTypes input[type="checkbox"]');
          checkboxes.forEach(checkbox => {
            if (checkbox.id.includes('Necessary')) {
              checkbox.checked = true;
            } else {
              checkbox.checked = false;
            }
          });
          
          // Save preferences
          if (saveButton) {
            safeClick(saveButton);
            notifyBackgroundScript('Cookiebot', true);
          } else {
            notifyBackgroundScript('Cookiebot', false);
          }
        }, 300);
        return;
      }
    } else if (preferences.necessary && preferences.analytics && preferences.marketing && preferences.personalization) {
      // User accepts all cookies
      if (acceptAllButton) {
        safeClick(acceptAllButton);
        notifyBackgroundScript('Cookiebot', true);
        return;
      }
    } else {
      // User has custom preferences
      if (detailsButton) {
        safeClick(detailsButton);
        
        // Wait for details panel to load
        setTimeout(() => {
          // Set checkboxes according to user preferences
          const preferenceMap = {
            'Necessary': preferences.necessary,
            'Preference': preferences.personalization,
            'Statistics': preferences.analytics,
            'Marketing': preferences.marketing,
            'Advertising': preferences.marketing // Alternative naming
          };
          
          // Look for checkboxes in the Cookiebot dialog
          for (const category in preferenceMap) {
            const checkboxes = document.querySelectorAll(`#CybotCookiebotDialogDetailBodyContentCookieContainerTypes input[type="checkbox"][id*="${category}"]`);
            checkboxes.forEach(checkbox => {
              checkbox.checked = preferenceMap[category];
            });
          }
          
          // Save preferences
          if (saveButton) {
            safeClick(saveButton);
            notifyBackgroundScript('Cookiebot', true);
          } else {
            notifyBackgroundScript('Cookiebot', false);
          }
        }, 300);
        return;
      }
    }
    
    // If we couldn't handle the dialog properly
    notifyBackgroundScript('Cookiebot', false);
  };
  
  // Start the process
  waitForDialog();
}
