/**
 * Cookie Away - Custom Cookie Banner Handler
 * 
 * This script provides detection and interaction functions for custom (non-standard) cookie consent dialogs
 * that aren't captured by the more specific handlers.
 */

// Enhanced detection for custom cookie banners with additional patterns
function detectCustomCookieBanner() {
  // Common selectors for custom cookie banners, privacy notices, GDPR notices, etc.
  const highConfidenceSelectors = [
    // Very specific cookie banner selectors
    '[class*="cookie-banner"]', '[id*="cookie-banner"]',
    '[class*="cookie-notice"]', '[id*="cookie-notice"]',
    '[class*="cookie-policy"]', '[id*="cookie-policy"]',
    '[class*="cookie-consent"]', '[id*="cookie-consent"]',
    '[class*="cookie-popup"]', '[id*="cookie-popup"]',
    '[class*="gdpr-banner"]', '[id*="gdpr-banner"]',
    '[class*="gdpr-notice"]', '[id*="gdpr-notice"]',
    '[class*="consent-dialog"]', '[id*="consent-dialog"]',
    '[class*="consent-modal"]', '[id*="consent-modal"]',
    '[class*="cookie-modal"]', '[id*="cookie-modal"]',
    // Specific CMP selectors
    '[id*="CybotCookiebot"]', '[id*="onetrust-consent"]',
    'div[aria-labelledby*="cookieModalLabel"]',
    '#cookieprivacy', '#cookienotice', '#cookie-law-info-bar',
    '#cmplz-cookiebanner', '#cookie-law', '#cookie-consent',
    '[data-testid="cookie-policy-banner"]'
  ];
  
  const mediumConfidenceSelectors = [
    // Less specific but still likely cookie-related
    '[class*="privacy-banner"]', '[id*="privacy-banner"]',
    '[class*="privacy-notice"]', '[id*="privacy-notice"]',
    '[class*="privacy-alert"]', '[id*="privacy-alert"]',
    '[class*="privacy-dialog"]', '[id*="privacy-dialog"]',
    'div[role="alert"][class*="cookie"]', 'div[role="dialog"][class*="cookie"]',
    'div[role="alert"][class*="privacy"]', 'div[role="dialog"][class*="privacy"]',
    'div[role="alert"][class*="consent"]', 'div[role="dialog"][class*="consent"]'
  ];
  
  // Very specific text patterns that are almost certainly cookie-related
  const highConfidenceTextPatterns = [
    /cookie consent/i, /cookie preferences/i, /cookie settings/i,
    /we use cookies/i, /this site uses cookies/i, /this website uses cookies/i,
    /accept cookies/i, /reject cookies/i, /manage cookies/i,
    /accept all cookies/i, /reject all cookies/i,
    /necessary cookies only/i, /only necessary cookies/i,
    /cookie policy/i, /privacy policy/i
  ];
  
  // Less specific but possibly cookie-related text patterns
  const mediumConfidenceTextPatterns = [
    /cookie/i, /cookies/i, /gdpr/i, /ccpa/i,
    /privacy/i, /consent/i, /data protection/i,
    /accept & close/i, /I agree/i, /I accept/i,
    /accept all/i, /agree to all/i, /agree all/i
  ];

  // First check for the high confidence elements by selector
  for (const selector of highConfidenceSelectors) {
    const elements = document.querySelectorAll(selector);
    for (const el of elements) {
      if (isElementVisible(el)) {
        // Check if it has buttons or links (interactive elements)
        const interactiveElements = el.querySelectorAll('button, a, [role="button"], input[type="button"], input[type="submit"]');
        
        // Check if there are interactive elements
        if (interactiveElements.length > 0) {
          return {
            element: el, 
            interactiveElements: Array.from(interactiveElements),
            confidence: 'high'
          };
        }
        
        // Check for high confidence text
        const text = el.textContent.toLowerCase();
        if (highConfidenceTextPatterns.some(pattern => pattern.test(text))) {
          return {
            element: el, 
            interactiveElements: [],
            confidence: 'high'
          };
        }
      }
    }
  }
  
  // Then check medium confidence selectors
  for (const selector of mediumConfidenceSelectors) {
    const elements = document.querySelectorAll(selector);
    for (const el of elements) {
      if (isElementVisible(el)) {
        const interactiveElements = el.querySelectorAll('button, a, [role="button"], input[type="button"], input[type="submit"]');
        
        // Check if there are interactive elements AND medium confidence text patterns
        if (interactiveElements.length > 0) {
          const text = el.textContent.toLowerCase();
          if (mediumConfidenceTextPatterns.some(pattern => pattern.test(text))) {
            return {
              element: el, 
              interactiveElements: Array.from(interactiveElements),
              confidence: 'medium'
            };
          }
        }
      }
    }
  }
  
  // Look for fixed/absolute positioned elements with cookie text (could be banners)
  const possibleBanners = Array.from(document.querySelectorAll('div, section, aside, footer, header'))
    .filter(el => {
      const style = window.getComputedStyle(el);
      return (
        isElementVisible(el) && 
        (style.position === 'fixed' || style.position === 'sticky' || style.position === 'absolute') &&
        (parseInt(style.bottom) === 0 || parseInt(style.top) === 0) &&
        el.offsetWidth > window.innerWidth * 0.5 // Must be at least half the window width
      );
    });
  
  for (const banner of possibleBanners) {
    const text = banner.textContent.toLowerCase();
    const interactiveElements = banner.querySelectorAll('button, a, [role="button"], input[type="button"], input[type="submit"]');
    
    // Only consider banners with interactive elements
    if (interactiveElements.length === 0) continue;
    
    // Check for high confidence text patterns
    if (highConfidenceTextPatterns.some(pattern => pattern.test(text))) {
      return {
        element: banner, 
        interactiveElements: Array.from(interactiveElements),
        confidence: 'high'
      };
    }
    
    // Check for medium confidence text patterns
    if (mediumConfidenceTextPatterns.some(pattern => pattern.test(text))) {
      return {
        element: banner, 
        interactiveElements: Array.from(interactiveElements),
        confidence: 'medium'
      };
    }
  }
  
  // Check for dialogs with cookie-related content
  const dialogs = Array.from(document.querySelectorAll('div[role="dialog"], div[aria-modal="true"], div[class*="modal"], div[class*="dialog"], div[class*="drawer"]'))
    .filter(el => isElementVisible(el));
  
  for (const dialog of dialogs) {
    const text = dialog.textContent.toLowerCase();
    const interactiveElements = dialog.querySelectorAll('button, a, [role="button"], input[type="button"], input[type="submit"]');
    
    // Only consider dialogs with interactive elements
    if (interactiveElements.length === 0) continue;
    
    // Check for high confidence text patterns
    if (highConfidenceTextPatterns.some(pattern => pattern.test(text))) {
      return {
        element: dialog, 
        interactiveElements: Array.from(interactiveElements),
        confidence: 'high'
      };
    }
    
    // Check for medium confidence text patterns with multiple matches
    const mediumMatches = mediumConfidenceTextPatterns.filter(pattern => pattern.test(text));
    if (mediumMatches.length >= 2) { // Need at least 2 matches for medium confidence
      return {
        element: dialog, 
        interactiveElements: Array.from(interactiveElements),
        confidence: 'medium'
      };
    }
  }
  
  return null;
}

// Handle custom cookie banners based on user preferences
function handleCustomCookieBanner(preferences) {
  // Detect the custom banner
  const result = detectCustomCookieBanner();
  if (!result || !result.element) {
    notifyBackgroundScript('Custom', false);
    return;
  }
  
  const banner = result.element;
  let interactiveElements = result.interactiveElements;
  
  // If no interactive elements were found during detection, try to find them now
  if (interactiveElements.length === 0) {
    interactiveElements = Array.from(banner.querySelectorAll('button, a, [role="button"], input[type="button"], input[type="submit"]'));
  }
  
  if (interactiveElements.length === 0) {
    notifyBackgroundScript('Custom', false);
    return;
  }
  
  // Define button text patterns for different consent actions
  const acceptAllPatterns = [
    /accept all/i, /accept cookies/i, /allow all/i, /agree/i, /accept/i, /allow/i, 
    /enable all/i, /agree to all/i, /i agree/i, /consent/i, /ok/i, /continue/i,
    /aceptar/i, /accepter/i, /akzeptieren/i
  ];
  
  const rejectAllPatterns = [
    /reject all/i, /reject cookies/i, /decline all/i, /deny all/i, /refuse all/i,
    /disagree/i, /reject/i, /decline/i, /deny/i, /refuse/i, /necessary only/i, 
    /only necessary/i, /essential only/i, /only essential/i, /required only/i,
    /only required/i, /rechazar/i, /refuser/i, /ablehnen/i
  ];
  
  const customizePatterns = [
    /customize/i, /settings/i, /preferences/i, /manage/i, /choose/i, /select/i,
    /manage cookies/i, /cookie settings/i, /more options/i, /more info/i,
    /personalizar/i, /parametres/i, /einstellungen/i
  ];
  
  // Decision logic based on user preferences
  if (preferences.necessary && !preferences.analytics && !preferences.marketing && !preferences.personalization) {
    // User wants only necessary cookies - look for "reject all" or "necessary only" buttons
    
    // First, try to find buttons by text
    for (const element of interactiveElements) {
      const text = element.textContent.trim().toLowerCase();
      
      // Check text content
      if (rejectAllPatterns.some(pattern => pattern.test(text))) {
        safeClick(element);
        notifyBackgroundScript('Custom', true);
        return;
      }
      
      // Check aria-label if available
      const ariaLabel = element.getAttribute('aria-label');
      if (ariaLabel && rejectAllPatterns.some(pattern => pattern.test(ariaLabel.toLowerCase()))) {
        safeClick(element);
        notifyBackgroundScript('Custom', true);
        return;
      }
    }
    
    // If no reject button was found, try to find customize button
    for (const element of interactiveElements) {
      const text = element.textContent.trim().toLowerCase();
      
      if (customizePatterns.some(pattern => pattern.test(text))) {
        safeClick(element);
        
        // Wait for the customize panel to load
        setTimeout(() => {
          // Try to find and uncheck all non-necessary checkboxes
          const allCheckboxes = document.querySelectorAll('input[type="checkbox"]');
          let manipulatedCheckboxes = false;
          
          for (const checkbox of allCheckboxes) {
            // Skip if checkbox is disabled (usually means it's necessary)
            if (checkbox.disabled) continue;
            
            // Try to determine if this is a necessary cookie checkbox
            const label = checkbox.closest('label') || 
                          document.querySelector(`label[for="${checkbox.id}"]`) ||
                          checkbox.parentElement;
            
            const labelText = label ? label.textContent.toLowerCase() : '';
            const isNecessary = labelText.includes('necessary') || 
                               labelText.includes('essential') || 
                               labelText.includes('required') ||
                               labelText.includes('functional') ||
                               checkbox.disabled;  // Disabled checkboxes are usually for necessary cookies
            
            // Uncheck non-necessary cookies
            if (!isNecessary && checkbox.checked) {
              safeClick(checkbox);
              manipulatedCheckboxes = true;
            }
          }
          
          // Look for a save or confirm button
          setTimeout(() => {
            const savePatterns = [/save/i, /confirm/i, /done/i, /apply/i, /ok/i, /continue/i, /guardar/i, /enregistrer/i, /speichern/i];
            const saveButtons = Array.from(document.querySelectorAll('button, a, [role="button"], input[type="button"], input[type="submit"]'))
              .filter(el => isElementVisible(el) && 
                (savePatterns.some(pattern => pattern.test(el.textContent.trim().toLowerCase())) ||
                 (el.getAttribute('aria-label') && savePatterns.some(pattern => pattern.test(el.getAttribute('aria-label').toLowerCase())))));
            
            if (saveButtons.length > 0) {
              safeClick(saveButtons[0]);
              notifyBackgroundScript('Custom', true);
            } else {
              // If no save button found, try the last or first button as a fallback
              const visibleButtons = Array.from(document.querySelectorAll('button, a, [role="button"], input[type="button"], input[type="submit"]'))
                .filter(el => isElementVisible(el));
              
              if (visibleButtons.length > 0) {
                // For "reject" scenarios, usually the last button is "save" or the first is "reject all"
                safeClick(visibleButtons[visibleButtons.length - 1]);
                notifyBackgroundScript('Custom', true);
              } else {
                notifyBackgroundScript('Custom', false);
              }
            }
          }, 500);
        }, 500);
        return;
      }
    }
    
    // If no specific button was found, click the last button (which is often "reject" or "close")
    if (interactiveElements.length > 0) {
      safeClick(interactiveElements[interactiveElements.length - 1]);
      notifyBackgroundScript('Custom', true);
      return;
    }
  } 
  else if (preferences.necessary && preferences.analytics && preferences.marketing && preferences.personalization) {
    // User accepts all cookies - look for "accept all" buttons
    
    for (const element of interactiveElements) {
      const text = element.textContent.trim().toLowerCase();
      
      // Check text content
      if (acceptAllPatterns.some(pattern => pattern.test(text))) {
        safeClick(element);
        notifyBackgroundScript('Custom', true);
        return;
      }
      
      // Check aria-label if available
      const ariaLabel = element.getAttribute('aria-label');
      if (ariaLabel && acceptAllPatterns.some(pattern => pattern.test(ariaLabel.toLowerCase()))) {
        safeClick(element);
        notifyBackgroundScript('Custom', true);
        return;
      }
    }
    
    // If no specific button was found, click the first button (which is often "accept all")
    if (interactiveElements.length > 0) {
      safeClick(interactiveElements[0]);
      notifyBackgroundScript('Custom', true);
      return;
    }
  }
  else {
    // User has custom preferences - try to find customize button
    
    for (const element of interactiveElements) {
      const text = element.textContent.trim().toLowerCase();
      
      if (customizePatterns.some(pattern => pattern.test(text))) {
        safeClick(element);
        
        // Wait for the customize panel to load
        setTimeout(() => {
          // Find and set checkboxes according to user preferences
          const allCheckboxes = document.querySelectorAll('input[type="checkbox"]');
          let manipulatedCheckboxes = false;
          
          for (const checkbox of allCheckboxes) {
            // Skip if checkbox is disabled (usually means it's necessary)
            if (checkbox.disabled) continue;
            
            // Try to determine checkbox type
            const label = checkbox.closest('label') || 
                          document.querySelector(`label[for="${checkbox.id}"]`) ||
                          checkbox.parentElement;
            
            const labelText = label ? label.textContent.toLowerCase() : '';
            let categoryMatch = null;
            
            if (labelText.includes('necessary') || labelText.includes('essential') || labelText.includes('required') || labelText.includes('functional')) {
              categoryMatch = 'necessary';
            } else if (labelText.includes('analytic') || labelText.includes('statistic') || labelText.includes('performance')) {
              categoryMatch = 'analytics';
            } else if (labelText.includes('market') || labelText.includes('advertis') || labelText.includes('ad') || labelText.includes('targeting')) {
              categoryMatch = 'marketing';
            } else if (labelText.includes('personal') || labelText.includes('preference') || labelText.includes('experience')) {
              categoryMatch = 'personalization';
            }
            
            // Set checkboxes according to preferences
            if (categoryMatch) {
              const shouldBeChecked = preferences[categoryMatch];
              if (checkbox.checked !== shouldBeChecked) {
                safeClick(checkbox);
                manipulatedCheckboxes = true;
              }
            }
          }
          
          // Look for a save or confirm button
          setTimeout(() => {
            const savePatterns = [/save/i, /confirm/i, /done/i, /apply/i, /ok/i, /continue/i, /guardar/i, /enregistrer/i, /speichern/i];
            const saveButtons = Array.from(document.querySelectorAll('button, a, [role="button"], input[type="button"], input[type="submit"]'))
              .filter(el => isElementVisible(el) && 
                (savePatterns.some(pattern => pattern.test(el.textContent.trim().toLowerCase())) ||
                 (el.getAttribute('aria-label') && savePatterns.some(pattern => pattern.test(el.getAttribute('aria-label').toLowerCase())))));
            
            if (saveButtons.length > 0) {
              safeClick(saveButtons[0]);
              notifyBackgroundScript('Custom', true);
            } else {
              // If no save button found, try the last button as a fallback
              const visibleButtons = Array.from(document.querySelectorAll('button, a, [role="button"], input[type="button"], input[type="submit"]'))
                .filter(el => isElementVisible(el));
              
              if (visibleButtons.length > 0) {
                safeClick(visibleButtons[visibleButtons.length - 1]);
                notifyBackgroundScript('Custom', true);
              } else {
                notifyBackgroundScript('Custom', false);
              }
            }
          }, 500);
        }, 500);
        return;
      }
    }
    
    // If no customize button was found, fall back to accept all or reject all based on which preferences are enabled
    const preferenceCount = (preferences.analytics ? 1 : 0) + 
                           (preferences.marketing ? 1 : 0) + 
                           (preferences.personalization ? 1 : 0);
    
    // If user has more preferences enabled than disabled, click accept all
    if (preferenceCount >= 2) {
      for (const element of interactiveElements) {
        const text = element.textContent.trim().toLowerCase();
        if (acceptAllPatterns.some(pattern => pattern.test(text))) {
          safeClick(element);
          notifyBackgroundScript('Custom', true);
          return;
        }
      }
      
      // Fallback to first button (often accept)
      if (interactiveElements.length > 0) {
        safeClick(interactiveElements[0]);
        notifyBackgroundScript('Custom', true);
      }
    } else {
      // Otherwise click reject all
      for (const element of interactiveElements) {
        const text = element.textContent.trim().toLowerCase();
        if (rejectAllPatterns.some(pattern => pattern.test(text))) {
          safeClick(element);
          notifyBackgroundScript('Custom', true);
          return;
        }
      }
      
      // Fallback to last button (often reject)
      if (interactiveElements.length > 0) {
        safeClick(interactiveElements[interactiveElements.length - 1]);
        notifyBackgroundScript('Custom', true);
      }
    }
  }
  
  notifyBackgroundScript('Custom', false);
}