/**
 * Cookie Away - DJI.com Handler
 * 
 * This handler is specifically designed to handle cookie banners on DJI.com
 * and related domains which have a unique cookie consent flow.
 */

// Detect DJI cookie banners
function detectDJICookieBanner() {
  try {
    // Check if we're on a DJI domain
    if (!window.location.hostname.includes('dji')) {
      return false;
    }
    
    console.log('Cookie Away: Checking for DJI cookie banner');
    
    // DJI uses several different banner classes
    // First, look for the most specific selectors
    const specificBannerSelectors = [
      '.cookie-policy-banner', 
      '#cookie-banner', 
      '#cookie-notice',
      '.cookie-notice',
      '.cookie-consent',
      '.cookie-policy-dialog',
      '.dji-cookie',
      '.gdpr-banner',
      '.privacy-banner'
    ];
    
    for (const selector of specificBannerSelectors) {
      const banner = document.querySelector(selector);
      if (banner && isElementVisible(banner)) {
        console.log(`Cookie Away: DJI cookie banner detected with selector: ${selector}`);
        return banner;
      }
    }
    
    // If specific selectors don't match, look for broader patterns
    const cookieBanner = document.querySelector('[class*="cookie"], [class*="gdprCookie"], [class*="cookiePolicy"]');
    
    if (cookieBanner && isElementVisible(cookieBanner)) {
      console.log('Cookie Away: DJI cookie banner detected with generic cookie selector');
      return cookieBanner;
    }
    
    // Secondary check for fixed position banner containers
    const potentialBanners = Array.from(document.querySelectorAll('div[class*="banner"], div[class*="policy"], div[class*="consent"], div[role="dialog"], div[aria-modal="true"]'))
      .filter(el => {
        const style = window.getComputedStyle(el);
        const text = el.textContent.toLowerCase();
        
        // Check if it's a fixed/sticky position element with cookie-related text
        return (style.position === 'fixed' || style.position === 'sticky') && 
               isElementVisible(el) && 
               (text.includes('cookie') || text.includes('gdpr') || 
                text.includes('privacy') || text.includes('consent'));
      });
    
    if (potentialBanners.length > 0) {
      console.log('Cookie Away: DJI alternative cookie banner detected via position/text');
      return potentialBanners[0];
    }
    
    // Third check: Look for any visible dialog that might be a cookie banner
    // This is specific to DJI which sometimes uses non-standard dialog structures
    const allDialogs = Array.from(document.querySelectorAll('div[role="dialog"], div[aria-modal="true"], div.modal, div.popup'))
      .filter(el => isElementVisible(el));
    
    for (const dialog of allDialogs) {
      const text = dialog.textContent.toLowerCase();
      if (text.includes('cookie') || text.includes('gdpr') || text.includes('privacy policy')) {
        console.log('Cookie Away: DJI cookie dialog detected via dialog element');
        return dialog;
      }
    }
    
    // Final check: If we're on a cookie policy page (from redirect), find the form/container
    if (window.location.href.includes('cookie') || 
        window.location.href.includes('privacy') || 
        window.location.href.includes('consent')) {
      
      // Look for forms or containers with cookie preferences
      const forms = document.querySelectorAll('form, div[class*="preference"], div[class*="consent"], div[class*="cookie-setting"]');
      for (const form of forms) {
        if (isElementVisible(form) && form.querySelectorAll('input[type="checkbox"], button').length > 0) {
          console.log('Cookie Away: Found DJI cookie preferences form on policy page');
          return form;
        }
      }
    }
    
    return false;
  } catch (error) {
    console.log(`Cookie Away: Error detecting DJI banner - ${error.message}`);
    return false;
  }
}

// Handle DJI cookie banner
async function handleDJICookieBanner() {
  try {
    const bannerElement = detectDJICookieBanner();
    if (!bannerElement) return false;
    
    console.log('Cookie Away: Handling DJI cookie banner');
    
    // Get user preferences
    const hostname = window.location.hostname;
    const { preferences } = await getUserPreferences(hostname);
    
    let buttonToClick = null;
    
    // First, check if we're on a cookie settings/preferences page
    // This happens if the user was redirected to a cookie policy page
    const isPreferencesPage = window.location.href.includes('cookie') || 
                             window.location.pathname.includes('privacy') || 
                             bannerElement.tagName === 'FORM';
    
    if (isPreferencesPage) {
      console.log('Cookie Away: Detected we are on a DJI cookie preferences page');
      
      // Look for checkboxes to uncheck (except necessary ones)
      const checkboxes = bannerElement.querySelectorAll('input[type="checkbox"]');
      let foundCheckboxes = false;
      
      if (checkboxes.length > 0) {
        console.log(`Cookie Away: Found ${checkboxes.length} checkboxes on DJI preferences page`);
        foundCheckboxes = true;
        
        // Log all checkbox labels
        for (const checkbox of checkboxes) {
          const label = checkbox.closest('label') || 
                       document.querySelector(`label[for="${checkbox.id}"]`) ||
                       checkbox.parentElement;
          
          const labelText = label ? label.textContent.trim() : 'No label';
          console.log(`Cookie Away: Checkbox - "${labelText}" - Checked: ${checkbox.checked}`);
          
          // Don't touch disabled checkboxes (usually necessary cookies)
          if (checkbox.disabled) continue;
          
          // Check if this is a necessary/required cookie checkbox
          const isNecessary = label && (
            label.textContent.toLowerCase().includes('necessary') ||
            label.textContent.toLowerCase().includes('essential') ||
            label.textContent.toLowerCase().includes('required') ||
            label.textContent.toLowerCase().includes('functional')
          );
          
          // Uncheck non-necessary checkboxes if user wants to reject all
          if (!isNecessary && preferences.rejectAll && checkbox.checked) {
            console.log(`Cookie Away: Unchecking non-necessary DJI checkbox - "${labelText}"`);
            checkbox.checked = false;
            
            // Trigger change event for dynamic forms
            const event = new Event('change', { bubbles: true });
            checkbox.dispatchEvent(event);
          }
        }
      }
      
      // Now look for the save/submit button
      const submitButton = findButtonByText(bannerElement, [
        'save', 'submit', 'confirm', 'apply', 
        'save settings', 'save preferences',
        'save changes', 'speichern', 'bestätigen'
      ]);
      
      if (submitButton) {
        console.log('Cookie Away: Found submit button on DJI preferences page');
        buttonToClick = submitButton;
      } else if (foundCheckboxes) {
        // If we toggled checkboxes but found no submit button, look for any button
        // that's not a "Cancel" or "Close" button
        const buttons = bannerElement.querySelectorAll('button, input[type="button"], input[type="submit"]');
        
        for (const button of buttons) {
          const buttonText = button.textContent.toLowerCase().trim();
          // Skip buttons that appear to be cancel/close
          if (!buttonText.includes('cancel') && !buttonText.includes('close') && 
              !buttonText.includes('abort') && !buttonText.includes('back')) {
            console.log(`Cookie Away: Using non-cancel button as submit - "${buttonText}"`);
            buttonToClick = button;
            break;
          }
        }
      }
    } else {
      // Normal cookie banner handling
      // Log all buttons for debugging
      const allButtons = bannerElement.querySelectorAll('button, a, [role="button"]');
      console.log(`Cookie Away: Found ${allButtons.length} buttons in DJI banner`);
      
      for (let i = 0; i < allButtons.length; i++) {
        const btn = allButtons[i];
        console.log(`Cookie Away: DJI Button ${i}: "${btn.textContent.trim()}" (${btn.tagName})`);
      }
      
      // DJI sites often have buttons with specific IDs or class names
      // First, look for the reject button by specific selectors
      const rejectButtons = bannerElement.querySelectorAll('[data-reject], [class*="reject"], [id*="reject"], [class*="necessary"], [class*="decline"]');
      
      if (rejectButtons.length > 0) {
        for (const button of rejectButtons) {
          const buttonText = button.textContent.toLowerCase().trim();
          // Verify it's not a policy link
          if (!buttonText.includes('policy') && !buttonText.includes('learn') && !buttonText.includes('more')) {
            buttonToClick = button;
            console.log('Cookie Away: Found DJI reject button by selector');
            break;
          }
        }
      }
      
      // If no specific reject button found, try finding by text
      if (!buttonToClick) {
        // Expanded text patterns for DJI banners, including multiple languages
        const rejectText = [
          'only necessary', 'necessary cookies', 'reject', 'decline', 
          'reject all', 'necessary only', 'nur notwendige', 'erforderliche',
          'ablehnen', 'verweigern', 'nicht akzeptieren', 'nein danke',
          'refuser', 'refuser tout', 'uniquement nécessaires', 
          'deny', 'no, thanks', 'no thanks', 'not now',
          'refuse', 'refuse cookies', 'avböj', 'avvisa',
          'avvis', 'decline cookies', 'only required',
          'cookie settings', 'cookie preferences' // Sometimes the settings button leads to better options
        ];
        
        for (const button of allButtons) {
          const buttonText = button.textContent.toLowerCase().trim();
          if (rejectText.some(text => buttonText.includes(text))) {
            // Enhanced policy link detection - avoid links to policy pages
            const isPolicyLink = buttonText.includes('policy') || 
                               buttonText.includes('learn more') || 
                               buttonText.includes('read more') ||
                               buttonText.includes('more information') ||
                               button.getAttribute('href')?.includes('policy');
                               
            if (!isPolicyLink) {
              buttonToClick = button;
              console.log(`Cookie Away: Found DJI reject button by text "${buttonText}"`);
              break;
            } else {
              console.log(`Cookie Away: Skipping policy link "${buttonText}"`);
            }
          }
        }
      }
      
      // Look for a "Cookie Settings" or "Customize" button if we couldn't find a reject button
      // Sometimes these lead to more granular controls
      if (!buttonToClick) {
        const settingsButton = findButtonByText(bannerElement, [
          'customize', 'settings', 'preferences', 'cookie settings',
          'manage cookies', 'manage preferences', 'einstellungen',
          'configure', 'paramètres', 'ajustes'
        ]);
        
        if (settingsButton) {
          console.log('Cookie Away: Attempting to click settings button to access preferences');
          const settingsClicked = safeClick(settingsButton);
          
          if (settingsClicked) {
            // Wait for preferences panel to appear
            console.log('Cookie Away: Clicked settings button, waiting for panel to appear');
            
            // Return true since we've started a process
            // DJI often shows a second dialog after this click
            return true;
          }
        }
      }
      
      // Fallback to the second button if we couldn't find a clear reject button
      // DJI often has Accept as first button and Reject as second
      if (!buttonToClick && allButtons.length >= 2) {
        // Skip the first button (usually accept) and use the second button
        const secondButton = allButtons[1];
        const buttonText = secondButton.textContent.toLowerCase().trim();
        
        // Verify it's not a policy link
        if (!buttonText.includes('policy') && !buttonText.includes('learn') && !buttonText.includes('more')) {
          buttonToClick = secondButton;
          console.log('Cookie Away: Using second button as fallback for DJI banner');
        }
      }
    }
    
    // Click the button if found
    if (buttonToClick) {
      console.log(`Cookie Away: Attempting to click DJI ${isPreferencesPage ? 'preferences' : 'banner'} button`);
      const clickResult = safeClick(buttonToClick);
      
      if (clickResult) {
        console.log('Cookie Away: Successfully clicked DJI button');
        return true;
      } else {
        console.log('Cookie Away: Failed to click DJI button');
        
        // One last attempt - if button has an onclick handler, try to call it directly
        if (buttonToClick.onclick) {
          try {
            console.log('Cookie Away: Trying to call onclick handler directly');
            buttonToClick.onclick();
            console.log('Cookie Away: Successfully called onclick handler');
            return true;
          } catch (e) {
            console.log(`Cookie Away: Failed to call onclick handler - ${e.message}`);
          }
        }
        return false;
      }
    } else {
      console.log('Cookie Away: No suitable button found in DJI banner');
      return false;
    }
  } catch (error) {
    console.log(`Cookie Away: Error in DJI handler - ${error.message}`);
    return false;
  }
}