/**
 * Europa websites handler for Cookie Away
 * 
 * This handler is specifically designed for European Union domains
 * (europa.eu and european-union.europa.eu) which have a unique cookie banner
 * implementation that can cause redirection loops.
 */

const EUROPA_DOMAINS = ['europa.eu', 'european-union.europa.eu'];

function detectEuropa() {
  try {
    // Check if we're on a Europa domain
    const hostname = window.location.hostname;
    if (!EUROPA_DOMAINS.some(domain => hostname.includes(domain))) {
      return false;
    }
    
    // Check for the specific EU cookie banner
    const cookieBanner = document.querySelector('.wt-cck-banner') || 
                         document.querySelector('#cookie-consent-banner') ||
                         document.querySelector('[data-ecl-cookie-banner]');
    
    // Check for cookie policy page
    const isCookiePolicyPage = window.location.pathname.includes('/cookies_');
    
    return cookieBanner !== null || isCookiePolicyPage;
  } catch (error) {
    console.log(`Cookie Away: Error detecting Europa CMP - ${error.message}`);
    return false;
  }
}

function handleEuropa(preferences) {
  try {
    // Check if we're on the cookies policy page
    const isCookiePolicyPage = window.location.pathname.includes('/cookies_');
    
    // Find the banner element
    let bannerElement = document.querySelector('.wt-cck-banner') || 
                         document.querySelector('#cookie-consent-banner') ||
                         document.querySelector('[data-ecl-cookie-banner]');
    
    // If we're on the cookies policy page and no banner is found, look for the form
    if (isCookiePolicyPage && !bannerElement) {
      bannerElement = document.querySelector('form[action*="cookies"]') ||
                      document.querySelector('form.cookie-form');
    }
    
    if (!bannerElement) {
      console.log('Cookie Away: Could not find Europa cookie banner');
      notifyBackgroundScript('Europa', false);
      return;
    }
    
    let buttonToClick = null;
    
    // Handle based on user preferences
    if (preferences.acceptAll) {
      // Look for accept all button
      buttonToClick = findButtonByText(bannerElement, [
        'accept all cookies', 'accept all', 'accept', 'agree'
      ]);
    } else {
      // Look for essential/necessary only button
      buttonToClick = findButtonByText(bannerElement, [
        'accept only essential cookies', 'only essential', 'essential cookies only',
        'necessary only', 'essential only', 'reject all', 'reject'
      ]);
      
      // If no reject button is found, try finding buttons with related text
      if (!buttonToClick) {
        const inputs = bannerElement.querySelectorAll('input[type="radio"], input[type="checkbox"]');
        
        for (const input of inputs) {
          const label = input.closest('label') || 
                        document.querySelector(`label[for="${input.id}"]`);
          
          if (label && label.textContent.toLowerCase().includes('essential')) {
            // Select this option
            if (!input.checked) {
              input.click();
            }
            
            // Now look for a save/submit button
            const saveButton = findButtonByText(bannerElement, [
              'save', 'submit', 'confirm', 'apply', 'continue'
            ]) || bannerElement.querySelector('input[type="submit"], button[type="submit"]');
            
            if (saveButton) {
              buttonToClick = saveButton;
            }
            
            break;
          }
        }
      }
    }
    
    // Click the identified button
    if (buttonToClick) {
      console.log('Cookie Away: Clicking button in Europa cookie banner');
      const clickResult = safeClick(buttonToClick);
      notifyBackgroundScript('Europa', clickResult);
    } else {
      console.log('Cookie Away: Could not find button to click in Europa cookie banner');
      notifyBackgroundScript('Europa', false);
    }
  } catch (error) {
    console.log(`Cookie Away: Error in Europa handler - ${error.message}`);
    notifyBackgroundScript('Europa', false);
  }
}