/**
 * Facebook Cookie Consent Handler for Cookie Away
 * 
 * This handler specifically targets Facebook's cookie consent mechanism which is used across
 * Meta properties (facebook.com, instagram.com, etc.). Meta uses a unique cookie consent flow
 * that often needs special handling.
 * 
 * Note: Facebook often redirects users to cookie policy pages, which we need to handle
 * differently from the regular cookie banners.
 */

// Detects Facebook/Meta cookie consent banner or cookie policy page
function detectFacebookCookieBanner() {
  try {
    const hostname = window.location.hostname;
    const pathname = window.location.pathname;
    const url = window.location.href;
    
    // Check if we're on a Meta domain
    if (!hostname.includes('facebook.') && 
        !hostname.includes('fb.') && 
        !hostname.includes('instagram.') && 
        !hostname.includes('meta.')) {
      return false;
    }
    
    // Check for cookie policy/consent page URLs
    if (pathname.includes('/policies/cookies') || 
        pathname.includes('/privacy/policies/cookies') ||
        pathname.includes('/privacy/consent') ||
        pathname.includes('/help/cookies') ||
        url.includes('cookie_policy_confirmed') ||
        url.includes('cookie_consent')) {
      return true;
    }
    
    // Method 1: Cookie banner dialog
    const cookieDialogs = document.querySelectorAll('[role="dialog"]');
    for (const dialog of cookieDialogs) {
      const dialogText = dialog.textContent.toLowerCase();
      if (dialogText.includes('cookie') || 
          dialogText.includes('privacy') || 
          dialogText.includes('data policy')) {
        return true;
      }
    }
    
    // Method 2: Fixed position banner at the bottom
    const possibleBanners = document.querySelectorAll('div[data-testid="cookie-policy-dialog"], div[data-testid="cookie-policy-banner"]');
    if (possibleBanners.length > 0) {
      return true;
    }
    
    // Method 3: Older style banner
    const fixedBanners = Array.from(document.querySelectorAll('div')).filter(div => {
      const computedStyle = window.getComputedStyle(div);
      return (computedStyle.position === 'fixed' || computedStyle.position === 'sticky') && 
             computedStyle.bottom === '0px' &&
             div.textContent.toLowerCase().includes('cookie');
    });
    
    if (fixedBanners.length > 0) {
      return true;
    }
    
    // Method 4: Check for cookie policy pages based on content
    const bodyText = document.body.textContent.toLowerCase();
    const isCookiePolicyPage = bodyText.includes('cookie policy') && 
                              (bodyText.includes('manage your cookie settings') || 
                               bodyText.includes('cookie preferences'));
                               
    if (isCookiePolicyPage) {
      const consentButtons = Array.from(document.querySelectorAll('button, [role="button"]')).filter(btn => {
        const text = btn.textContent.toLowerCase();
        return text.includes('accept') || 
               text.includes('allow') || 
               text.includes('reject') || 
               text.includes('necessary') ||
               text.includes('essential') ||
               text.includes('cookie preferences') ||
               text.includes('manage');
      });
      
      return consentButtons.length > 0;
    }
    
    return false;
  } catch (error) {
    console.log(`Cookie Away: Error detecting Facebook cookie banner - ${error.message}`);
    return false;
  }
}

// Handles Facebook/Meta cookie consent
function handleFacebookCookieBanner(preferences) {
  try {
    // Special handling for cookie policy/consent pages
    const pathname = window.location.pathname;
    const url = window.location.href;
    
    if (pathname.includes('/policies/cookies') || 
        pathname.includes('/privacy/policies/cookies') ||
        pathname.includes('/privacy/consent') ||
        pathname.includes('/help/cookies') ||
        url.includes('cookie_policy_confirmed') ||
        url.includes('cookie_consent')) {
      
      return handleFacebookCookiePolicyPage(preferences);
    }
    
    // Method 1: Dialog approach
    const cookieDialogs = document.querySelectorAll('[role="dialog"]');
    for (const dialog of cookieDialogs) {
      const dialogText = dialog.textContent.toLowerCase();
      if (dialogText.includes('cookie') || 
          dialogText.includes('privacy') || 
          dialogText.includes('data policy')) {
        
        // Find buttons within the dialog
        const buttons = Array.from(dialog.querySelectorAll('button'));
        
        if (preferences.rejectAll) {
          // Look for buttons that reject or only allow essential cookies
          const rejectButtons = buttons.filter(btn => {
            const text = btn.textContent.toLowerCase();
            return text.includes('essential') || 
                   text.includes('necessary') || 
                   text.includes('reject') || 
                   text.includes('decline') ||
                   text.includes('manage') ||
                   text.includes('settings') ||
                   text.includes('choose');
          });
          
          if (rejectButtons.length > 0) {
            // Find the most likely reject button based on position and text
            const rejectBtn = rejectButtons.find(btn => btn.textContent.toLowerCase().includes('reject')) || 
                             rejectButtons[0];
            
            safeClick(rejectBtn);
            
            // If we clicked a "Manage" or "Settings" button, we need to find the "Reject All" option
            if (rejectBtn.textContent.toLowerCase().includes('manage') || 
                rejectBtn.textContent.toLowerCase().includes('settings') ||
                rejectBtn.textContent.toLowerCase().includes('choose')) {
              
              setTimeout(() => {
                // Find reject all button in the settings panel
                const allButtons = Array.from(document.querySelectorAll('button'));
                const rejectAllBtn = allButtons.find(btn => {
                  const text = btn.textContent.toLowerCase();
                  return text.includes('reject all') || 
                         text.includes('only essential') ||
                         text.includes('save');
                });
                
                if (rejectAllBtn) {
                  safeClick(rejectAllBtn);
                  notifyBackgroundScript('Facebook', true);
                } else {
                  // Try to toggle all switches/checkboxes to off except essential
                  const checkboxes = Array.from(document.querySelectorAll('input[type="checkbox"]'));
                  checkboxes.forEach(checkbox => {
                    if (checkbox.checked) {
                      // Don't uncheck if it's for essential cookies
                      const label = document.querySelector(`label[for="${checkbox.id}"]`) || 
                                   checkbox.closest('label') ||
                                   checkbox.parentElement;
                      
                      if (label && !label.textContent.toLowerCase().includes('essential') && 
                         !label.textContent.toLowerCase().includes('necessary')) {
                        checkbox.click();
                      }
                    }
                  });
                  
                  // Now find save button
                  const saveBtn = allButtons.find(btn => {
                    const text = btn.textContent.toLowerCase();
                    return text.includes('save') || text.includes('confirm');
                  });
                  
                  if (saveBtn) {
                    safeClick(saveBtn);
                    notifyBackgroundScript('Facebook', true);
                  } else {
                    notifyBackgroundScript('Facebook-Settings', false);
                  }
                }
              }, 1000); // Facebook's settings panel may take longer to load
            } else {
              notifyBackgroundScript('Facebook', true);
            }
            return;
          }
        } else {
          // User wants to accept cookies, find "Accept all" button
          const acceptButton = buttons.find(btn => {
            const text = btn.textContent.toLowerCase();
            return text.includes('accept') || 
                   text.includes('allow') || 
                   text.includes('ok') ||
                   text.includes('agree');
          });
          
          if (acceptButton) {
            safeClick(acceptButton);
            notifyBackgroundScript('Facebook', true);
            return;
          }
        }
      }
    }
    
    // Method 2: Fixed banner approach
    const possibleBanners = document.querySelectorAll('div[data-testid="cookie-policy-dialog"], div[data-testid="cookie-policy-banner"]');
    
    if (possibleBanners.length > 0) {
      const banner = possibleBanners[0];
      const buttons = Array.from(banner.querySelectorAll('button'));
      
      if (preferences.rejectAll) {
        // Look for buttons that reject or only allow essential cookies
        const rejectButtons = buttons.filter(btn => {
          const text = btn.textContent.toLowerCase();
          return text.includes('essential') || 
                 text.includes('necessary') || 
                 text.includes('reject') || 
                 text.includes('decline') ||
                 text.includes('manage') ||
                 text.includes('settings');
        });
        
        if (rejectButtons.length > 0) {
          // Find the most likely reject button based on position and text
          const rejectBtn = rejectButtons.find(btn => btn.textContent.toLowerCase().includes('reject')) || 
                           rejectButtons[0];
          
          safeClick(rejectBtn);
          
          // If we clicked a "Manage" or "Settings" button, we need to find the "Reject All" option
          if (rejectBtn.textContent.toLowerCase().includes('manage') || 
              rejectBtn.textContent.toLowerCase().includes('settings')) {
            
            setTimeout(() => {
              // Find reject all button in the settings panel
              const allButtons = Array.from(document.querySelectorAll('button'));
              const rejectAllBtn = allButtons.find(btn => {
                const text = btn.textContent.toLowerCase();
                return text.includes('reject all') || 
                       text.includes('only essential') ||
                       text.includes('save');
              });
              
              if (rejectAllBtn) {
                safeClick(rejectAllBtn);
                notifyBackgroundScript('Facebook', true);
              } else {
                // Try to toggle all switches/checkboxes to off except essential
                const checkboxes = Array.from(document.querySelectorAll('input[type="checkbox"]'));
                checkboxes.forEach(checkbox => {
                  if (checkbox.checked) {
                    // Don't uncheck if it's for essential cookies
                    const label = document.querySelector(`label[for="${checkbox.id}"]`) || 
                                 checkbox.closest('label') ||
                                 checkbox.parentElement;
                    
                    if (label && !label.textContent.toLowerCase().includes('essential') && 
                       !label.textContent.toLowerCase().includes('necessary')) {
                      checkbox.click();
                    }
                  }
                });
                
                // Now find save button
                const saveBtn = allButtons.find(btn => {
                  const text = btn.textContent.toLowerCase();
                  return text.includes('save') || text.includes('confirm');
                });
                
                if (saveBtn) {
                  safeClick(saveBtn);
                  notifyBackgroundScript('Facebook', true);
                } else {
                  notifyBackgroundScript('Facebook-Settings', false);
                }
              }
            }, 1000); // Facebook's settings panel may take longer to load
          } else {
            notifyBackgroundScript('Facebook', true);
          }
          return;
        }
      } else {
        // User wants to accept cookies, find "Accept all" button
        const acceptButton = buttons.find(btn => {
          const text = btn.textContent.toLowerCase();
          return text.includes('accept') || 
                 text.includes('allow') || 
                 text.includes('ok') ||
                 text.includes('agree');
        });
        
        if (acceptButton) {
          safeClick(acceptButton);
          notifyBackgroundScript('Facebook', true);
          return;
        }
      }
    }
    
    // Method 3: Handle older style banner
    const fixedBanners = Array.from(document.querySelectorAll('div')).filter(div => {
      const computedStyle = window.getComputedStyle(div);
      return (computedStyle.position === 'fixed' || computedStyle.position === 'sticky') && 
             computedStyle.bottom === '0px' &&
             div.textContent.toLowerCase().includes('cookie');
    });
    
    if (fixedBanners.length > 0) {
      const banner = fixedBanners[0];
      const buttons = Array.from(banner.querySelectorAll('button, a.button, [role="button"]'));
      
      if (preferences.rejectAll) {
        // Look for buttons that reject or only allow essential cookies
        const rejectButton = buttons.find(btn => {
          const text = btn.textContent.toLowerCase();
          return text.includes('essential') || 
                 text.includes('necessary') || 
                 text.includes('reject') || 
                 text.includes('decline') ||
                 text.includes('manage') ||
                 text.includes('settings');
        });
        
        if (rejectButton) {
          safeClick(rejectButton);
          notifyBackgroundScript('Facebook-Legacy', true);
          return;
        }
      } else {
        // User wants to accept cookies, find "Accept all" button
        const acceptButton = buttons.find(btn => {
          const text = btn.textContent.toLowerCase();
          return text.includes('accept') || 
                 text.includes('allow') || 
                 text.includes('ok') ||
                 text.includes('agree');
        });
        
        if (acceptButton) {
          safeClick(acceptButton);
          notifyBackgroundScript('Facebook-Legacy', true);
          return;
        }
      }
    }
    
    // If we get here, we couldn't find buttons to handle consent
    notifyBackgroundScript('Facebook', false);
  } catch (error) {
    console.log(`Cookie Away: Error handling Facebook cookie banner - ${error.message}`);
    notifyBackgroundScript('Facebook-Error', false);
  }
}

/**
 * Handle Facebook cookie policy/settings pages specifically
 * These pages appear when Facebook redirects users from the main site
 * to a dedicated cookie management page
 */
function handleFacebookCookiePolicyPage(preferences) {
  try {
    console.log('Cookie Away: Handling Facebook cookie policy page');
    
    // First check if there are specific cookie management forms or sections
    // Method 1: Look for cookie preferences form
    const forms = document.querySelectorAll('form');
    for (const form of forms) {
      if (form.textContent.toLowerCase().includes('cookie') || 
          form.textContent.toLowerCase().includes('privacy') ||
          form.id.toLowerCase().includes('cookie') ||
          form.className.toLowerCase().includes('cookie')) {
        
        // We found a cookie management form
        if (preferences.rejectAll) {
          // Try to find checkboxes and uncheck them
          const checkboxes = form.querySelectorAll('input[type="checkbox"], input[type="radio"]');
          
          // First pass: Get essential/necessary checkboxes
          const essentialCheckboxes = Array.from(checkboxes).filter(checkbox => {
            const parentText = checkbox.parentElement?.textContent.toLowerCase() || '';
            return parentText.includes('essential') || parentText.includes('necessary');
          });
          
          // Handle other checkboxes based on preferences
          checkboxes.forEach(checkbox => {
            if (essentialCheckboxes.includes(checkbox)) {
              // This is an essential cookie checkbox, ensure it's checked
              if (!checkbox.checked && checkbox.type !== 'radio') {
                safeClick(checkbox);
              }
            } else {
              // Non-essential checkbox, uncheck it
              if (checkbox.checked) {
                safeClick(checkbox);
              }
            }
          });
          
          // Look for a save/submit button
          const submitButtons = Array.from(form.querySelectorAll('button[type="submit"], input[type="submit"], button, [role="button"]'));
          const saveButton = submitButtons.find(btn => {
            const text = btn.textContent.toLowerCase();
            return text.includes('save') || 
                  text.includes('confirm') || 
                  text.includes('submit') ||
                  text.includes('continue');
          });
          
          if (saveButton) {
            safeClick(saveButton);
            notifyBackgroundScript('Facebook-Policy', true);
            return true;
          }
        } else {
          // User wants to accept all, look for "Accept All" button
          const buttons = Array.from(form.querySelectorAll('button, [role="button"]'));
          const acceptAllButton = buttons.find(btn => {
            const text = btn.textContent.toLowerCase();
            return text.includes('accept all') || 
                  text.includes('allow all');
          });
          
          if (acceptAllButton) {
            safeClick(acceptAllButton);
            notifyBackgroundScript('Facebook-Policy', true);
            return true;
          }
          
          // If no "Accept All" button, check all checkboxes
          const checkboxes = form.querySelectorAll('input[type="checkbox"]');
          checkboxes.forEach(checkbox => {
            if (!checkbox.checked) {
              safeClick(checkbox);
            }
          });
          
          // Look for a save/submit button
          const submitButtons = Array.from(form.querySelectorAll('button[type="submit"], input[type="submit"], button, [role="button"]'));
          const saveButton = submitButtons.find(btn => {
            const text = btn.textContent.toLowerCase();
            return text.includes('save') || 
                  text.includes('confirm') || 
                  text.includes('submit') ||
                  text.includes('continue');
          });
          
          if (saveButton) {
            safeClick(saveButton);
            notifyBackgroundScript('Facebook-Policy', true);
            return true;
          }
        }
      }
    }
    
    // Method 2: Look for specific buttons/sections on the policy page
    const allButtons = Array.from(document.querySelectorAll('button, [role="button"], a.button, a[href*="cookie"]'));
    
    if (preferences.rejectAll) {
      // Look for buttons that suggest rejecting non-essential cookies
      const rejectButtons = allButtons.filter(btn => {
        const text = btn.textContent.toLowerCase();
        return (text.includes('essential') && text.includes('cookies')) || 
               text.includes('reject all') || 
               text.includes('necessary only') ||
               text.includes('required only') ||
               (text.includes('manage') && text.includes('cookie')) ||
               (text.includes('cookie') && text.includes('settings'));
      });
      
      if (rejectButtons.length > 0) {
        // Sort buttons by specificity, preferring "reject all" type buttons
        const sortedButtons = rejectButtons.sort((a, b) => {
          const textA = a.textContent.toLowerCase();
          const textB = b.textContent.toLowerCase();
          
          // Prioritize buttons with "reject all" text
          if (textA.includes('reject all') && !textB.includes('reject all')) return -1;
          if (!textA.includes('reject all') && textB.includes('reject all')) return 1;
          
          // Next priority: "necessary only"
          if (textA.includes('necessary only') && !textB.includes('necessary only')) return -1;
          if (!textA.includes('necessary only') && textB.includes('necessary only')) return 1;
          
          return 0;
        });
        
        safeClick(sortedButtons[0]);
        
        // If we clicked a "manage" or "settings" button, we need to wait for the panel to load
        // and then handle that panel
        if (sortedButtons[0].textContent.toLowerCase().includes('manage') || 
            sortedButtons[0].textContent.toLowerCase().includes('settings')) {
          
          setTimeout(() => {
            // After clicking the settings button, look for checkboxes to toggle
            const checkboxes = Array.from(document.querySelectorAll('input[type="checkbox"]'));
            
            // Process checkboxes: keep essential on, turn others off
            checkboxes.forEach(checkbox => {
              if (checkbox.checked) {
                // Don't uncheck if it's for essential cookies
                const label = document.querySelector(`label[for="${checkbox.id}"]`) || 
                            checkbox.closest('label') ||
                            checkbox.parentElement;
                
                if (label && !label.textContent.toLowerCase().includes('essential') && 
                  !label.textContent.toLowerCase().includes('necessary')) {
                  checkbox.click();
                }
              }
            });
            
            // Find and click save button
            const saveButtons = Array.from(document.querySelectorAll('button, [role="button"]')).filter(btn => {
              const text = btn.textContent.toLowerCase();
              return text.includes('save') || 
                    text.includes('confirm') || 
                    text.includes('apply');
            });
            
            if (saveButtons.length > 0) {
              safeClick(saveButtons[0]);
              notifyBackgroundScript('Facebook-Policy-Settings', true);
            } else {
              notifyBackgroundScript('Facebook-Policy-Settings', false);
            }
          }, 1200); // Give more time for Facebook's panel to load
          
          return true;
        }
        
        notifyBackgroundScript('Facebook-Policy', true);
        return true;
      }
    } else {
      // User wants to accept all cookies
      const acceptButtons = allButtons.filter(btn => {
        const text = btn.textContent.toLowerCase();
        return text.includes('accept all') || 
              text.includes('allow all') ||
              text.includes('agree') ||
              text.includes('accept cookies');
      });
      
      if (acceptButtons.length > 0) {
        safeClick(acceptButtons[0]);
        notifyBackgroundScript('Facebook-Policy', true);
        return true;
      }
    }
    
    // Method 3: More generic approach - just try to find anything related to cookies
    // and make a best guess at the right buttons to click
    const cookieRelatedElements = Array.from(document.querySelectorAll('*')).filter(el => {
      return el.textContent.toLowerCase().includes('cookie') && 
            (el.tagName === 'BUTTON' || 
             el.tagName === 'A' || 
             el.getAttribute('role') === 'button' ||
             el.tagName === 'DIV' && el.querySelector('button, a, [role="button"]'));
    });
    
    if (cookieRelatedElements.length > 0) {
      for (const element of cookieRelatedElements) {
        const buttons = element.tagName === 'BUTTON' || 
                        element.tagName === 'A' || 
                        element.getAttribute('role') === 'button' ? 
                        [element] : 
                        Array.from(element.querySelectorAll('button, a, [role="button"]'));
        
        if (buttons.length > 0) {
          if (preferences.rejectAll) {
            // Try to find a reject/necessary-only button
            const rejectButton = buttons.find(btn => {
              const text = btn.textContent.toLowerCase();
              return text.includes('reject') || 
                    text.includes('essential only') || 
                    text.includes('necessary only');
            });
            
            if (rejectButton) {
              safeClick(rejectButton);
              notifyBackgroundScript('Facebook-Policy-Generic', true);
              return true;
            }
          } else {
            // Try to find an accept button
            const acceptButton = buttons.find(btn => {
              const text = btn.textContent.toLowerCase();
              return text.includes('accept all') || 
                    text.includes('allow all');
            });
            
            if (acceptButton) {
              safeClick(acceptButton);
              notifyBackgroundScript('Facebook-Policy-Generic', true);
              return true;
            }
          }
        }
      }
    }
    
    // If we got here, we couldn't handle the policy page
    console.log('Cookie Away: Could not find appropriate controls on Facebook cookie policy page');
    notifyBackgroundScript('Facebook-Policy', false);
    return false;
  } catch (error) {
    console.log(`Cookie Away: Error handling Facebook cookie policy page - ${error.message}`);
    notifyBackgroundScript('Facebook-Policy-Error', false);
    return false;
  }
}