/**
 * Generic cookie banner handler for Cookie Away
 * 
 * This is a fallback handler that attempts to identify and interact with 
 * cookie banners that aren't recognized by other specific handlers.
 * 
 * IMPORTANT: This handler follows strict safety rules to prevent unintended behavior:
 * 1. Only interacts with elements that look like cookie consent elements
 * 2. Avoids clicking elements that might open new tabs/windows
 * 3. Limits interaction depth to avoid cascading effects
 * 4. Takes snapshots of unknown banners for future analysis
 */

// Common cookie-related terms used to identify banners and buttons
const COOKIE_TERMS = [
  'cookie', 'cookies', 'consent', 'gdpr', 'privacy', 
  'tracking', 'data collection', 'accept', 'agree', 
  'ccpa', 'manage preferences', 'política de cookies',
  'cookiebeleid', 'cookie policy', 'cookie consent',
  'cookie settings', 'cookie preferences'
];

// Maximum depth to search for elements (for safety)
const MAX_SEARCH_DEPTH = 5;

// Detect unknown cookie banners using common patterns
function detectGenericCookieBanner() {
  try {
    // First, check if any specific handlers already detected something
    if (window.cookieResponseManagerDetected) {
      return false;
    }
    
    // Look for common cookie banner selectors
    const possibleBanners = document.querySelectorAll(
      '[class*="cookie"], [class*="consent"], [class*="privacy"], ' +
      '[id*="cookie"], [id*="consent"], [id*="privacy"], ' +
      '[class*="gdpr"], [id*="gdpr"], ' +
      '[class*="ccpa"], [id*="ccpa"], ' +
      '[class*="banner"], [id*="banner"], ' +
      '[aria-label*="cookie"], [aria-labelledby*="cookie"], ' +
      '[role="dialog"], [role="alertdialog"]'
    );
    
    // Filter to find likely cookie banners
    for (const element of possibleBanners) {
      if (!isElementVisible(element)) continue;
      
      // Check if the element or its children contain cookie-related text
      const elementText = element.textContent.toLowerCase();
      if (COOKIE_TERMS.some(term => elementText.includes(term))) {
        // Take a snapshot for analysis if it's appropriate to report
        captureUnknownBanner(element);
        return true;
      }
    }
    
    // If we didn't find clear cookie banners, check for fixed position elements
    // that might be banners (these are common for cookie notices)
    const fixedElements = Array.from(document.querySelectorAll('*')).filter(el => {
      const style = window.getComputedStyle(el);
      return (style.position === 'fixed' || style.position === 'sticky') && 
             isElementVisible(el) &&
             (el.offsetWidth > 200) &&
             (el.offsetHeight > 50);
    });
    
    for (const element of fixedElements) {
      const elementText = element.textContent.toLowerCase();
      if (COOKIE_TERMS.some(term => elementText.includes(term))) {
        captureUnknownBanner(element);
        return true;
      }
    }
    
    return false;
  } catch (error) {
    console.log(`Cookie Away: Error in generic detection - ${error.message}`);
    return false;
  }
}

// Blocklist of domains with known issues
const PROBLEMATIC_DOMAINS = [
  'dbschenker.com',
  'european-union.europa.eu',
  'europa.eu'
];

// Safely handle an unknown cookie banner
function handleGenericCookieBanner(preferences) {
  try {
    // Check if current domain is in the blocklist
    const currentHostname = window.location.hostname;
    if (PROBLEMATIC_DOMAINS.some(domain => currentHostname.includes(domain))) {
      console.log(`Cookie Away: Domain ${currentHostname} is in blocklist, skipping generic handling`);
      notifyBackgroundScript('Generic-Blocklisted', false);
      return;
    }
    
    // Add a flag to prevent repeated handling of the same page
    if (window.cookieResponseManagerHandled) {
      console.log('Cookie Away: Already handled this page, skipping to prevent loops');
      return;
    }
    
    // Set flag to indicate we've handled this page
    window.cookieResponseManagerHandled = true;
    
    // Find the most likely cookie banner
    const banners = document.querySelectorAll(
      '[class*="cookie"], [class*="consent"], [class*="privacy"], ' +
      '[id*="cookie"], [id*="consent"], [id*="privacy"], ' +
      '[class*="gdpr"], [id*="gdpr"], ' +
      '[class*="ccpa"], [id*="ccpa"], ' +
      '[class*="banner"], [id*="banner"], ' +
      '[aria-label*="cookie"], [aria-labelledby*="cookie"], ' +
      '[role="dialog"], [role="alertdialog"]'
    );
    
    let bannerElement = null;
    
    for (const element of banners) {
      if (isElementVisible(element) && 
          COOKIE_TERMS.some(term => element.textContent.toLowerCase().includes(term))) {
        bannerElement = element;
        break;
      }
    }
    
    // If no cookie banner is found, check fixed position elements
    if (!bannerElement) {
      // Limit the number of elements we check to avoid performance issues
      const fixedElements = Array.from(document.querySelectorAll('*'))
        .filter(el => {
          const style = window.getComputedStyle(el);
          return (style.position === 'fixed' || style.position === 'sticky') && 
                 isElementVisible(el) &&
                 (el.offsetWidth > 200) &&
                 (el.offsetHeight > 50);
        })
        .slice(0, 20); // Limit to 20 elements to prevent performance issues
      
      for (const element of fixedElements) {
        if (COOKIE_TERMS.some(term => element.textContent.toLowerCase().includes(term))) {
          bannerElement = element;
          break;
        }
      }
    }
    
    if (!bannerElement) {
      console.log('Cookie Away: Generic handler could not find a cookie banner');
      notifyBackgroundScript('Generic', false);
      return;
    }
    
    // Additional verification - check if the element actually contains text that suggests it's a cookie banner
    const bannerText = bannerElement.textContent.toLowerCase();
    const hasCookieReferences = COOKIE_TERMS.filter(term => bannerText.includes(term)).length >= 2;
    
    if (!hasCookieReferences) {
      console.log('Cookie Away: Element does not appear to be a cookie banner (insufficient cookie terms)');
      notifyBackgroundScript('Generic-NotBanner', false);
      return;
    }
    
    let buttonToClick = null;
    
    // Handle based on user preferences
    if (preferences.acceptAll) {
      // Look for accept/allow buttons
      buttonToClick = findButtonByText(bannerElement, [
        'accept all', 'accept cookies', 'allow all', 'agree', 'agree to all', 
        'i agree', 'ok', 'allow cookies', 'consent', 'accept'
      ]);
    } 
    else if (preferences.rejectAll) {
      // Look for reject/decline buttons
      buttonToClick = findButtonByText(bannerElement, [
        'reject all', 'decline', 'decline all', 'reject cookies', 
        'necessary only', 'only necessary', 'refuse', 'no, thanks', 
        'no thanks', 'reject', 'opt out', 'deny', 'decline cookies',
        'continue without accepting', 'continue without agreeing',
        'i do not accept', 'disagree', 'i disagree', 'later',
        'not now', 'without accepting', 'not accept',
        // German variations
        'nur notwendige', 'ablehnen', 'alle ablehnen', 'nicht akzeptieren',
        'erforderliche', 'nur erforderlich',
        // Spanish variations
        'rechazar', 'rechazar todo', 'solo necesarias', 'no acepto',
        // French variations
        'refuser', 'refuser tout', 'uniquement nécessaires', 'je refuse',
        // Italian variations
        'rifiuta', 'rifiuta tutto', 'solo necessari', 'non accetto',
        // Swedish variations 
        'neka', 'avvisa alla', 'endast nödvändiga', 'jag accepterar inte'
      ]);
      
      // If no reject button is found, look for settings/customize button
      if (!buttonToClick) {
        const customizeButton = findButtonByText(bannerElement, [
          'customize', 'settings', 'preferences', 'configure', 'manage', 
          'privacy settings', 'cookie settings', 'more options'
        ]);
        
        if (customizeButton) {
          // Click the customize button to open settings panel
          const clickResult = safeClick(customizeButton);
          
          if (!clickResult) {
            console.log('Cookie Away: Failed to click customize button');
            notifyBackgroundScript('Generic-ClickFailed', false);
            return;
          }
          
          // Wait for settings panel to appear
          setTimeout(() => {
            // Look for checkboxes to uncheck everything except necessary
            const checkboxesToggled = toggleCheckboxesBasedOnPreferences(document, preferences, false);
            
            // Look for a save/confirm button
            const saveButton = findButtonByText(document.body, [
              'save', 'confirm', 'apply', 'save settings', 'confirm choices',
              'save preferences', 'submit', 'save changes'
            ]);
            
            if (saveButton) {
              safeClick(saveButton);
              notifyBackgroundScript('Generic', true);
            } else {
              console.log('Cookie Away: No save button found after toggling checkboxes');
              notifyBackgroundScript('Generic-NoSaveButton', checkboxesToggled);
            }
          }, 800); // Increased timeout for slow-loading panels
          
          return;
        }
      }
    } 
    else {
      // Custom preferences - try to find settings button
      const customizeButton = findButtonByText(bannerElement, [
        'customize', 'settings', 'preferences', 'configure', 'manage', 
        'privacy settings', 'cookie settings', 'more options', 'manage consent'
      ]);
      
      if (customizeButton) {
        // Click the customize button to open settings panel
        const clickResult = safeClick(customizeButton);
        
        if (!clickResult) {
          console.log('Cookie Away: Failed to click customize button');
          notifyBackgroundScript('Generic-ClickFailed', false);
          return;
        }
        
        // Wait for settings panel to appear
        setTimeout(() => {
          // Apply preferences to checkboxes
          const checkboxesToggled = toggleCheckboxesBasedOnPreferences(document, preferences, true);
          
          // Look for a save/confirm button
          const saveButton = findButtonByText(document.body, [
            'save', 'confirm', 'apply', 'save settings', 'confirm choices',
            'save preferences', 'submit', 'save changes'
          ]);
          
          if (saveButton) {
            safeClick(saveButton);
            notifyBackgroundScript('Generic', true);
          } else {
            console.log('Cookie Away: No save button found after toggling checkboxes');
            notifyBackgroundScript('Generic-NoSaveButton', checkboxesToggled);
          }
        }, 800); // Increased timeout for slow-loading panels
        
        return;
      }
      
      // If no customize button found, fall back to accept all
      buttonToClick = findButtonByText(bannerElement, [
        'accept all', 'accept cookies', 'allow all', 'agree', 'agree to all', 
        'i agree', 'ok', 'allow cookies', 'consent', 'accept'
      ]);
    }
    
    // Click the identified button
    if (buttonToClick) {
      const clickResult = safeClick(buttonToClick);
      
      if (clickResult) {
        console.log('Cookie Away: Successfully clicked button');
        notifyBackgroundScript('Generic', true);
      } else {
        console.log('Cookie Away: Failed to click button');
        notifyBackgroundScript('Generic-ClickFailed', false);
      }
    } else {
      console.log('Cookie Away: Generic handler could not find a button to click');
      notifyBackgroundScript('Generic-NoButton', false);
    }
  } catch (error) {
    console.log(`Cookie Away: Error in generic handler - ${error.message}`);
    notifyBackgroundScript('Generic-Error', false);
  }
}

// Helper function to toggle checkboxes based on user preferences
function toggleCheckboxesBasedOnPreferences(rootElement, preferences, respectPreferences) {
  try {
    // Find all checkbox inputs
    const checkboxes = rootElement.querySelectorAll('input[type="checkbox"]');
    
    checkboxes.forEach(checkbox => {
      // Skip disabled checkboxes (usually necessary cookies)
      if (checkbox.disabled) return;
      
      // Get the label text for this checkbox
      const labelElement = checkbox.closest('label') || 
                           document.querySelector(`label[for="${checkbox.id}"]`) ||
                           checkbox.parentElement;
      
      if (!labelElement) return;
      
      const labelText = labelElement.textContent.toLowerCase();
      
      // Skip if this is for necessary/required cookies
      if (labelText.includes('necessary') || 
          labelText.includes('required') || 
          labelText.includes('essential') ||
          labelText.includes('technical') ||
          labelText.includes('functional')) {
        return;
      }
      
      if (respectPreferences) {
        // Set checkbox based on preferences
        if (
          (labelText.includes('analytic') && preferences.analytics) ||
          (labelText.includes('statistic') && preferences.analytics) ||
          (labelText.includes('performance') && preferences.analytics) ||
          (labelText.includes('functional') && preferences.functional) ||
          (labelText.includes('preference') && preferences.functional) ||
          (labelText.includes('personal') && preferences.functional) ||
          (labelText.includes('experience') && preferences.functional) ||
          (labelText.includes('advertis') && preferences.advertising) ||
          (labelText.includes('market') && preferences.advertising) ||
          (labelText.includes('target') && preferences.advertising) ||
          (labelText.includes('social') && preferences.advertising)
        ) {
          if (!checkbox.checked) checkbox.click();
        } else {
          if (checkbox.checked) checkbox.click();
        }
      } else {
        // Reject all: uncheck everything except necessary cookies
        if (checkbox.checked) checkbox.click();
      }
    });
  } catch (error) {
    console.log(`Cookie Away: Error in checkbox handling - ${error.message}`);
  }
}

// Capture a snapshot of an unknown banner for analysis
function captureUnknownBanner(element) {
  // Don't capture if we've already seen this banner
  if (window.cookieResponseManagerCaptured) {
    return;
  }
  
  try {
    // Mark that we've captured a banner
    window.cookieResponseManagerCaptured = true;
    
    // Create a minimal snapshot of the HTML structure
    const snapshot = createElementSnapshot(element, 0);
    
    // Send to background script for storage
    chrome.runtime.sendMessage({
      action: 'reportUnhandledBanner',
      url: window.location.href,
      htmlSnapshot: snapshot
    });
  } catch (error) {
    console.log(`Cookie Away: Error capturing banner - ${error.message}`);
  }
}

// Create a simplified snapshot of an element's structure
function createElementSnapshot(element, depth) {
  // Safety check to prevent too deep recursion
  if (depth > MAX_SEARCH_DEPTH) {
    return { tagName: element.tagName, truncated: true };
  }
  
  const snapshot = {
    tagName: element.tagName,
    id: element.id || undefined,
    className: element.className || undefined,
    textContent: element.textContent ? element.textContent.substring(0, 100) : undefined,
    attributes: {}
  };
  
  // Capture relevant attributes
  ['role', 'aria-label', 'type', 'name'].forEach(attr => {
    if (element.hasAttribute(attr)) {
      snapshot.attributes[attr] = element.getAttribute(attr);
    }
  });
  
  // Only include children if below depth limit
  if (depth < MAX_SEARCH_DEPTH - 1) {
    snapshot.children = Array.from(element.children)
      .filter(child => isElementVisible(child))
      .slice(0, 10) // Limit to 10 children for safety
      .map(child => createElementSnapshot(child, depth + 1));
  }
  
  return snapshot;
}