/**
 * Google Cookie Consent Handler for Cookie Away
 * 
 * This handler specifically targets Google's cookie consent mechanism which is used across
 * Google's properties (google.com, youtube.com, etc.). Google uses a unique cookie consent flow
 * that often redirects to a separate cookie settings page.
 */

// Detects Google's cookie consent banner
function detectGoogleCookieBanner() {
  try {
    const hostname = window.location.hostname;
    
    // Check if we're on a Google domain
    if (!hostname.includes('google.') && !hostname.includes('youtube.com')) {
      return false;
    }
    
    // Look for Google's cookie consent dialog
    // Method 1: Dialog with "Accept all" and "Reject all" buttons
    const consentDialog = document.querySelector('div[aria-modal="true"][role="dialog"]');
    if (consentDialog) {
      const text = consentDialog.textContent.toLowerCase();
      if (text.includes('cookie') || text.includes('consent') || text.includes('privacy')) {
        return true;
      }
    }
    
    // Method 2: Bottom bar with "Accept all" and "Reject all" buttons
    const consentBar = document.querySelector('.QS5gu, .sy4vM');
    if (consentBar) {
      const text = consentBar.textContent.toLowerCase();
      if (text.includes('cookie') || text.includes('consent') || text.includes('privacy')) {
        return true;
      }
    }
    
    // Method 3: Check for redirect to consent.google.com
    if (hostname.includes('consent.google.com')) {
      return true;
    }
    
    // Method 4: Check for specific cookie notice on YouTube
    if (hostname.includes('youtube.com')) {
      const youtubeConsent = document.querySelector('tp-yt-paper-dialog');
      if (youtubeConsent && youtubeConsent.textContent.toLowerCase().includes('cookie')) {
        return true;
      }
    }
    
    return false;
  } catch (error) {
    console.log(`Cookie Away: Error detecting Google cookie banner - ${error.message}`);
    return false;
  }
}

// Handles Google's cookie consent banner
function handleGoogleCookieBanner(preferences) {
  try {
    const hostname = window.location.hostname;
    
    // Handle consent.google.com page differently
    if (hostname.includes('consent.google.com')) {
      return handleGoogleConsentPage(preferences);
    }
    
    // Handle YouTube consent differently
    if (hostname.includes('youtube.com')) {
      return handleYouTubeConsent(preferences);
    }
    
    // Standard Google domains
    // Look for the main consent dialog
    const consentDialog = document.querySelector('div[aria-modal="true"][role="dialog"]');
    if (consentDialog) {
      // Find buttons within the dialog
      const buttons = Array.from(consentDialog.querySelectorAll('button'));
      
      if (preferences.rejectAll) {
        // Look for "Reject all" button
        const rejectButton = buttons.find(btn => 
          btn.textContent.toLowerCase().includes('reject all') || 
          btn.textContent.toLowerCase().includes('deny') ||
          btn.textContent.toLowerCase().includes('customize'));
        
        if (rejectButton) {
          safeClick(rejectButton);
          notifyBackgroundScript('Google', true);
          return;
        }
        
        // If no reject button, look for "Customize" or "More options"
        const customizeButton = buttons.find(btn => 
          btn.textContent.toLowerCase().includes('customize') || 
          btn.textContent.toLowerCase().includes('more options'));
        
        if (customizeButton) {
          safeClick(customizeButton);
          
          // Wait for settings panel
          setTimeout(() => {
            // Find and click the reject/save button in the settings
            const saveButtons = Array.from(document.querySelectorAll('button'));
            const rejectOrSaveBtn = saveButtons.find(btn => 
              btn.textContent.toLowerCase().includes('confirm') || 
              btn.textContent.toLowerCase().includes('save') ||
              btn.textContent.toLowerCase().includes('reject all'));
            
            if (rejectOrSaveBtn) {
              safeClick(rejectOrSaveBtn);
              notifyBackgroundScript('Google', true);
            } else {
              notifyBackgroundScript('Google', false);
            }
          }, 800);
          return;
        }
      } else {
        // User wants to accept, find "Accept all" button
        const acceptButton = buttons.find(btn => 
          btn.textContent.toLowerCase().includes('accept all') || 
          btn.textContent.toLowerCase().includes('agree') || 
          btn.textContent.toLowerCase().includes('ok'));
        
        if (acceptButton) {
          safeClick(acceptButton);
          notifyBackgroundScript('Google', true);
          return;
        }
      }
    }
    
    // Check for bottom bar consent
    const consentBar = document.querySelector('.QS5gu, .sy4vM');
    if (consentBar) {
      const buttons = Array.from(consentBar.querySelectorAll('button, a.button'));
      
      if (preferences.rejectAll) {
        // Look for "Reject all" or "Customize" button
        const rejectButton = buttons.find(btn => 
          btn.textContent.toLowerCase().includes('reject all') || 
          btn.textContent.toLowerCase().includes('customize'));
        
        if (rejectButton) {
          safeClick(rejectButton);
          
          // If it was the "Customize" button, wait and click reject all
          if (rejectButton.textContent.toLowerCase().includes('customize')) {
            setTimeout(() => {
              const allButtons = Array.from(document.querySelectorAll('button'));
              const rejectAllButton = allButtons.find(btn => 
                btn.textContent.toLowerCase().includes('reject all') ||
                btn.textContent.toLowerCase().includes('confirm') ||
                btn.textContent.toLowerCase().includes('save'));
              
              if (rejectAllButton) {
                safeClick(rejectAllButton);
                notifyBackgroundScript('Google', true);
              } else {
                notifyBackgroundScript('Google', false);
              }
            }, 800);
          } else {
            notifyBackgroundScript('Google', true);
          }
          return;
        }
      } else {
        // User wants to accept, find "Accept all" button
        const acceptButton = buttons.find(btn => 
          btn.textContent.toLowerCase().includes('accept all') || 
          btn.textContent.toLowerCase().includes('agree') || 
          btn.textContent.toLowerCase().includes('ok'));
        
        if (acceptButton) {
          safeClick(acceptButton);
          notifyBackgroundScript('Google', true);
          return;
        }
      }
    }
    
    // If we get here, we couldn't find buttons to handle consent
    notifyBackgroundScript('Google', false);
  } catch (error) {
    console.log(`Cookie Away: Error handling Google cookie banner - ${error.message}`);
    notifyBackgroundScript('Google-Error', false);
  }
}

// Special handler for consent.google.com redirect page
function handleGoogleConsentPage(preferences) {
  try {
    // Find the form on the consent page
    const form = document.querySelector('form');
    if (!form) {
      notifyBackgroundScript('Google-ConsentPage', false);
      return;
    }
    
    if (preferences.rejectAll) {
      // Look for "Reject all" button
      const buttons = Array.from(document.querySelectorAll('button, input[type="submit"]'));
      const rejectButton = buttons.find(btn => 
        btn.textContent.toLowerCase().includes('reject') || 
        (btn.value && btn.value.toLowerCase().includes('reject')));
      
      if (rejectButton) {
        safeClick(rejectButton);
        notifyBackgroundScript('Google-ConsentPage', true);
        return;
      }
      
      // If no reject button, look for radio buttons to select "Off" for personalization
      const radioButtons = Array.from(document.querySelectorAll('input[type="radio"]'));
      let foundAndToggled = false;
      
      radioButtons.forEach((radio, index) => {
        // Every other radio button is typically the "off" option in Google's form
        if (index % 2 === 1) {
          radio.checked = true;
          foundAndToggled = true;
        }
      });
      
      if (foundAndToggled) {
        // Find the submit/save button
        const saveButton = buttons.find(btn => 
          btn.textContent.toLowerCase().includes('save') || 
          btn.textContent.toLowerCase().includes('confirm') ||
          (btn.value && btn.value.toLowerCase().includes('save')));
        
        if (saveButton) {
          safeClick(saveButton);
          notifyBackgroundScript('Google-ConsentPage', true);
          return;
        }
      }
    } else {
      // User wants to accept, find "Accept all" button
      const buttons = Array.from(document.querySelectorAll('button, input[type="submit"]'));
      const acceptButton = buttons.find(btn => 
        btn.textContent.toLowerCase().includes('accept') || 
        btn.textContent.toLowerCase().includes('agree') ||
        (btn.value && btn.value.toLowerCase().includes('accept')));
      
      if (acceptButton) {
        safeClick(acceptButton);
        notifyBackgroundScript('Google-ConsentPage', true);
        return;
      }
    }
    
    // If we get here, we couldn't handle the consent page
    notifyBackgroundScript('Google-ConsentPage', false);
  } catch (error) {
    console.log(`Cookie Away: Error handling Google consent page - ${error.message}`);
    notifyBackgroundScript('Google-ConsentPage-Error', false);
  }
}

// Special handler for YouTube's consent dialog
function handleYouTubeConsent(preferences) {
  try {
    // YouTube typically uses a paper dialog for consent
    const youtubeConsent = document.querySelector('tp-yt-paper-dialog');
    if (!youtubeConsent) {
      notifyBackgroundScript('Google-YouTube', false);
      return;
    }
    
    const buttons = Array.from(youtubeConsent.querySelectorAll('button, yt-button-renderer'));
    
    if (preferences.rejectAll) {
      // Look for "Reject all" button
      const rejectButton = buttons.find(btn => 
        btn.textContent.toLowerCase().includes('reject') || 
        btn.textContent.toLowerCase().includes('no, thanks') ||
        btn.textContent.toLowerCase().includes('customize'));
      
      if (rejectButton) {
        safeClick(rejectButton);
        
        // If it was the "Customize" button, wait and reject all
        if (rejectButton.textContent.toLowerCase().includes('customize')) {
          setTimeout(() => {
            const allButtons = Array.from(document.querySelectorAll('button, yt-button-renderer'));
            const rejectAllButton = allButtons.find(btn => 
              btn.textContent.toLowerCase().includes('reject all') ||
              btn.textContent.toLowerCase().includes('confirm') ||
              btn.textContent.toLowerCase().includes('save'));
            
            if (rejectAllButton) {
              safeClick(rejectAllButton);
              notifyBackgroundScript('Google-YouTube', true);
            } else {
              notifyBackgroundScript('Google-YouTube', false);
            }
          }, 800);
        } else {
          notifyBackgroundScript('Google-YouTube', true);
        }
        return;
      }
    } else {
      // User wants to accept, find "Accept all" button
      const acceptButton = buttons.find(btn => 
        btn.textContent.toLowerCase().includes('accept') || 
        btn.textContent.toLowerCase().includes('agree') || 
        btn.textContent.toLowerCase().includes('ok') ||
        btn.textContent.toLowerCase().includes('got it'));
      
      if (acceptButton) {
        safeClick(acceptButton);
        notifyBackgroundScript('Google-YouTube', true);
        return;
      }
    }
    
    // If we get here, we couldn't find buttons to handle YouTube consent
    notifyBackgroundScript('Google-YouTube', false);
  } catch (error) {
    console.log(`Cookie Away: Error handling YouTube consent - ${error.message}`);
    notifyBackgroundScript('Google-YouTube-Error', false);
  }
}