/**
 * Cookie Away - Iubenda Handler
 * 
 * This script provides detection and interaction functions for Iubenda consent dialogs
 */

// Detect if Iubenda is present on the page
function detectIubenda() {
  // Check for Iubenda script
  if (
    document.querySelector('script[src*="iubenda"]') ||
    document.querySelector('script[src*="cs.iubenda.com"]') ||
    window._iub ||
    document.querySelector('[id^="iubenda"]') ||
    document.querySelector('.iubenda-cs-container')
  ) {
    return true;
  }
  
  // Look for Iubenda DOM elements
  const iubendaElements = [
    '#iubenda-cs-banner',
    '.iubenda-cs-container',
    '.iubenda-cs-accept-btn',
    '.iubenda-cs-reject-btn',
    '.iubenda-cs-customize-btn',
    '[class*="iubenda"]'
  ];
  
  for (const selector of iubendaElements) {
    if (document.querySelector(selector)) {
      return true;
    }
  }
  
  return false;
}

// Handle Iubenda consent dialog based on user preferences
function handleIubenda(preferences) {
  // Wait for the Iubenda dialog to be fully loaded
  const waitForDialog = () => {
    // Find banner element
    const banner = document.querySelector('.iubenda-cs-container') || 
                  document.getElementById('iubenda-cs-banner');
    
    // If banner is not found or not visible, retry after a short delay
    if (!banner || !isElementVisible(banner)) {
      setTimeout(waitForDialog, 300);
      return;
    }
    
    // Look for key Iubenda buttons
    const customizeButton = document.querySelector('.iubenda-cs-customize-btn');
    const acceptAllButton = document.querySelector('.iubenda-cs-accept-btn');
    const rejectAllButton = document.querySelector('.iubenda-cs-reject-btn');
    
    // Decision logic based on user preferences
    if (preferences.necessary && !preferences.analytics && !preferences.marketing && !preferences.personalization) {
      // User wants only necessary cookies
      if (rejectAllButton && isElementVisible(rejectAllButton)) {
        safeClick(rejectAllButton);
        notifyBackgroundScript('Iubenda', true);
        return;
      }
      
      // If no reject all button, try to go to customize and uncheck all options
      if (customizeButton && isElementVisible(customizeButton)) {
        safeClick(customizeButton);
        
        // Wait for the customize panel to load
        setTimeout(() => {
          // Uncheck all non-necessary options
          const purposeInputs = document.querySelectorAll('.purposes-item input[type="checkbox"]');
          
          purposeInputs.forEach(input => {
            const purposeLabel = input.closest('.purposes-item')?.querySelector('.purposes-item-title');
            const isNecessary = purposeLabel && 
                              (purposeLabel.textContent.toLowerCase().includes('necessary') || 
                               purposeLabel.textContent.toLowerCase().includes('essential') ||
                               input.disabled); // Disabled checkboxes are usually necessary cookies
            
            if (!isNecessary && input.checked) {
              safeClick(input);
            }
          });
          
          // Save preferences
          const saveButton = document.querySelector('.iubenda-cs-customize-btn') || 
                            document.querySelector('.iub-btn-save-preferences');
          
          if (saveButton) {
            safeClick(saveButton);
            notifyBackgroundScript('Iubenda', true);
            return;
          }
          
          notifyBackgroundScript('Iubenda', false);
        }, 500);
        return;
      }
    } else if (preferences.necessary && preferences.analytics && preferences.marketing && preferences.personalization) {
      // User accepts all cookies
      if (acceptAllButton && isElementVisible(acceptAllButton)) {
        safeClick(acceptAllButton);
        notifyBackgroundScript('Iubenda', true);
        return;
      }
    } else {
      // User has custom preferences
      if (customizeButton && isElementVisible(customizeButton)) {
        safeClick(customizeButton);
        
        // Wait for the customize panel to load
        setTimeout(() => {
          // Map user preferences to Iubenda categories
          const purposeInputs = document.querySelectorAll('.purposes-item input[type="checkbox"]');
          
          purposeInputs.forEach(input => {
            if (input.disabled) return; // Skip necessary cookies that can't be changed
            
            const purposeLabel = input.closest('.purposes-item')?.querySelector('.purposes-item-title');
            if (!purposeLabel) return;
            
            const labelText = purposeLabel.textContent.toLowerCase();
            let shouldBeChecked = false;
            
            if (labelText.includes('necessary') || labelText.includes('essential') || labelText.includes('technical')) {
              shouldBeChecked = preferences.necessary;
            } else if (labelText.includes('analytics') || labelText.includes('statistical') || labelText.includes('performance')) {
              shouldBeChecked = preferences.analytics;
            } else if (labelText.includes('functional') || labelText.includes('preference') || labelText.includes('personaliz')) {
              shouldBeChecked = preferences.personalization;
            } else if (labelText.includes('marketing') || labelText.includes('advertising') || labelText.includes('targeting')) {
              shouldBeChecked = preferences.marketing;
            }
            
            // Update the checkbox if it doesn't match the user preference
            if (input.checked !== shouldBeChecked) {
              safeClick(input);
            }
          });
          
          // Save preferences
          const saveButton = document.querySelector('.iubenda-cs-customize-btn') || 
                            document.querySelector('.iub-btn-save-preferences');
          
          if (saveButton) {
            safeClick(saveButton);
            notifyBackgroundScript('Iubenda', true);
            return;
          }
          
          notifyBackgroundScript('Iubenda', false);
        }, 500);
        return;
      }
    }
    
    // If we couldn't handle the dialog properly
    notifyBackgroundScript('Iubenda', false);
  };
  
  // Start the process
  waitForDialog();
}
