/**
 * Cookie Away - OneTrust Handler
 * 
 * This script provides detection and interaction functions for OneTrust consent dialogs
 */

// Detect if OneTrust is present on the page
function detectOneTrust() {
  // Check for OneTrust script
  if (
    document.querySelector('script[src*="onetrust"]') ||
    document.querySelector('script[src*="otSDKStub"]') ||
    window.OneTrust ||
    typeof window.OptanonWrapper === 'function' ||
    document.getElementById('onetrust-banner-sdk') ||
    document.getElementById('onetrust-consent-sdk')
  ) {
    return true;
  }
  
  // Look for OneTrust DOM elements
  const oneTrustElements = [
    '#onetrust-banner-sdk',
    '#onetrust-consent-sdk',
    '#onetrust-pc-sdk',
    '.onetrust-pc-dark-filter',
    '[class*="onetrust"]',
    '#ot-sdk-btn'
  ];
  
  for (const selector of oneTrustElements) {
    if (document.querySelector(selector)) {
      return true;
    }
  }
  
  return false;
}

// Handle OneTrust consent dialog based on user preferences
function handleOneTrust(preferences) {
  // Wait for the OneTrust dialog to be fully loaded
  const waitForDialog = () => {
    // Find banner element
    const banner = document.getElementById('onetrust-banner-sdk') || 
                   document.getElementById('onetrust-consent-sdk');
    
    // If banner is not found or not visible, retry after a short delay
    if (!banner || !isElementVisible(banner)) {
      setTimeout(waitForDialog, 300);
      return;
    }
    
    // Look for key OneTrust buttons
    const settingsButton = document.getElementById('onetrust-pc-btn-handler');
    const acceptAllButton = document.getElementById('onetrust-accept-btn-handler');
    const rejectAllButton = document.getElementById('onetrust-reject-all-handler');
    
    // Decision logic based on user preferences
    if (preferences.necessary && !preferences.analytics && !preferences.marketing && !preferences.personalization) {
      // User wants only necessary cookies
      if (rejectAllButton && isElementVisible(rejectAllButton)) {
        safeClick(rejectAllButton);
        notifyBackgroundScript('OneTrust', true);
        return;
      }
      
      // If no reject all button is available, try to go to settings and customize
      if (settingsButton && isElementVisible(settingsButton)) {
        safeClick(settingsButton);
        
        // Wait for settings panel to load
        setTimeout(() => {
          // Look for the "reject all" button in the settings panel
          const pcRejectAllButton = document.getElementById('onetrust-reject-all-handler') || 
                                   document.querySelector('.ot-pc-refuse-all-handler');
          
          if (pcRejectAllButton && isElementVisible(pcRejectAllButton)) {
            safeClick(pcRejectAllButton);
            notifyBackgroundScript('OneTrust', true);
            return;
          }
          
          // If no reject button found, try to uncheck all toggle switches except necessary
          const toggles = document.querySelectorAll('.ot-tgl');
          let hasCustomized = false;
          
          toggles.forEach(toggle => {
            const toggleLabel = toggle.closest('.ot-toggle-group')?.querySelector('.ot-always-active');
            const isNecessary = toggleLabel && toggleLabel.textContent.trim().toLowerCase() === 'always active';
            
            // If it's not a necessary category, disable it
            if (!isNecessary) {
              const input = toggle.querySelector('input[type="checkbox"]');
              if (input && input.checked) {
                safeClick(input);
                hasCustomized = true;
              }
            }
          });
          
          // Save preferences
          if (hasCustomized) {
            const saveButton = document.getElementById('accept-recommended-btn-handler') || 
                              document.querySelector('.save-preference-btn-handler');
            if (saveButton) {
              safeClick(saveButton);
              notifyBackgroundScript('OneTrust', true);
              return;
            }
          }
          
          notifyBackgroundScript('OneTrust', false);
        }, 500);
        return;
      }
    } else if (preferences.necessary && preferences.analytics && preferences.marketing && preferences.personalization) {
      // User accepts all cookies
      if (acceptAllButton && isElementVisible(acceptAllButton)) {
        safeClick(acceptAllButton);
        notifyBackgroundScript('OneTrust', true);
        return;
      }
    } else {
      // User has custom preferences
      if (settingsButton && isElementVisible(settingsButton)) {
        safeClick(settingsButton);
        
        // Wait for settings panel to load
        setTimeout(() => {
          // Map user preferences to OneTrust categories
          // We'll look at both the toggles and their labels to determine category
          const toggles = document.querySelectorAll('.ot-tgl');
          let hasCustomized = false;
          
          toggles.forEach(toggle => {
            const toggleContainer = toggle.closest('.ot-toggle-group');
            if (!toggleContainer) return;
            
            const labelElement = toggleContainer.querySelector('.ot-cat-header') || 
                                toggleContainer.querySelector('.ot-toggle-group label');
            
            if (!labelElement) return;
            
            const labelText = labelElement.textContent.trim().toLowerCase();
            const input = toggle.querySelector('input[type="checkbox"]');
            
            if (!input) return;
            
            // Determine category based on label text
            let shouldBeChecked = false;
            
            if (labelText.includes('strict') || labelText.includes('necessary') || labelText.includes('essential')) {
              shouldBeChecked = preferences.necessary;
            } else if (labelText.includes('performance') || labelText.includes('analytics') || labelText.includes('statistics')) {
              shouldBeChecked = preferences.analytics;
            } else if (labelText.includes('functional') || labelText.includes('preference') || labelText.includes('personaliz')) {
              shouldBeChecked = preferences.personalization;
            } else if (labelText.includes('target') || labelText.includes('marketing') || labelText.includes('advertising')) {
              shouldBeChecked = preferences.marketing;
            }
            
            // Update the toggle if it doesn't match the user preference
            if (input.checked !== shouldBeChecked) {
              safeClick(input);
              hasCustomized = true;
            }
          });
          
          // Save preferences
          if (hasCustomized) {
            const saveButton = document.getElementById('accept-recommended-btn-handler') || 
                              document.querySelector('.save-preference-btn-handler');
            if (saveButton) {
              safeClick(saveButton);
              notifyBackgroundScript('OneTrust', true);
              return;
            }
          }
          
          notifyBackgroundScript('OneTrust', false);
        }, 500);
        return;
      }
    }
    
    // If we couldn't handle the dialog properly
    notifyBackgroundScript('OneTrust', false);
  };
  
  // Start the process
  waitForDialog();
}
