/**
 * Secure Privacy handler for Cookie Away
 */

function detectSecurePrivacy() {
  try {
    // Look for Secure Privacy specific elements
    return (
      document.querySelector('.sp-consent-modal') ||
      document.querySelector('[id^="sp-consent"]') ||
      document.querySelector('[class^="sp-consent"]') ||
      document.querySelector('.sp-message-container') ||
      typeof window.SPConsent !== 'undefined' ||
      typeof window.__sp_scconsent !== 'undefined'
    );
  } catch (error) {
    console.log(`Cookie Away: Error detecting Secure Privacy - ${error.message}`);
    return false;
  }
}

function handleSecurePrivacy(preferences) {
  try {
    // Get the Secure Privacy dialog
    const dialog = document.querySelector('.sp-consent-modal') || 
                  document.querySelector('.sp-message-container') ||
                  document.querySelector('[id^="sp-consent"]');
    
    if (!dialog) {
      console.log('Cookie Away: Secure Privacy dialog not found');
      notifyBackgroundScript('Secure Privacy', false);
      return;
    }
    
    // Find the appropriate buttons based on preferences
    let buttonToClick;
    
    // Accept all cookies
    if (preferences.acceptAll) {
      buttonToClick = findButtonByText(dialog, [
        'accept all', 'allow all', 'agree to all', 'accept cookies', 
        'allow cookies', 'consent to all', 'I accept', 'accept'
      ]) || dialog.querySelector('[data-role="accept-all"]');
    }
    // Reject non-essential cookies
    else if (preferences.rejectAll) {
      buttonToClick = findButtonByText(dialog, [
        'reject all', 'decline all', 'necessary only', 'reject non-essential',
        'reject cookies', 'deny', 'decline cookies', 'reject'
      ]) || dialog.querySelector('[data-role="reject-all"]');
    }
    // Apply specific preferences
    else {
      // First try to find a customize/settings button
      const customizeButton = findButtonByText(dialog, [
        'customize', 'settings', 'preferences', 'configure', 'manage', 'cookie settings',
        'manage preferences', 'consent preferences'
      ]) || dialog.querySelector('[data-role="configure"]');
      
      if (customizeButton) {
        safeClick(customizeButton);
        
        // Wait a moment for settings panel to appear
        setTimeout(() => {
          // Try to find the settings dialog
          const settingsDialog = document.querySelector('.sp-consent-modal__configview') ||
                              document.querySelector('[class*="sp-consent-modal__config"]');
          
          if (settingsDialog) {
            // Find the category toggles
            const toggles = settingsDialog.querySelectorAll('input[type="checkbox"]');
            
            // Loop through toggles and set preferences
            toggles.forEach(toggle => {
              if (toggle.disabled) return;
              
              const categoryLabel = toggle.closest('tr, div, li').textContent.toLowerCase();
              
              // Skip if necessary/required cookies (usually can't be toggled)
              if (categoryLabel.includes('necessary') || categoryLabel.includes('required') || categoryLabel.includes('essential')) {
                return;
              }
              
              // Set toggle according to preferences
              if (
                (categoryLabel.includes('analytic') && preferences.analytics) ||
                (categoryLabel.includes('statistic') && preferences.analytics) ||
                (categoryLabel.includes('performance') && preferences.analytics) ||
                (categoryLabel.includes('functional') && preferences.functional) ||
                (categoryLabel.includes('preference') && preferences.functional) ||
                (categoryLabel.includes('advertis') && preferences.advertising) ||
                (categoryLabel.includes('market') && preferences.advertising) ||
                (categoryLabel.includes('target') && preferences.advertising)
              ) {
                if (!toggle.checked) toggle.click();
              } else {
                if (toggle.checked) toggle.click();
              }
            });
            
            // Find and click the save/confirm button
            const saveButton = findButtonByText(settingsDialog, [
              'save', 'confirm', 'apply', 'update', 'save preferences', 'save settings', 'submit',
              'save and close', 'save & close', 'confirm my choices'
            ]) || settingsDialog.querySelector('[data-role="save"]');
            
            if (saveButton) {
              safeClick(saveButton);
              notifyBackgroundScript('Secure Privacy', true);
            }
          }
        }, 500);
      }
    }
    
    if (buttonToClick) {
      safeClick(buttonToClick);
      notifyBackgroundScript('Secure Privacy', true);
    }
  } catch (error) {
    console.log(`Cookie Away: Error handling Secure Privacy - ${error.message}`);
    notifyBackgroundScript('Secure Privacy', false);
  }
}