/**
 * Simple Cookie Banner Handler for Cookie Away
 * 
 * This handler specifically targets very simple cookie banners that have a consistent
 * structure with just "Accept" and "Decline" buttons. These are common across many websites
 * and don't require complex interaction patterns.
 */

// Detects simple cookie banners
function detectSimpleCookieBanner() {
  try {
    // 1. Look for a fixed/sticky position element with cookie-related text and simple buttons
    const fixedElements = Array.from(document.querySelectorAll('*')).filter(el => {
      const style = window.getComputedStyle(el);
      return (style.position === 'fixed' || style.position === 'sticky') && 
             (style.bottom === '0px' || style.top === '0px') &&
             isElementVisible(el);
    });
    
    // Filter to elements with cookie-related text
    const cookieElements = fixedElements.filter(el => {
      const text = el.textContent.toLowerCase();
      return text.includes('cookie') || 
             text.includes('privacy') || 
             text.includes('consent') ||
             text.includes('gdpr') ||
             text.includes('we use') ||
             text.includes('we value');
    });
    
    // Check if any of these elements have simple accept/decline buttons
    for (const element of cookieElements) {
      // Look more broadly for clickable elements
      const buttons = element.querySelectorAll('button, a, [role="button"], .btn, input[type="button"], input[type="submit"], .cookie-button, [data-action], [data-consent], .consent-btn');
      
      if (buttons.length >= 1 && buttons.length <= 6) { // Allow up to 6 buttons now
        // Check if buttons contain simple accept/decline text
        const buttonTexts = Array.from(buttons).map(btn => (btn.textContent || '').toLowerCase().trim());
        
        const hasAcceptButton = buttonTexts.some(text => 
          text.includes('accept') || 
          text.includes('agree') || 
          text.includes('ok') || 
          text.includes('allow') ||
          text.includes('got it') ||
          text.includes('i accept') ||
          text.includes('allow cookies') ||
          text.includes('accept cookies') ||
          text.includes('continue') ||
          text.includes('understood') ||
          text.includes('approve') ||
          text.includes('consent') ||
          text.includes('accept all'));
        
        const hasDeclineButton = buttonTexts.some(text => 
          text.includes('decline') || 
          text.includes('reject') || 
          text.includes('reject all') ||
          text.includes('decline all') ||
          text.includes('opt out') ||
          text.includes('refuse') ||
          text.includes('disagree') ||
          text.includes('disable all') ||
          text.includes('only essential') ||
          text.includes('necessary only') ||
          text.includes('non-essential') ||
          text.includes('deny all') ||
          text.includes('optional') ||
          text.includes('do not consent') ||
          text.includes('i disagree') ||
          text.includes('refuse cookies') ||
          text.includes('no, thanks') ||
          text.includes('no thanks') ||
          text.includes('no, thank you') ||
          text.includes('customize') ||
          text.includes('settings') ||
          text.includes('reject cookies') ||
          text.includes('preferences') ||
          text.includes('cookie settings') ||
          text.includes('manage cookies') ||
          text.includes('cookie preferences') ||
          text.includes('no'));
        
        // Check if any button has text similar to "X" or "Close" for closing the banner
        const hasCloseButton = Array.from(buttons).some(btn => {
          return (btn.textContent === 'X' || 
                  btn.textContent === '×' || 
                  btn.textContent.toLowerCase().includes('close') ||
                  btn.classList.contains('close') ||
                  btn.classList.contains('dismiss') ||
                  btn.getAttribute('aria-label') === 'Close' ||
                  btn.getAttribute('data-dismiss') === 'alert');
        });
        
        // Also check for buttons with icons only (which might be close buttons)
        const hasIconButton = Array.from(buttons).some(btn => {
          return btn.textContent.trim() === '' && 
                (btn.querySelector('svg') || 
                 btn.querySelector('img') || 
                 btn.querySelector('i.fa') || 
                 btn.querySelector('i.icon'));
        });
        
        // If it has any of these button types, it's likely a simple banner
        if (hasAcceptButton || hasDeclineButton || hasCloseButton || hasIconButton) {
          console.log('Cookie Away: Simple banner detected with button types:', {
            hasAcceptButton, hasDeclineButton, hasCloseButton, hasIconButton,
            buttonTexts
          });
          return true;
        }
      }
    }
    
    // 2. Check for common simple banner patterns like the ones in the screenshots
    // Pattern: "We value your privacy" or "We use cookies" with simple buttons
    const simpleBanners = document.querySelectorAll('.cookie-banner, .cookie-notification, .cookie-notice, .cookie-consent, .privacy-notice, .privacy-banner');
    
    for (const banner of simpleBanners) {
      if (isElementVisible(banner)) {
        const text = banner.textContent.toLowerCase();
        
        if (text.includes('cookie') || text.includes('privacy') || text.includes('we value')) {
          const buttons = banner.querySelectorAll('button, a.button, [role="button"], .btn');
          
          if (buttons.length > 0 && buttons.length <= 4) {
            return true;
          }
        }
      }
    }
    
    return false;
  } catch (error) {
    console.log(`Cookie Away: Error detecting simple cookie banner - ${error.message}`);
    return false;
  }
}

// Handles simple cookie banners
function handleSimpleCookieBanner(preferences) {
  try {
    // Find all potential simple banners
    const potentialBanners = [];
    
    // 1. Fixed/sticky position elements
    const fixedElements = Array.from(document.querySelectorAll('*')).filter(el => {
      const style = window.getComputedStyle(el);
      return (style.position === 'fixed' || style.position === 'sticky') && 
             (style.bottom === '0px' || style.top === '0px') &&
             isElementVisible(el);
    });
    
    const cookieFixedElements = fixedElements.filter(el => {
      const text = el.textContent.toLowerCase();
      return text.includes('cookie') || 
             text.includes('privacy') || 
             text.includes('consent') ||
             text.includes('gdpr') ||
             text.includes('we use') ||
             text.includes('we value');
    });
    
    potentialBanners.push(...cookieFixedElements);
    
    // 2. Common banner selectors
    const simpleBanners = document.querySelectorAll('.cookie-banner, .cookie-notification, .cookie-notice, .cookie-consent, .privacy-notice, .privacy-banner');
    potentialBanners.push(...Array.from(simpleBanners).filter(isElementVisible));
    
    // Find the most likely banner (the one with buttons and cookie text)
    let bestBanner = null;
    let bestButtonCount = 0;
    
    for (const banner of potentialBanners) {
      // Use the same improved selector as in the detection function
      const buttons = banner.querySelectorAll('button, a, [role="button"], .btn, input[type="button"], input[type="submit"], .cookie-button, [data-action], [data-consent], .consent-btn');
      
      if (buttons.length > 0 && buttons.length <= 6) {
        const buttonTexts = Array.from(buttons).map(btn => (btn.textContent || '').toLowerCase().trim());
        
        // Use expanded list of accept button patterns
        const hasAcceptButton = buttonTexts.some(text => 
          text.includes('accept') || 
          text.includes('agree') || 
          text.includes('ok') || 
          text.includes('allow') ||
          text.includes('got it') ||
          text.includes('i accept') ||
          text.includes('allow cookies') ||
          text.includes('accept cookies') ||
          text.includes('continue') ||
          text.includes('understood') ||
          text.includes('approve') ||
          text.includes('consent') ||
          text.includes('accept all'));
        
        // Use expanded list of decline button patterns
        const hasDeclineButton = buttonTexts.some(text => 
          text.includes('decline') || 
          text.includes('reject') || 
          text.includes('reject all') ||
          text.includes('decline all') ||
          text.includes('opt out') ||
          text.includes('refuse') ||
          text.includes('disagree') ||
          text.includes('disable all') ||
          text.includes('only essential') ||
          text.includes('necessary only') ||
          text.includes('non-essential') ||
          text.includes('deny all') ||
          text.includes('optional') ||
          text.includes('do not consent') ||
          text.includes('i disagree') ||
          text.includes('refuse cookies') ||
          text.includes('no, thanks') ||
          text.includes('no thanks') ||
          text.includes('no, thank you') ||
          text.includes('customize') ||
          text.includes('settings') ||
          text.includes('reject cookies') ||
          text.includes('preferences') ||
          text.includes('cookie settings') ||
          text.includes('manage cookies') ||
          text.includes('cookie preferences') ||
          text.includes('no'));
        
        // Check for close buttons
        const hasCloseButton = Array.from(buttons).some(btn => {
          return (btn.textContent === 'X' || 
                  btn.textContent === '×' || 
                  btn.textContent.toLowerCase().includes('close') ||
                  btn.classList.contains('close') ||
                  btn.classList.contains('dismiss') ||
                  btn.getAttribute('aria-label') === 'Close' ||
                  btn.getAttribute('data-dismiss') === 'alert');
        });
        
        // Check for icon-only buttons
        const hasIconButton = Array.from(buttons).some(btn => {
          return btn.textContent.trim() === '' && 
                (btn.querySelector('svg') || 
                 btn.querySelector('img') || 
                 btn.querySelector('i.fa') || 
                 btn.querySelector('i.icon'));
        });
        
        if ((hasAcceptButton || hasDeclineButton || hasCloseButton || hasIconButton) && buttons.length > bestButtonCount) {
          bestBanner = banner;
          bestButtonCount = buttons.length;
          console.log('Cookie Away: Found potential banner with buttons:', buttonTexts);
        }
      }
    }
    
    if (!bestBanner) {
      console.log('Cookie Away: Could not find a suitable simple banner');
      notifyBackgroundScript('SimpleBanner', false);
      return;
    }
    
    // Now handle based on user preferences
    // Use broader selector to match our detection function
    const buttons = bestBanner.querySelectorAll('button, a, [role="button"], .btn, input[type="button"], input[type="submit"], .cookie-button, [data-action], [data-consent], .consent-btn');
    console.log('Cookie Away: Found', buttons.length, 'buttons in the banner');
    
    if (preferences.rejectAll) {
      // Look for reject/decline button with expanded patterns
      const declineButton = Array.from(buttons).find(btn => {
        const text = (btn.textContent || '').toLowerCase().trim();
        return text.includes('decline') || 
               text.includes('reject') || 
               text.includes('reject all') ||
               text.includes('decline all') ||
               text.includes('opt out') ||
               text.includes('refuse') ||
               text.includes('disagree') ||
               text.includes('disable all') ||
               text.includes('only essential') ||
               text.includes('necessary only') ||
               text.includes('non-essential') ||
               text.includes('deny all') ||
               text.includes('optional') ||
               text.includes('do not consent') ||
               text.includes('i disagree') ||
               text.includes('refuse cookies') ||
               text.includes('no, thanks') ||
               text.includes('no thanks') ||
               text.includes('no, thank you') ||
               text.includes('reject cookies') ||
               (text === 'no');
      });
      
      if (declineButton) {
        console.log('Cookie Away: Found decline button with text:', declineButton.textContent);
        safeClick(declineButton);
        notifyBackgroundScript('SimpleBanner', true);
        return;
      }
      
      // If no decline button, see if there's a customize/settings button
      const customizeButton = Array.from(buttons).find(btn => {
        const text = (btn.textContent || '').toLowerCase().trim();
        return text.includes('customize') || 
               text.includes('settings') || 
               text.includes('manage') ||
               text.includes('preferences') ||
               text.includes('cookie settings') ||
               text.includes('manage cookies') ||
               text.includes('cookie preferences');
      });
      
      if (customizeButton) {
        console.log('Cookie Away: Found customize button with text:', customizeButton.textContent);
        safeClick(customizeButton);
        
        // Wait for settings panel to appear
        setTimeout(() => {
          // Try to uncheck all non-essential checkboxes
          const checkboxes = document.querySelectorAll('input[type="checkbox"], input[type="radio"][value="off"], input[type="radio"][value="no"]');
          let toggledAny = false;
          
          console.log('Cookie Away: Found', checkboxes.length, 'potential checkboxes');
          
          checkboxes.forEach(checkbox => {
            const label = document.querySelector(`label[for="${checkbox.id}"]`) || 
                          checkbox.closest('label') ||
                          checkbox.parentElement;
            
            // Skip necessary/required cookies
            if (label && (
                label.textContent.toLowerCase().includes('necessary') || 
                label.textContent.toLowerCase().includes('essential') ||
                label.textContent.toLowerCase().includes('required') ||
                label.textContent.toLowerCase().includes('functional')
            )) {
              console.log('Cookie Away: Skipping necessary checkbox:', label.textContent);
              return;
            }
            
            // Check if this is a toggle type control
            const isToggle = checkbox.classList.contains('toggle') || 
                           checkbox.classList.contains('switch') ||
                           checkbox.parentElement?.classList.contains('toggle') ||
                           checkbox.parentElement?.classList.contains('switch');
            
            // For regular checkboxes: uncheck if checked
            // For toggles: they often work opposite, so treat carefully
            if ((checkbox.checked && !isToggle) || (!checkbox.checked && isToggle)) {
              console.log('Cookie Away: Toggling checkbox/toggle:', label?.textContent);
              checkbox.click();
              toggledAny = true;
            }
          });
          
          // Find save/confirm button
          const allButtons = document.querySelectorAll('button, a, [role="button"], .btn, input[type="button"], input[type="submit"], [data-action="save"], [data-action="confirm"]');
          const saveButton = Array.from(allButtons).find(btn => {
            const text = (btn.textContent || '').toLowerCase().trim();
            return text.includes('save') || 
                   text.includes('confirm') || 
                   text.includes('apply') ||
                   text.includes('update') ||
                   text.includes('submit') ||
                   text.includes('done');
          });
          
          if (saveButton) {
            console.log('Cookie Away: Found save button with text:', saveButton.textContent);
            safeClick(saveButton);
            notifyBackgroundScript('SimpleBanner', true);
          } else if (toggledAny) {
            // We toggled checkboxes but couldn't find a save button
            console.log('Cookie Away: Toggled preferences but could not find save button');
            notifyBackgroundScript('SimpleBanner-PartialSuccess', true);
          } else {
            console.log('Cookie Away: Could not find any toggleable preferences');
            notifyBackgroundScript('SimpleBanner-NoSettings', false);
          }
        }, 800);
        return;
      }
      
      // If no decline or customize button, look for close button as last resort
      const closeButton = Array.from(buttons).find(btn => {
        return (btn.textContent === 'X' || 
                btn.textContent === '×' || 
                btn.textContent.toLowerCase().includes('close') ||
                btn.classList.contains('close') ||
                btn.classList.contains('dismiss') ||
                btn.getAttribute('aria-label') === 'Close' ||
                btn.getAttribute('data-dismiss') === 'alert');
      });
      
      if (closeButton) {
        console.log('Cookie Away: Using close button as last resort');
        safeClick(closeButton);
        notifyBackgroundScript('SimpleBanner-Close', true);
        return;
      }
      
    } else {
      // Look for accept button with expanded patterns
      const acceptButton = Array.from(buttons).find(btn => {
        const text = (btn.textContent || '').toLowerCase().trim();
        return text.includes('accept') || 
               text.includes('agree') || 
               text.includes('ok') || 
               text.includes('allow') ||
               text.includes('got it') ||
               text.includes('i accept') ||
               text.includes('allow cookies') ||
               text.includes('accept cookies') ||
               text.includes('continue') ||
               text.includes('understood') ||
               text.includes('approve') ||
               text.includes('consent') ||
               text.includes('accept all');
      });
      
      if (acceptButton) {
        console.log('Cookie Away: Found accept button with text:', acceptButton.textContent);
        safeClick(acceptButton);
        notifyBackgroundScript('SimpleBanner', true);
        return;
      }
      
      // As a fallback for accept, try a close button too
      const closeButton = Array.from(buttons).find(btn => {
        return (btn.textContent === 'X' || 
                btn.textContent === '×' || 
                btn.textContent.toLowerCase().includes('close') ||
                btn.classList.contains('close') ||
                btn.classList.contains('dismiss') ||
                btn.getAttribute('aria-label') === 'Close' ||
                btn.getAttribute('data-dismiss') === 'alert');
      });
      
      if (closeButton) {
        console.log('Cookie Away: Using close button as fallback for accept');
        safeClick(closeButton);
        notifyBackgroundScript('SimpleBanner-Close', true);
        return;
      }
    }
    
    // If we reach here, we couldn't find appropriate buttons
    console.log('Cookie Away: Found simple banner but could not find appropriate buttons');
    notifyBackgroundScript('SimpleBanner-NoButtons', false);
  } catch (error) {
    console.log(`Cookie Away: Error handling simple cookie banner - ${error.message}`);
    notifyBackgroundScript('SimpleBanner-Error', false);
  }
}