/**
 * Tealium handler for Cookie Away
 */

function detectTealium() {
  try {
    // Look for Tealium specific elements
    return (
      document.querySelector('.tealium-modal') ||
      document.querySelector('.truste_overlay') ||
      document.querySelector('.truste_box_overlay') ||
      document.querySelector('[id^="consent_blackbar"]') ||
      document.querySelector('[class*="tealium-"]') ||
      typeof window.utag !== 'undefined' ||
      typeof window.tealium !== 'undefined' ||
      typeof window.tealiumCmpApi !== 'undefined'
    );
  } catch (error) {
    console.log(`Cookie Away: Error detecting Tealium - ${error.message}`);
    return false;
  }
}

function handleTealium(preferences) {
  try {
    // Get the Tealium modal or banner
    const modal = document.querySelector('.tealium-modal') || 
                 document.querySelector('.truste_overlay') ||
                 document.querySelector('.truste_box_overlay') ||
                 document.querySelector('[id^="consent_blackbar"]');
    
    if (!modal) {
      console.log('Cookie Away: Tealium modal not found');
      notifyBackgroundScript('Tealium', false);
      return;
    }
    
    // Find the appropriate buttons based on preferences
    let buttonToClick;
    
    // Accept all cookies
    if (preferences.acceptAll) {
      buttonToClick = findButtonByText(modal, [
        'accept all', 'allow all', 'agree to all', 'accept cookies', 
        'allow cookies', 'consent to all', 'I accept', 'accept', 'agree'
      ]) || modal.querySelector('.call[data-action="accept"]') ||
          modal.querySelector('[data-action="accept_all"]');
    }
    // Reject non-essential cookies
    else if (preferences.rejectAll) {
      buttonToClick = findButtonByText(modal, [
        'reject all', 'decline all', 'necessary only', 'reject non-essential',
        'reject cookies', 'deny', 'decline cookies', 'reject'
      ]) || modal.querySelector('.call[data-action="reject"]') ||
          modal.querySelector('[data-action="reject_all"]');
    }
    // Apply specific preferences
    else {
      // First try to find a customize/settings button
      const customizeButton = findButtonByText(modal, [
        'customize', 'settings', 'preferences', 'configure', 'manage', 'cookie settings',
        'preference center', 'manage consent', 'manage my preferences'
      ]) || modal.querySelector('.call[data-action="preferences"]') ||
          modal.querySelector('[data-action="open_preferences"]');
      
      if (customizeButton) {
        safeClick(customizeButton);
        
        // Wait a moment for preferences panel to appear
        setTimeout(() => {
          // Try to find the preferences panel
          const preferencesPanel = document.querySelector('.tealium-preferences-container') ||
                                document.querySelector('.tealium_cmp__preferences') ||
                                document.querySelector('.truste-preference-center');
          
          if (preferencesPanel) {
            // Find the category toggles/checkboxes
            const toggles = preferencesPanel.querySelectorAll('input[type="checkbox"], input[type="radio"], [role="switch"]');
            
            // Loop through toggles and set preferences
            toggles.forEach(toggle => {
              if (toggle.disabled) return;
              
              // Skip if this is a radio button that's not for enabling the category
              if (toggle.type === 'radio' && !toggle.value.includes('on') && !toggle.value.includes('true')) {
                return;
              }
              
              const categoryItem = toggle.closest('div[class*="category"], div[class*="preference"]');
              if (!categoryItem) return;
              
              const categoryLabel = categoryItem.textContent.toLowerCase();
              
              // Skip if necessary/required cookies (usually can't be toggled)
              if (categoryLabel.includes('necessary') || categoryLabel.includes('required') || 
                  categoryLabel.includes('essential') || categoryLabel.includes('functional') ||
                  categoryLabel.includes('strictly')) {
                return;
              }
              
              // Get current checked state
              const isChecked = toggle.type === 'checkbox' ? toggle.checked : 
                                toggle.type === 'radio' ? toggle.checked : 
                                toggle.getAttribute('aria-checked') === 'true';
              
              // Set toggle according to preferences
              if (
                (categoryLabel.includes('analytic') && preferences.analytics) ||
                (categoryLabel.includes('performance') && preferences.analytics) ||
                (categoryLabel.includes('statistic') && preferences.analytics) ||
                (categoryLabel.includes('functional') && preferences.functional) ||
                (categoryLabel.includes('preference') && preferences.functional) ||
                (categoryLabel.includes('experience') && preferences.functional) ||
                (categoryLabel.includes('advertis') && preferences.advertising) ||
                (categoryLabel.includes('market') && preferences.advertising) ||
                (categoryLabel.includes('target') && preferences.advertising) ||
                (categoryLabel.includes('social') && preferences.advertising)
              ) {
                if (!isChecked) toggle.click();
              } else {
                if (isChecked) toggle.click();
              }
            });
            
            // Find and click the save/confirm button
            const saveButton = findButtonByText(preferencesPanel, [
              'save', 'confirm', 'apply', 'update', 'save preferences', 'save settings', 'submit',
              'save & exit', 'confirm choices', 'save my preferences'
            ]) || preferencesPanel.querySelector('[data-action="submit_preferences"]') ||
               preferencesPanel.querySelector('[data-action="save"]');
            
            if (saveButton) {
              safeClick(saveButton);
              notifyBackgroundScript('Tealium', true);
            }
          }
        }, 500);
      }
    }
    
    if (buttonToClick) {
      safeClick(buttonToClick);
      notifyBackgroundScript('Tealium', true);
    }
  } catch (error) {
    console.log(`Cookie Away: Error handling Tealium - ${error.message}`);
    notifyBackgroundScript('Tealium', false);
  }
}