/**
 * Cookie Away - Usercentrics Handler
 * 
 * This script provides detection and interaction functions for Usercentrics consent dialogs
 */

// Detect if Usercentrics is present on the page
function detectUsercentrics() {
  // Check for Usercentrics script or global objects
  if (
    document.querySelector('script[src*="usercentrics"]') ||
    document.querySelector('script[id*="usercentrics"]') ||
    window.usercentrics ||
    document.querySelector('#usercentrics-root') ||
    document.querySelector('[data-testid="uc-app-container"]') ||
    document.querySelector('[data-testid="uc-banner-container"]')
  ) {
    return true;
  }
  
  // Look for Usercentrics DOM elements
  const usercentricsElements = [
    '#usercentrics-button',
    '.usercentrics-button',
    '[data-testid="uc-banner"]',
    '[data-testid="uc-privacy-button"]',
    '[data-testid="uc-center-container"]',
    '[id*="usercentrics"]',
    '[class*="usercentrics"]'
  ];
  
  for (const selector of usercentricsElements) {
    if (document.querySelector(selector)) {
      return true;
    }
  }
  
  return false;
}

// Handle Usercentrics consent dialog based on user preferences
function handleUsercentrics(preferences) {
  // Wait for the Usercentrics dialog to be fully loaded
  const waitForDialog = () => {
    // Find banner element
    const banner = document.querySelector('#usercentrics-root') || 
                   document.querySelector('[data-testid="uc-app-container"]') ||
                   document.querySelector('[data-testid="uc-banner-container"]');
    
    // If banner is not found or not visible, retry after a short delay
    if (!banner || !isElementVisible(banner)) {
      setTimeout(waitForDialog, 300);
      return;
    }

    // Common button selectors for Usercentrics
    const acceptAllSelector = '[data-testid="uc-accept-all-button"], [data-testid="uc-accept-button"]';
    const rejectAllSelector = '[data-testid="uc-deny-all-button"], [data-testid="uc-deny-button"]';
    const settingsSelector = '[data-testid="uc-customize-button"], [data-testid="uc-more-button"], [data-testid="uc-settings-button"]';
    const saveSelector = '[data-testid="uc-save-button"], [data-testid="uc-confirm-button"]';

    // Find buttons based on selectors
    const acceptAllButton = document.querySelector(acceptAllSelector);
    const rejectAllButton = document.querySelector(rejectAllSelector);
    const settingsButton = document.querySelector(settingsSelector);
    
    // Decision logic based on user preferences
    if (preferences.necessary && !preferences.analytics && !preferences.marketing && !preferences.personalization) {
      // User wants only necessary cookies
      if (rejectAllButton && isElementVisible(rejectAllButton)) {
        safeClick(rejectAllButton);
        notifyBackgroundScript('Usercentrics', true);
        return;
      }
      
      // If no reject all button is available, try to go to settings and customize
      if (settingsButton && isElementVisible(settingsButton)) {
        safeClick(settingsButton);
        
        // Wait for settings panel to load
        setTimeout(() => {
          // Category toggle containers
          const toggleContainers = document.querySelectorAll('[data-testid="uc-category-container"]');
          let hasCustomized = false;
          
          // Try to deselect all non-necessary categories
          toggleContainers.forEach(container => {
            const toggleSwitch = container.querySelector('input[type="checkbox"]');
            const isEssential = container.textContent.toLowerCase().includes('essential') || 
                              container.textContent.toLowerCase().includes('necessary') ||
                              container.textContent.toLowerCase().includes('erforderlich') ||
                              container.querySelector('[data-testid="uc-category-required-label"]');
            
            // If not essential/necessary and toggle exists, turn it off if it's on
            if (toggleSwitch && !isEssential && toggleSwitch.checked) {
              safeClick(toggleSwitch);
              hasCustomized = true;
            }
          });
          
          // Save preferences
          if (hasCustomized) {
            const saveButton = document.querySelector(saveSelector);
            if (saveButton) {
              safeClick(saveButton);
              notifyBackgroundScript('Usercentrics', true);
              return;
            }
          }
          
          notifyBackgroundScript('Usercentrics', false);
        }, 500);
        return;
      }
    } else if (preferences.necessary && preferences.analytics && preferences.marketing && preferences.personalization) {
      // User accepts all cookies
      if (acceptAllButton && isElementVisible(acceptAllButton)) {
        safeClick(acceptAllButton);
        notifyBackgroundScript('Usercentrics', true);
        return;
      }
    } else {
      // User has custom preferences
      if (settingsButton && isElementVisible(settingsButton)) {
        safeClick(settingsButton);
        
        // Wait for settings panel to load
        setTimeout(() => {
          // Category toggle containers
          const toggleContainers = document.querySelectorAll('[data-testid="uc-category-container"]');
          let hasCustomized = false;
          
          // Map user preferences to categories
          toggleContainers.forEach(container => {
            const toggleSwitch = container.querySelector('input[type="checkbox"]');
            if (!toggleSwitch) return;
            
            // Skip if disabled (usually means it's necessary)
            if (toggleSwitch.disabled) return;
            
            const containerText = container.textContent.toLowerCase();
            let shouldBeChecked = false;
            
            // Determine category by text content
            if (containerText.includes('analytics') || containerText.includes('statistics') || containerText.includes('statistik')) {
              shouldBeChecked = preferences.analytics;
            } else if (containerText.includes('marketing') || containerText.includes('advertising') || containerText.includes('werbung')) {
              shouldBeChecked = preferences.marketing;
            } else if (containerText.includes('personal') || containerText.includes('functional') || containerText.includes('funktionell')) {
              shouldBeChecked = preferences.personalization;
            }
            
            // Update the toggle if it doesn't match the user preference
            if (toggleSwitch.checked !== shouldBeChecked) {
              safeClick(toggleSwitch);
              hasCustomized = true;
            }
          });
          
          // Save preferences
          if (hasCustomized) {
            const saveButton = document.querySelector(saveSelector);
            if (saveButton) {
              safeClick(saveButton);
              notifyBackgroundScript('Usercentrics', true);
              return;
            }
          }
          
          notifyBackgroundScript('Usercentrics', false);
        }, 500);
        return;
      }
    }
    
    // If we couldn't handle the dialog properly
    notifyBackgroundScript('Usercentrics', false);
  };
  
  // Start the process
  waitForDialog();
}