/**
 * Cookie Away - Utility Functions
 * 
 * This script provides helper functions used across the extension
 */

// Global flag to track extension state
window.cookieResponseManagerActive = true;

// Global counter to limit detection attempts
window.cookieResponseManagerAttempts = 0;

// Handle extension context invalidated errors
window.addEventListener('error', function(event) {
  // Check if this is an extension context invalidated error
  if (event.message && (
      event.message.includes('Extension context invalidated') || 
      event.message.includes('Receiving end does not exist')
    )) {
    window.cookieResponseManagerActive = false;
    console.log('Cookie Away: Extension context invalidated - this is normal during updates');
    
    // Prevent error from being shown in console
    event.preventDefault();
  }
});

// Safe wrapper for chrome API calls
function safeApiCall(callback) {
  if (!window.cookieResponseManagerActive) return;
  
  try {
    return callback();
  } catch (error) {
    if (error.message && (
        error.message.includes('Extension context invalidated') || 
        error.message.includes('Receiving end does not exist')
      )) {
      window.cookieResponseManagerActive = false;
      console.log(`Cookie Away: Extension context invalidated - ${error.message}`);
    } else {
      console.error(`Cookie Away: Error in API call - ${error.message}`);
    }
    return null;
  }
}

// Check if an element is visible on the screen
function isElementVisible(element) {
  if (!element) return false;
  
  const style = window.getComputedStyle(element);
  
  // Check basic visibility properties
  if (style.display === 'none' || style.visibility === 'hidden' || style.opacity === '0') {
    return false;
  }
  
  // Check if the element has zero dimensions
  const rect = element.getBoundingClientRect();
  if (rect.width <= 0 || rect.height <= 0) {
    return false;
  }
  
  // Check if the element is within the viewport (with some margin)
  const viewportHeight = window.innerHeight || document.documentElement.clientHeight;
  if (rect.bottom < -100 || rect.top > viewportHeight + 100) {
    return false;
  }
  
  return true;
}

// Check if it's safe to interact with an element (won't open new tabs/windows)
function isSafeToClick(element) {
  if (!element) return false;
  
  // Check for target="_blank" attribute on links
  if (element.tagName === 'A' && element.getAttribute('target') === '_blank') {
    console.log('Cookie Away: Skipping link with target="_blank"');
    return false;
  }
  
  // Check for common onclick patterns that might open new windows
  const onclickAttr = element.getAttribute('onclick');
  if (onclickAttr && (
      onclickAttr.includes('window.open') || 
      onclickAttr.includes('open(') ||
      onclickAttr.includes('location.href') ||
      onclickAttr.includes('target=') ||
      onclickAttr.includes('newWindow') ||
      onclickAttr.includes('new window')
    )) {
    console.log('Cookie Away: Skipping element with potentially unsafe onclick handler');
    return false;
  }
  
  // Check for href that might open a new window or redirect to external sites
  const href = element.getAttribute('href');
  if (href && (
      href.startsWith('javascript:window.open') ||
      href.includes('_blank') ||
      href.includes('newwindow') ||
      href.includes('popup')
    )) {
    console.log('Cookie Away: Skipping element with potentially unsafe href', href);
    return false;
  }
  
  // Check for data attributes that might indicate window opening
  for (const attr of element.attributes) {
    if (attr.name.startsWith('data-') && attr.value && (
        attr.value.includes('window.open') ||
        attr.value.includes('new window') ||
        attr.value.includes('popup') ||
        attr.value.includes('target=_blank')
      )) {
      console.log('Cookie Away: Skipping element with potentially unsafe data attribute');
      return false;
    }
  }
  
  return true;
}

// Find a button by its text content
function findButtonByText(rootElement, textPatterns) {
  // Check if input is a string or an array
  if (typeof textPatterns === 'string') {
    textPatterns = [textPatterns];
  }
  
  // Make case-insensitive patterns
  const patterns = textPatterns.map(pattern => 
    typeof pattern === 'string' ? new RegExp(pattern, 'i') : pattern
  );
  
  // Terms that indicate the button is a "learn more" that we want to avoid
  const avoidTerms = [
    'policy', 'learn more', 'details', 'information', 'more info', 
    'view', 'read', 'cookie policy', 'privacy policy', 'see details',
    'how we use', 'about cookies', 'what are', 'find out'
  ];
  
  // Look for buttons, links, and elements with role="button"
  const buttons = rootElement.querySelectorAll('button, a, [role="button"], input[type="button"], input[type="submit"]');
  
  // For logging purposes, collect all buttons
  if (buttons.length > 0) {
    console.log('Cookie Away: Found', buttons.length, 'potential buttons/links');
    
    // Log basic info about all found buttons for debugging
    Array.from(buttons).forEach((btn, index) => {
      console.log(`Cookie Away: Button ${index+1}:`, 
                 btn.tagName, 
                 btn.textContent ? `text="${btn.textContent.trim().substring(0, 30)}"` : '(no text)',
                 btn.className ? `class="${btn.className}"` : '',
                 btn.id ? `id="${btn.id}"` : '');
    });
  }
  
  // First pass: look for action buttons that are NOT policy links
  // We prioritize buttons with direct action terms over links with policy terms
  const actionButtons = [];
  const policyLinks = [];
  
  for (const button of buttons) {
    // Skip if element is not safe to click
    if (!isSafeToClick(button)) continue;
    
    const buttonText = button.textContent.trim().toLowerCase();
    const isLink = button.tagName === 'A' && button.hasAttribute('href');
    
    // Check if this is a policy link we should avoid
    const isPolicyLink = avoidTerms.some(term => buttonText.includes(term));
    
    // Check if this button text matches our desired patterns
    let matchesPattern = false;
    for (const pattern of patterns) {
      if (pattern.test(buttonText)) {
        matchesPattern = true;
        break;
      }
    }
    
    // Also check aria-label attribute
    const ariaLabel = button.getAttribute('aria-label');
    if (ariaLabel) {
      for (const pattern of patterns) {
        if (pattern.test(ariaLabel.toLowerCase())) {
          matchesPattern = true;
          break;
        }
      }
    }
    
    if (matchesPattern) {
      if (isPolicyLink) {
        policyLinks.push({button, text: buttonText});
        console.log('Cookie Away: Found policy link/button (lower priority):', buttonText);
      } else {
        // Higher priority for non-policy buttons, especially if they're not links
        const priority = isLink ? 1 : 2;
        actionButtons.push({button, text: buttonText, priority});
        console.log('Cookie Away: Found action button (higher priority):', buttonText);
      }
    }
  }
  
  // Sort action buttons by priority (higher first)
  actionButtons.sort((a, b) => b.priority - a.priority);
  
  // First return action buttons if we have any
  if (actionButtons.length > 0) {
    console.log('Cookie Away: Selected action button:', actionButtons[0].text);
    return actionButtons[0].button;
  }
  
  // Only if we have no action buttons, consider policy links as a last resort
  if (policyLinks.length > 0) {
    console.log('Cookie Away: No action buttons found, using policy link as last resort:', policyLinks[0].text);
    return policyLinks[0].button;
  }
  
  console.log('Cookie Away: No matching buttons found');
  return null;
}

// Function to check if detection should be attempted (limits to max 2 attempts)
function canAttemptDetection() {
  if (window.cookieResponseManagerAttempts >= 2) {
    console.log('Cookie Away: Maximum detection attempts reached (2), skipping further attempts');
    return false;
  }
  
  window.cookieResponseManagerAttempts++;
  return true;
}

// Get the user's preferences for the current site
async function getUserPreferences(hostname) {
  return new Promise((resolve) => {
    // Default preferences in case of failure
    const defaultResponse = {
      preferences: { necessary: true, analytics: false, marketing: false, personalization: false },
      isException: false,
      settings: { autoApplyConsent: true, notifyOnAction: true }
    };
    
    // Check if extension is active before making API calls
    if (!window.cookieResponseManagerActive) {
      resolve(defaultResponse);
      return;
    }
    
    try {
      chrome.runtime.sendMessage(
        { action: 'getPreferences' },
        (response) => {
          // Check for error or missing response
          if (chrome.runtime.lastError) {
            console.log(`Cookie Away: ${chrome.runtime.lastError.message}`);
            resolve(defaultResponse);
            return;
          }
          
          if (!response) {
            console.log('Cookie Away: Could not get preferences from the extension');
            resolve(defaultResponse);
            return;
          }
          
          // Check if this domain has specific exceptions
          const isException = response.siteExceptions && hostname in response.siteExceptions;
          const preferences = isException ? response.siteExceptions[hostname] : response.preferences;
          
          resolve({
            preferences,
            isException,
            settings: response.settings
          });
        }
      );
    } catch (error) {
      console.log(`Cookie Away: Error getting preferences - ${error.message}`);
      resolve(defaultResponse);
    }
  });
}

// Safe click function that avoids Content Security Policy issues
// and prevents opening new tabs or windows - with multiple fallback strategies
function safeClick(element) {
  if (!element) {
    console.log('Cookie Away: Cannot click null element');
    return false;
  }
  
  try {
    // Log what we're trying to click for debugging
    console.log('Cookie Away: Attempting to click element:', 
               element.tagName, 
               element.id ? `id="${element.id}"` : '', 
               element.className ? `class="${element.className}"` : '',
               element.textContent ? `text="${element.textContent.substring(0, 20).trim()}${element.textContent.length > 20 ? '...' : ''}"` : '');
    
    // If element is not visible, try forcing it to be visible
    if (!isElementVisible(element)) {
      console.log('Cookie Away: Element is not visible, attempting to make it visible');
      
      // Store original styles
      const originalDisplay = element.style.display;
      const originalVisibility = element.style.visibility;
      const originalOpacity = element.style.opacity;
      const originalPointerEvents = element.style.pointerEvents;
      const originalPosition = element.style.position;
      const originalZIndex = element.style.zIndex;
      
      // Force the element to be visible and interactive
      element.style.display = 'block';
      element.style.visibility = 'visible';
      element.style.opacity = '1';
      element.style.pointerEvents = 'auto';
      element.style.position = 'relative';
      element.style.zIndex = '9999';
      
      // Create a brief delay to allow styles to take effect
      setTimeout(() => {
        // Check if now visible
        if (!isElementVisible(element)) {
          console.log('Cookie Away: Could not make element visible, trying click anyway');
        }
      }, 10);
    }
    
    // Prepare element for safe clicking, regardless of visibility
    // Store original interactive attributes
    const originalTarget = element.getAttribute('target');
    const originalRel = element.getAttribute('rel');
    const originalHref = element.getAttribute('href');
    const originalOnClick = element.getAttribute('onclick');
    
    // Modify element to be safe
    if (originalTarget === '_blank' || originalTarget === '_new') {
      element.setAttribute('target', '_self');
    }
    
    // For potentially dangerous links, temporarily neutralize
    if (originalHref) {
      if (originalHref.startsWith('http') || originalHref.startsWith('www')) {
        element.setAttribute('rel', 'noopener noreferrer');
        
        // Only for external links that might navigate away
        if (!originalHref.includes(window.location.hostname)) {
          const tempHref = element.getAttribute('href');
          element.setAttribute('href', 'javascript:void(0)');
          
          // Click and restore
          console.log('Cookie Away: Modified external link to prevent navigation');
          element.click();
          
          // Restore original href
          element.setAttribute('href', tempHref);
          
          console.log('Cookie Away: Successfully clicked element with modified href');
          return true;
        }
      }
    }
    
    // Try different click strategies in order of preference
    
    // Strategy 1: Direct click (most natural)
    try {
      console.log('Cookie Away: Attempting direct click');
      element.click();
      console.log('Cookie Away: Direct click succeeded');
      
      // Restore original attributes
      restoreAttributes();
      return true;
    } catch (directClickError) {
      console.log(`Cookie Away: Direct click failed - ${directClickError.message}, trying alternatives`);
    }
    
    // Strategy 2: Click via MouseEvent
    try {
      console.log('Cookie Away: Attempting MouseEvent click');
      const clickEvent = new MouseEvent('click', {
        view: window,
        bubbles: true,
        cancelable: true,
        clientX: 1,
        clientY: 1
      });
      element.dispatchEvent(clickEvent);
      console.log('Cookie Away: MouseEvent click succeeded');
      
      // Restore original attributes
      restoreAttributes();
      return true;
    } catch (mouseEventError) {
      console.log(`Cookie Away: MouseEvent click failed - ${mouseEventError.message}, trying more alternatives`);
    }
    
    // Strategy 3: Focus + Enter key (for form elements)
    try {
      if (element.tagName === 'INPUT' || element.tagName === 'BUTTON' || element.tagName === 'SELECT' || element.tagName === 'A') {
        console.log('Cookie Away: Attempting focus + Enter key simulation');
        element.focus();
        
        const keyEvent = new KeyboardEvent('keydown', {
          key: 'Enter',
          code: 'Enter',
          keyCode: 13,
          which: 13,
          bubbles: true,
          cancelable: true
        });
        element.dispatchEvent(keyEvent);
        console.log('Cookie Away: Focus + Enter key simulation succeeded');
        
        // Restore original attributes
        restoreAttributes();
        return true;
      }
    } catch (keyEventError) {
      console.log(`Cookie Away: Focus + Enter key simulation failed - ${keyEventError.message}`);
    }
    
    // Strategy 4: Synthetic event handlers
    try {
      console.log('Cookie Away: Attempting synthetic event handler execution');
      
      // For links, try navigation simulation
      if (element.tagName === 'A' && element.href) {
        console.log('Cookie Away: Attempting to simulate link navigation');
        
        // Get the href and try to follow it
        const href = element.getAttribute('href');
        if (href && !href.startsWith('javascript:') && !href.includes('#')) {
          // Only simulate navigation if it's a fragment link to avoid actual page changes
          if (href.startsWith('#')) {
            window.location.hash = href.substring(1);
            console.log('Cookie Away: Simulated navigation to fragment');
            
            // Restore original attributes
            restoreAttributes();
            return true;
          }
        }
      }
      
      // For buttons, try to trigger any 'onclick' behavior
      if (originalOnClick) {
        // Note: This is a last resort and should be used carefully
        console.log('Cookie Away: Attempting to trigger attached onclick handler');
        
        // Use a timeout to help avoid potential loops or immediate page transitions
        setTimeout(() => {
          try {
            // Create a function from the onclick attribute and execute it
            // This is risky, so we only do it as a last resort
            new Function(originalOnClick)();
            console.log('Cookie Away: Successfully executed onclick handler');
          } catch (execError) {
            console.log(`Cookie Away: Failed to execute onclick handler - ${execError.message}`);
          }
        }, 10);
        
        // Restore original attributes
        restoreAttributes();
        return true;
      }
    } catch (syntheticEventError) {
      console.log(`Cookie Away: Synthetic event handler execution failed - ${syntheticEventError.message}`);
    }
    
    // Final fallback - try one more direct click after all modifications
    try {
      console.log('Cookie Away: Final attempt with direct click');
      element.click();
      console.log('Cookie Away: Final direct click succeeded');
      
      // Restore original attributes
      restoreAttributes();
      return true;
    } catch (finalError) {
      console.log(`Cookie Away: All click methods failed - ${finalError.message}`);
      
      // Restore original attributes
      restoreAttributes();
      return false;
    }
    
    // Helper function to restore element to original state
    function restoreAttributes() {
      // Restore target
      if (originalTarget !== null) {
        element.setAttribute('target', originalTarget);
      } else if (element.hasAttribute('target')) {
        element.removeAttribute('target');
      }
      
      // Restore rel
      if (originalRel !== null) {
        element.setAttribute('rel', originalRel);
      } else if (element.hasAttribute('rel')) {
        element.removeAttribute('rel');
      }
      
      // Restore href
      if (originalHref !== null && element.getAttribute('href') !== originalHref) {
        element.setAttribute('href', originalHref);
      }
      
      // Restore onclick
      if (originalOnClick !== null && element.getAttribute('onclick') !== originalOnClick) {
        element.setAttribute('onclick', originalOnClick);
      }
    }
  } catch (outerError) {
    console.log(`Cookie Away: Critical error in safeClick function - ${outerError.message}`);
    return false;
  }
}

// Add Global Privacy Control (GPC) implementation
(function setUpGPC() {
  // Only proceed if GPC is not already defined
  if (navigator.globalPrivacyControl === undefined) {
    // Check if GPC is enabled in user preferences - using safe API call
    safeApiCall(() => {
      chrome.runtime.sendMessage(
        { action: 'getPreferences' },
        (response) => {
          // Check for runtime errors
          if (chrome.runtime.lastError) {
            console.log(`Cookie Away GPC: ${chrome.runtime.lastError.message}`);
            return;
          }
          
          if (response && response.preferences && response.preferences.sendGPC) {
            try {
              console.log('Cookie Away: Setting up Global Privacy Control (GPC) signal');
              
              // Set up DOM signal - this is the main way browsers detect GPC
              Object.defineProperty(navigator, 'globalPrivacyControl', {
                value: true,
                enumerable: true,
                configurable: false,
                writable: false
              });
              
              // Add a meta tag for sites that look for it in the HTML
              const meta = document.createElement('meta');
              meta.name = 'globalPrivacyControl';
              meta.content = '1';
              document.head.appendChild(meta);
              
              // Add the Sec-GPC header through a Service Worker (note: this will be simulated
              // since content scripts can't directly modify headers)
              // Signal to the background script to add the header for future requests
              chrome.runtime.sendMessage({ action: 'enableGPC' });
              
              // Set up an event listener for the DOM Privacy signal
              window.addEventListener('secgpcchange', () => {
                console.log('Cookie Away: GPC signal change detected');
              });
              
              // Create and dispatch a custom event to notify any listeners on the page
              const gpcEvent = new CustomEvent('gpcEnabled', { 
                detail: { enabled: true },
                bubbles: true
              });
              document.dispatchEvent(gpcEvent);
              
              // Intercept the GPC status check from various CMPs
              if (window.Didomi) {
                try {
                  // Override Didomi's GPC detection
                  Object.defineProperty(window.Didomi.prototype, 'isGPCEnabled', {
                    value: () => true,
                    writable: false
                  });
                } catch (e) {
                  console.log('Cookie Away: Could not override Didomi GPC check');
                }
              }
              
              console.log('Cookie Away: GPC signal successfully enabled');
            } catch (error) {
              console.log(`Cookie Away: Could not set GPC signal - ${error.message}`);
            }
          } else {
            console.log('Cookie Away: GPC is disabled in user preferences');
          }
        }
      );
    });
  } else {
    // GPC is already defined in the navigator, log this information
    console.log(`Cookie Away: GPC already defined in navigator: ${navigator.globalPrivacyControl}`);
  }
})();
