/**
 * Cookie Response Manager - Storage Service
 * 
 * This module provides an interface for managing storage operations,
 * handling defaults, and validating data.
 */

// Default cookie preferences
const DEFAULT_PREFERENCES = {
  necessary: true,  // Always enabled
  analytics: false,
  marketing: false,
  personalization: false,
  sendGPC: true     // Global Privacy Control
};

// Default settings
const DEFAULT_SETTINGS = {
  autoApplyConsent: true,
  notifyOnAction: true,
  handleAmbiguousBanners: false // Default to false to prevent unwanted clicks on ambiguous elements
};

// Get all stored data
async function getAllStoredData() {
  return new Promise((resolve) => {
    chrome.storage.local.get(['preferences', 'siteExceptions', 'settings'], (data) => {
      resolve({
        preferences: data.preferences || DEFAULT_PREFERENCES,
        siteExceptions: data.siteExceptions || {},
        settings: data.settings || DEFAULT_SETTINGS
      });
    });
  });
}

// Get user preferences (global or site-specific)
async function getUserPreferences(hostname = null) {
  const data = await getAllStoredData();
  
  // If no hostname provided, return global preferences
  if (!hostname) {
    return data.preferences;
  }
  
  // Check if this domain has specific exceptions
  const isException = data.siteExceptions && hostname in data.siteExceptions;
  const preferences = isException ? data.siteExceptions[hostname] : data.preferences;
  
  return {
    preferences,
    isException,
    settings: data.settings
  };
}

// Save global preferences
async function saveGlobalPreferences(preferences) {
  // Ensure necessary cookies are always enabled
  preferences.necessary = true;
  
  return new Promise((resolve) => {
    chrome.storage.local.set({ preferences }, () => {
      resolve({ success: true });
    });
  });
}

// Add a site exception
async function addSiteException(domain, preferences) {
  // Ensure necessary cookies are always enabled
  preferences.necessary = true;
  
  return new Promise((resolve) => {
    chrome.storage.local.get('siteExceptions', (data) => {
      const siteExceptions = data.siteExceptions || {};
      siteExceptions[domain] = preferences;
      
      chrome.storage.local.set({ siteExceptions }, () => {
        resolve({ success: true });
      });
    });
  });
}

// Remove a site exception
async function removeSiteException(domain) {
  return new Promise((resolve) => {
    chrome.storage.local.get('siteExceptions', (data) => {
      const siteExceptions = data.siteExceptions || {};
      delete siteExceptions[domain];
      
      chrome.storage.local.set({ siteExceptions }, () => {
        resolve({ success: true });
      });
    });
  });
}

// Update extension settings
async function updateSettings(settings) {
  return new Promise((resolve) => {
    chrome.storage.local.set({ settings }, () => {
      resolve({ success: true });
    });
  });
}

// Export functions for use in other scripts
window.storageService = {
  getUserPreferences,
  saveGlobalPreferences,
  addSiteException,
  removeSiteException,
  updateSettings
};
