/**
 * Cookie Response Manager - Options Page Script
 */

// DOM Elements
const necessaryCheckbox = document.getElementById('necessary');
const analyticsCheckbox = document.getElementById('analytics');
const marketingCheckbox = document.getElementById('marketing');
const personalizationCheckbox = document.getElementById('personalization');
const gpcCheckbox = document.getElementById('gpc');
const autoApplyCheckbox = document.getElementById('auto-apply');
const notifyActionCheckbox = document.getElementById('notify-action');
const collectStatsCheckbox = document.getElementById('collect-stats');
const reportUnhandledCheckbox = document.getElementById('report-unhandled');
const savePreferencesButton = document.getElementById('save-preferences');
const saveSettingsButton = document.getElementById('save-settings');
const siteExceptionsContainer = document.getElementById('site-exceptions-container');
const extensionVersionSpan = document.getElementById('extension-version');
const statusMessage = document.getElementById('status-message');

// Load preferences and settings when the page loads
document.addEventListener('DOMContentLoaded', async () => {
  try {
    // Display extension version
    const manifestData = chrome.runtime.getManifest();
    extensionVersionSpan.textContent = manifestData.version;
    
    // Load user preferences
    await loadPreferences();
    
    // Load site exceptions
    await loadSiteExceptions();
  } catch (error) {
    console.error('Error initializing options page:', error);
    showStatus('Error loading settings', true);
  }
});

// Load user preferences from storage
async function loadPreferences() {
  try {
    const data = await new Promise(resolve => {
      chrome.runtime.sendMessage({ action: 'getPreferences' }, (response) => {
        resolve(response || {});
      });
    });
    
    const preferences = data.preferences || { necessary: true };
    const settings = data.settings || { 
      autoApplyConsent: true, 
      notifyOnAction: true,
      collectStats: true,
      reportUnhandledBanners: true
    };
    
    // Update checkbox states
    analyticsCheckbox.checked = preferences.analytics || false;
    marketingCheckbox.checked = preferences.marketing || false;
    personalizationCheckbox.checked = preferences.personalization || false;
    gpcCheckbox.checked = preferences.sendGPC || false;
    
    // Update settings checkboxes
    autoApplyCheckbox.checked = settings.autoApplyConsent;
    notifyActionCheckbox.checked = settings.notifyOnAction;
    collectStatsCheckbox.checked = settings.collectStats;
    reportUnhandledCheckbox.checked = settings.reportUnhandledBanners;
  } catch (error) {
    console.error('Error loading preferences:', error);
    showStatus('Error loading preferences', true);
  }
}

// Load site exceptions
async function loadSiteExceptions() {
  try {
    const data = await new Promise(resolve => {
      chrome.runtime.sendMessage({ action: 'getPreferences' }, (response) => {
        resolve(response || {});
      });
    });
    
    const siteExceptions = data.siteExceptions || {};
    const domains = Object.keys(siteExceptions);
    
    if (domains.length === 0) {
      siteExceptionsContainer.innerHTML = `
        <p class="empty-state">No site exceptions added yet.</p>
        <p class="note">You can add exceptions for specific sites from the extension popup when visiting a website.</p>
      `;
      return;
    }
    
    // Create table to display site exceptions
    let tableHtml = `
      <table class="exceptions-table">
        <thead>
          <tr>
            <th>Website</th>
            <th>Necessary</th>
            <th>Analytics</th>
            <th>Marketing</th>
            <th>Personalization</th>
            <th>Actions</th>
          </tr>
        </thead>
        <tbody>
    `;
    
    domains.forEach(domain => {
      const prefs = siteExceptions[domain];
      
      // Skip entries with "noAction" set
      if (prefs.noAction) {
        tableHtml += `
          <tr>
            <td>${domain}</td>
            <td colspan="4"><span class="no-action">No automatic action</span></td>
            <td>
              <button class="remove-exception" data-domain="${domain}">Remove</button>
            </td>
          </tr>
        `;
        return;
      }
      
      tableHtml += `
        <tr>
          <td>${domain}</td>
          <td><span class="checkbox-indicator ${prefs.necessary ? 'checked' : ''}">✓</span></td>
          <td><span class="checkbox-indicator ${prefs.analytics ? 'checked' : ''}">✓</span></td>
          <td><span class="checkbox-indicator ${prefs.marketing ? 'checked' : ''}">✓</span></td>
          <td><span class="checkbox-indicator ${prefs.personalization ? 'checked' : ''}">✓</span></td>
          <td>
            <button class="remove-exception" data-domain="${domain}">Remove</button>
          </td>
        </tr>
      `;
    });
    
    tableHtml += `
        </tbody>
      </table>
    `;
    
    siteExceptionsContainer.innerHTML = tableHtml;
    
    // Add event listeners to remove buttons
    document.querySelectorAll('.remove-exception').forEach(button => {
      button.addEventListener('click', removeSiteException);
    });
  } catch (error) {
    console.error('Error loading site exceptions:', error);
    showStatus('Error loading site exceptions', true);
    siteExceptionsContainer.innerHTML = '<p class="error">Error loading site exceptions</p>';
  }
}

// Save global preferences
savePreferencesButton.addEventListener('click', async () => {
  try {
    const preferences = {
      necessary: true, // Always enabled
      analytics: analyticsCheckbox.checked,
      marketing: marketingCheckbox.checked,
      personalization: personalizationCheckbox.checked,
      sendGPC: gpcCheckbox.checked
    };
    
    await new Promise(resolve => {
      chrome.runtime.sendMessage({ 
        action: 'setPreferences', 
        preferences: preferences 
      }, (response) => {
        resolve(response || {});
      });
    });
    
    showStatus('Global preferences saved!');
  } catch (error) {
    console.error('Error saving preferences:', error);
    showStatus('Error saving preferences', true);
  }
});

// Save extension settings
saveSettingsButton.addEventListener('click', async () => {
  try {
    const settings = {
      autoApplyConsent: autoApplyCheckbox.checked,
      notifyOnAction: notifyActionCheckbox.checked,
      collectStats: collectStatsCheckbox.checked,
      reportUnhandledBanners: reportUnhandledCheckbox.checked
    };
    
    await new Promise(resolve => {
      chrome.runtime.sendMessage({ 
        action: 'updateSettings', 
        settings: settings 
      }, (response) => {
        resolve(response || {});
      });
    });
    
    showStatus('Settings saved!');
  } catch (error) {
    console.error('Error saving settings:', error);
    showStatus('Error saving settings', true);
  }
});

// Remove site exception
async function removeSiteException(event) {
  const domain = event.target.dataset.domain;
  if (!domain) return;
  
  try {
    await new Promise(resolve => {
      chrome.runtime.sendMessage({ 
        action: 'removeSiteException', 
        domain: domain
      }, (response) => {
        resolve(response || {});
      });
    });
    
    // Reload site exceptions to update the UI
    loadSiteExceptions();
    showStatus(`Exception for ${domain} removed!`);
  } catch (error) {
    console.error('Error removing site exception:', error);
    showStatus('Error removing site exception', true);
  }
}

// Show status message
function showStatus(message, isError = false) {
  statusMessage.textContent = message;
  statusMessage.className = isError ? 'error' : 'success';
  
  // Clear the message after 3 seconds
  setTimeout(() => {
    statusMessage.textContent = '';
    statusMessage.className = '';
  }, 3000);
  
  // Scroll to the bottom to make the status message visible
  window.scrollTo(0, document.body.scrollHeight);
}
