/**
 * Cookie Away - Popup Script
 */

// DOM Elements
const necessaryCheckbox = document.getElementById('necessary');
const analyticsCheckbox = document.getElementById('analytics');
const marketingCheckbox = document.getElementById('marketing');
const personalizationCheckbox = document.getElementById('personalization');
const gpcCheckbox = document.getElementById('gpc');
const currentSiteInfo = document.getElementById('current-site-info');
const addExceptionButton = document.getElementById('add-exception');
const removeExceptionButton = document.getElementById('remove-exception');
const savePreferencesButton = document.getElementById('save-preferences');
const openOptionsLink = document.getElementById('open-options');
const detectionStatus = document.getElementById('detection-status');
const viewStatsLink = document.getElementById('view-stats');
const clearActivityButton = document.getElementById('clear-activity');
const statusMessage = document.getElementById('status-message');

// Update section elements
const updateSection = document.getElementById('update-section');
const updateMessage = document.getElementById('update-message');
const updateLink = document.getElementById('update-link');
const directUpdateButton = document.getElementById('direct-update-button');
const currentVersionSpan = document.getElementById('current-version');

// Store current tab info
let currentTab = null;
let currentDomain = '';
let hasException = false;

// Load current preferences and site information
document.addEventListener('DOMContentLoaded', async () => {
  try {
    // Get the active tab
    const tabs = await chrome.tabs.query({ active: true, currentWindow: true });
    currentTab = tabs[0];
    
    // Check for updates
    await checkForUpdates();
    
    if (currentTab && currentTab.url) {
      // Get domain from URL
      const url = new URL(currentTab.url);
      currentDomain = url.hostname;
      
      // Update site information
      currentSiteInfo.innerHTML = `
        <p><strong>Current site:</strong> ${currentDomain}</p>
      `;
      
      // Load user preferences
      await loadPreferences();
      
      // Check for banner detection status
      await loadDetectionStatus();
    } else {
      currentSiteInfo.innerHTML = `<p>No website detected.</p>`;
      detectionStatus.innerHTML = `<p>No active website to check.</p>`;
      disableSiteControls();
    }
  } catch (error) {
    console.error('Error loading popup:', error);
    currentSiteInfo.innerHTML = `<p>Error: Could not load site information.</p>`;
    detectionStatus.innerHTML = `<p>Error: Could not check detection status.</p>`;
    disableSiteControls();
  }
});

// Load user preferences from storage
async function loadPreferences() {
  try {
    const data = await new Promise(resolve => {
      chrome.runtime.sendMessage({ action: 'getPreferences' }, (response) => {
        resolve(response || {});
      });
    });
    
    const preferences = data.preferences || { necessary: true };
    const siteExceptions = data.siteExceptions || {};
    
    // Check if current site has an exception
    hasException = siteExceptions.hasOwnProperty(currentDomain);
    
    // If there's a site exception, use those preferences instead
    const activePrefs = hasException ? siteExceptions[currentDomain] : preferences;
    
    // Update checkbox states
    analyticsCheckbox.checked = activePrefs.analytics || false;
    marketingCheckbox.checked = activePrefs.marketing || false;
    personalizationCheckbox.checked = activePrefs.personalization || false;
    gpcCheckbox.checked = activePrefs.sendGPC || false;
    
    // Update buttons
    if (hasException) {
      addExceptionButton.textContent = 'Update Site Exception';
      removeExceptionButton.disabled = false;
      currentSiteInfo.innerHTML += `<p class="exception-note">This site has custom settings.</p>`;
    } else {
      addExceptionButton.textContent = 'Add Site Exception';
      removeExceptionButton.disabled = true;
    }
  } catch (error) {
    console.error('Error loading preferences:', error);
    showStatus('Error loading preferences', true);
  }
}

// Save global preferences
savePreferencesButton.addEventListener('click', async () => {
  try {
    const preferences = {
      necessary: true, // Always enabled
      analytics: analyticsCheckbox.checked,
      marketing: marketingCheckbox.checked,
      personalization: personalizationCheckbox.checked,
      sendGPC: gpcCheckbox.checked
    };
    
    await new Promise(resolve => {
      chrome.runtime.sendMessage({ 
        action: 'setPreferences', 
        preferences: preferences 
      }, (response) => {
        resolve(response || {});
      });
    });
    
    showStatus('Preferences saved!');
    
    // If we're on a site and it doesn't have an exception, apply the changes immediately
    if (currentTab && currentTab.url && !hasException) {
      // Message the content script to refresh
      try {
        chrome.tabs.sendMessage(currentTab.id, { action: 'refreshPreferences' });
      } catch (e) {
        // Tab might not have content script loaded, which is fine
      }
    }
  } catch (error) {
    console.error('Error saving preferences:', error);
    showStatus('Error saving preferences', true);
  }
});

// Add site exception
addExceptionButton.addEventListener('click', async () => {
  if (!currentDomain) return;
  
  try {
    const sitePreferences = {
      necessary: true, // Always enabled
      analytics: analyticsCheckbox.checked,
      marketing: marketingCheckbox.checked,
      personalization: personalizationCheckbox.checked
    };
    
    await new Promise(resolve => {
      chrome.runtime.sendMessage({ 
        action: 'addSiteException', 
        domain: currentDomain,
        preferences: sitePreferences
      }, (response) => {
        resolve(response || {});
      });
    });
    
    hasException = true;
    addExceptionButton.textContent = 'Update Site Exception';
    removeExceptionButton.disabled = false;
    
    // Update site info
    currentSiteInfo.innerHTML = `
      <p><strong>Current site:</strong> ${currentDomain}</p>
      <p class="exception-note">This site has custom settings.</p>
    `;
    
    showStatus('Site exception added!');
    
    // Apply changes immediately
    if (currentTab && currentTab.url) {
      try {
        chrome.tabs.sendMessage(currentTab.id, { action: 'refreshPreferences' });
      } catch (e) {
        // Tab might not have content script loaded, which is fine
      }
    }
  } catch (error) {
    console.error('Error adding site exception:', error);
    showStatus('Error adding site exception', true);
  }
});

// Remove site exception
removeExceptionButton.addEventListener('click', async () => {
  if (!currentDomain || !hasException) return;
  
  try {
    await new Promise(resolve => {
      chrome.runtime.sendMessage({ 
        action: 'removeSiteException', 
        domain: currentDomain
      }, (response) => {
        resolve(response || {});
      });
    });
    
    hasException = false;
    addExceptionButton.textContent = 'Add Site Exception';
    removeExceptionButton.disabled = true;
    
    // Update site info
    currentSiteInfo.innerHTML = `<p><strong>Current site:</strong> ${currentDomain}</p>`;
    
    // Reset checkboxes to global preferences
    loadPreferences();
    
    showStatus('Site exception removed!');
    
    // Apply changes immediately
    if (currentTab && currentTab.url) {
      try {
        chrome.tabs.sendMessage(currentTab.id, { action: 'refreshPreferences' });
      } catch (e) {
        // Tab might not have content script loaded, which is fine
      }
    }
  } catch (error) {
    console.error('Error removing site exception:', error);
    showStatus('Error removing site exception', true);
  }
});

// Open options page
openOptionsLink.addEventListener('click', (e) => {
  e.preventDefault();
  chrome.runtime.openOptionsPage();
});

// Clear site activity
clearActivityButton.addEventListener('click', async () => {
  try {
    // Call the background script to clear site activity
    await new Promise(resolve => {
      chrome.runtime.sendMessage({ 
        action: 'clearSiteActivity'
      }, (response) => {
        resolve(response || {});
      });
    });
    
    // Update the detection status
    detectionStatus.innerHTML = `
      <p class="neutral-status">No cookie banner detection has been attempted on this site yet.</p>
    `;
    
    showStatus('Site activity cleared!');
  } catch (error) {
    console.error('Error clearing site activity:', error);
    showStatus('Error clearing site activity', true);
  }
});

// Show status message
function showStatus(message, isError = false) {
  statusMessage.textContent = message;
  statusMessage.className = isError ? 'error' : 'success';
  
  // Clear the message after 3 seconds
  setTimeout(() => {
    statusMessage.textContent = '';
    statusMessage.className = '';
  }, 3000);
}

// Disable site-specific controls
function disableSiteControls() {
  addExceptionButton.disabled = true;
  removeExceptionButton.disabled = true;
}

// Load cookie banner detection status for the current site
async function loadDetectionStatus() {
  if (!currentTab || !currentTab.id) {
    detectionStatus.innerHTML = `<p>No active tab to check.</p>`;
    return;
  }

  try {
    // Get recent detection stats for this domain
    const data = await new Promise(resolve => {
      chrome.runtime.sendMessage({ action: 'getStats' }, (response) => {
        resolve(response || {});
      });
    });

    const stats = data.stats || { 
      recentActivity: [], 
      totalAttempts: 0, 
      successCount: 0, 
      failureCount: 0 
    };
    
    // Check if we have recent activity for this domain
    const domainActivities = stats.recentActivity.filter(activity => 
      activity.url === currentDomain
    );
    
    if (domainActivities.length > 0) {
      // Get the most recent activity
      const latestActivity = domainActivities[0];
      
      if (latestActivity.success) {
        detectionStatus.innerHTML = `
          <p class="success-status">
            <span class="status-icon">✓</span>
            Successfully handled a ${latestActivity.cmpType} cookie banner on this site.
          </p>
          <p class="status-time">Last handled: ${formatTime(new Date(latestActivity.timestamp))}</p>
        `;
      } else {
        detectionStatus.innerHTML = `
          <p class="failure-status">
            <span class="status-icon">⚠</span>
            Attempted but failed to handle a ${latestActivity.cmpType} cookie banner on this site.
          </p>
          <p class="status-time">Last attempt: ${formatTime(new Date(latestActivity.timestamp))}</p>
        `;
      }
    } else {
      // No detection attempted on this site yet
      detectionStatus.innerHTML = `
        <p class="neutral-status">No cookie banner detection has been attempted on this site yet.</p>
      `;
    }
    
    // Help message is now static in the HTML, no need to add it dynamically
  } catch (error) {
    console.error('Error loading detection status:', error);
    detectionStatus.innerHTML = `<p class="error-status">Error loading detection status.</p>`;
  }
}

// Helper function to format timestamp
function formatTime(date) {
  const now = new Date();
  const diffMs = now - date;
  const diffMins = Math.round(diffMs / 60000);
  
  if (diffMins < 1) {
    return 'Just now';
  } else if (diffMins < 60) {
    return `${diffMins} minute${diffMins === 1 ? '' : 's'} ago`;
  } else if (diffMins < 1440) { // less than 24 hours
    const hours = Math.floor(diffMins / 60);
    return `${hours} hour${hours === 1 ? '' : 's'} ago`;
  } else {
    const days = Math.floor(diffMins / 1440);
    return `${days} day${days === 1 ? '' : 's'} ago`;
  }
}

// Check for extension updates
async function checkForUpdates() {
  try {
    // Get stored update information
    const data = await chrome.storage.local.get('updateInfo');
    const updateInfo = data.updateInfo || {};
    
    // Get the current extension version
    const manifest = chrome.runtime.getManifest();
    const currentVersion = manifest.version;
    
    // Update version display
    currentVersionSpan.textContent = `Current version: ${currentVersion}`;
    
    // If an update is available, show the notification
    if (updateInfo.available) {
      updateMessage.textContent = `A new version (${updateInfo.newVersion}) is available. Your current version is ${currentVersion}.`;
      updateSection.classList.remove('hidden');
      
      // Setup update link to view details
      updateLink.addEventListener('click', (e) => {
        e.preventDefault();
        
        // Make sure the update URL is absolute
        let finalUpdateUrl = updateInfo.updateUrl || '/release-notes.html';
        if (finalUpdateUrl.startsWith('/')) {
          // Convert relative URL to absolute (using buy.cookieaway.com as base)
          finalUpdateUrl = 'https://buy.cookieaway.com' + finalUpdateUrl;
        }
        
        // Open the release notes page in a new tab
        chrome.tabs.create({ url: finalUpdateUrl });
      });
      
      // Setup direct update button
      directUpdateButton.addEventListener('click', async () => {
        try {
          // Show updating status
          showStatus('Starting update process...');
          
          // Request the extension update
          const response = await new Promise(resolve => {
            chrome.runtime.sendMessage({ 
              action: 'performUpdate',
              version: updateInfo.newVersion
            }, (response) => {
              resolve(response || {});
            });
          });
          
          if (!response.success) {
            throw new Error(response.error || 'Failed to initiate update');
          }
          
          // Make sure we have a valid URL
          const updateUrl = updateInfo.updateUrl;
          if (!updateUrl) {
            throw new Error('Update URL is missing');
          }
          
          // Make sure the update URL is absolute
          let finalUpdateUrl = updateUrl;
          if (updateUrl.startsWith('/')) {
            // Convert relative URL to absolute (using buy.cookieaway.com as base)
            finalUpdateUrl = 'https://buy.cookieaway.com' + updateUrl;
          }
          
          console.log('Opening update URL:', finalUpdateUrl);
          
          // Download the new version - use the update URL provided by the server
          chrome.tabs.create({ 
            url: finalUpdateUrl
          });
          
          // Close the popup
          window.close();
        } catch (error) {
          console.error('Error during update:', error);
          showStatus('Update failed. Please try again.', true);
        }
      });
    }
  } catch (error) {
    console.error('Error checking for updates:', error);
  }
}
