/**
 * Cookie Response Manager - Statistics
 * 
 * This script handles the display and manipulation of extension usage statistics
 */

document.addEventListener('DOMContentLoaded', async function() {
  // Load the stats
  await loadStats();
  
  // Set up event listeners
  document.getElementById('reset-stats').addEventListener('click', resetStats);
  document.getElementById('download-stats').addEventListener('click', downloadStats);
});

// Load and display stats
async function loadStats() {
  try {
    // Get stats from extension storage
    const data = await chrome.runtime.sendMessage({ action: 'getStats' });
    
    if (!data || !data.stats) {
      showStatus('Failed to load statistics', true);
      return;
    }
    
    const stats = data.stats;
    
    // Update overview stats
    document.getElementById('total-attempts').textContent = stats.totalAttempts;
    document.getElementById('success-count').textContent = stats.successCount;
    document.getElementById('failure-count').textContent = stats.failureCount;
    
    // Calculate success rate
    let successRate = 0;
    if (stats.totalAttempts > 0) {
      successRate = Math.round((stats.successCount / stats.totalAttempts) * 100);
    }
    document.getElementById('success-rate').textContent = `${successRate}%`;
    
    // Update platform stats
    const platformsDiv = document.getElementById('platform-stats');
    platformsDiv.innerHTML = '';
    
    if (Object.keys(stats.byPlatform).length === 0) {
      platformsDiv.innerHTML = '<p class="loading">No platform data available yet</p>';
    } else {
      // Sort platforms by total attempts (success + failure) in descending order
      const sortedPlatforms = Object.entries(stats.byPlatform)
        .sort((a, b) => (b[1].success + b[1].failure) - (a[1].success + a[1].failure));
      
      // Create a platform item for each CMP
      for (const [platform, counts] of sortedPlatforms) {
        const total = counts.success + counts.failure;
        const successRate = total > 0 ? Math.round((counts.success / total) * 100) : 0;
        
        const platformItem = document.createElement('div');
        platformItem.className = 'platform-item';
        
        platformItem.innerHTML = `
          <div class="platform-name">${platform}</div>
          <div class="platform-success-rate">${successRate}%</div>
          <div class="platform-counts">${counts.success} of ${total}</div>
          <div class="progress-container">
            <div class="progress-bar" style="width: ${successRate}%"></div>
          </div>
        `;
        
        platformsDiv.appendChild(platformItem);
      }
    }
    
    // Update recent activity
    const activityTbody = document.getElementById('activity-tbody');
    activityTbody.innerHTML = '';
    
    if (stats.recentActivity.length === 0) {
      activityTbody.innerHTML = '<tr><td colspan="4" class="loading">No activity data available yet</td></tr>';
    } else {
      for (const activity of stats.recentActivity) {
        const tr = document.createElement('tr');
        
        // Format the date
        const date = new Date(activity.timestamp);
        const formattedDate = formatDateTime(date);
        
        // Create the result badge
        const resultBadge = `<span class="result-badge ${activity.success ? 'result-success' : 'result-failure'}">
          ${activity.success ? 'Success' : 'Failed'}
        </span>`;
        
        tr.innerHTML = `
          <td>${formattedDate}</td>
          <td>${activity.url}</td>
          <td>${activity.cmpType}</td>
          <td>${resultBadge}</td>
        `;
        
        activityTbody.appendChild(tr);
      }
    }
    
    // Update unhandled banners
    const unhandledTbody = document.getElementById('unhandled-tbody');
    unhandledTbody.innerHTML = '';
    
    if (stats.unhandledBanners.length === 0) {
      unhandledTbody.innerHTML = '<tr><td colspan="4" class="loading">No unhandled banner reports available</td></tr>';
    } else {
      for (const report of stats.unhandledBanners) {
        const tr = document.createElement('tr');
        
        // Format the date
        const date = new Date(report.timestamp);
        const formattedDate = formatDateTime(date);
        
        // Get hostname from URL
        let hostname = report.url;
        try {
          hostname = new URL(report.url).hostname;
        } catch (error) {
          // If URL parsing fails, just use the original
        }
        
        // Source label
        const sourceLabel = report.userSubmitted ? 'User Reported' : 'Auto-detected';
        
        tr.innerHTML = `
          <td>${formattedDate}</td>
          <td>${hostname}</td>
          <td>${sourceLabel}</td>
          <td>
            <button class="secondary-button view-details" data-url="${report.url}">View Details</button>
          </td>
        `;
        
        unhandledTbody.appendChild(tr);
      }
      
      // Add event listeners to view details buttons
      const viewButtons = document.querySelectorAll('.view-details');
      viewButtons.forEach(button => {
        button.addEventListener('click', () => {
          const url = button.getAttribute('data-url');
          // In a full implementation, this would show details about the unhandled banner
          alert(`Banner details for ${url}\n\nThis feature will be implemented in a future update to provide developers with necessary information to create new handlers.`);
        });
      });
    }
    
  } catch (error) {
    showStatus(`Error loading statistics: ${error.message}`, true);
    console.error('Error loading statistics:', error);
  }
}

// Reset all stats
async function resetStats() {
  if (confirm('Are you sure you want to reset all statistics? This cannot be undone.')) {
    try {
      const response = await chrome.runtime.sendMessage({ action: 'clearStats' });
      
      if (response && response.success) {
        showStatus('Statistics have been reset successfully');
        await loadStats(); // Reload the stats
      } else {
        showStatus('Failed to reset statistics', true);
      }
    } catch (error) {
      showStatus(`Error resetting statistics: ${error.message}`, true);
      console.error('Error resetting statistics:', error);
    }
  }
}

// Download stats as JSON file
async function downloadStats() {
  try {
    const data = await chrome.runtime.sendMessage({ action: 'getStats' });
    
    if (!data || !data.stats) {
      showStatus('Failed to download statistics', true);
      return;
    }
    
    const stats = data.stats;
    const statsJson = JSON.stringify(stats, null, 2);
    const blob = new Blob([statsJson], { type: 'application/json' });
    const url = URL.createObjectURL(blob);
    
    // Create a temporary link and trigger download
    const a = document.createElement('a');
    a.href = url;
    a.download = `cookie-response-manager-stats-${formatDate(new Date())}.json`;
    document.body.appendChild(a);
    a.click();
    
    // Clean up
    setTimeout(() => {
      document.body.removeChild(a);
      URL.revokeObjectURL(url);
    }, 100);
    
    showStatus('Statistics downloaded successfully');
  } catch (error) {
    showStatus(`Error downloading statistics: ${error.message}`, true);
    console.error('Error downloading statistics:', error);
  }
}

// Helper function to show status message
function showStatus(message, isError = false) {
  const statusElement = document.getElementById('status-message');
  statusElement.textContent = message;
  statusElement.className = isError ? 'error' : 'success';
  
  // Hide after 5 seconds
  setTimeout(() => {
    statusElement.textContent = '';
    statusElement.className = '';
  }, 5000);
}

// Helper function to format date time
function formatDateTime(date) {
  const options = { 
    month: 'short', 
    day: 'numeric', 
    hour: '2-digit', 
    minute: '2-digit'
  };
  return date.toLocaleDateString(undefined, options);
}

// Helper function to format date for filename
function formatDate(date) {
  const year = date.getFullYear();
  const month = String(date.getMonth() + 1).padStart(2, '0');
  const day = String(date.getDate()).padStart(2, '0');
  return `${year}-${month}-${day}`;
}